@@ -3,6 +3,9 @@
 extern "C" int
 LLVMFuzzerInitialize(int *argc, char ***argv)
 {
+	if (VIPS_INIT(*argv[0]))
+		return -1;
+
 	vips_concurrency_set(1);
 	return 0;
 }
@@ -42,9 +45,6 @@ LLVMFuzzerTestOneInput(const guint8 *data, size_t size)
 {
 	char *name;
 
-	if (size > 100 * 1024 * 1024)
-		return 0;
-
 	if (!(name = vips__temp_name("%s")))
 		return 0;
 
