@@ -3,6 +3,9 @@
 extern "C" int
 LLVMFuzzerInitialize(int *argc, char ***argv)
 {
+	if (VIPS_INIT(*argv[0]))
+		return -1;
+
 	vips_concurrency_set(1);
 	return 0;
 }
@@ -13,9 +16,6 @@ LLVMFuzzerTestOneInput(const guint8 *data, size_t size)
 	VipsImage *image, *out;
 	double d;
 
-	if (size > 100 * 1024 * 1024)
-		return 0;
-
 	if (!(image = vips_image_new_from_buffer(data, size, "", nullptr)))
 		return 0;
 
