@@ -211,14 +211,17 @@ typedef struct _VipsColourRoute {
 #define scRGB VIPS_INTERPRETATION_scRGB
 #define sRGB VIPS_INTERPRETATION_sRGB
 #define HSV VIPS_INTERPRETATION_HSV
+#define BW VIPS_INTERPRETATION_B_W
 #define RGB16 VIPS_INTERPRETATION_RGB16
 #define GREY16 VIPS_INTERPRETATION_GREY16
 #define YXY VIPS_INTERPRETATION_YXY
-#define BW VIPS_INTERPRETATION_B_W
+#define OKLAB VIPS_INTERPRETATION_OKLAB
+#define OKLCH VIPS_INTERPRETATION_OKLCH
 
 /* All the routes we know about.
  */
 static VipsColourRoute vips_colour_routes[] = {
+	{ XYZ, XYZ, { vips_cast_float, NULL } },
 	{ XYZ, LAB, { vips_XYZ2Lab, NULL } },
 	{ XYZ, LABQ, { vips_XYZ2Lab, vips_Lab2LabQ, NULL } },
 	{ XYZ, LCH, { vips_XYZ2Lab, vips_Lab2LCh, NULL } },
@@ -232,8 +235,11 @@ static VipsColourRoute vips_colour_routes[] = {
 	{ XYZ, RGB16, { vips_XYZ2scRGB, vips_scRGB2RGB16, NULL } },
 	{ XYZ, GREY16, { vips_XYZ2scRGB, vips_scRGB2BW16, NULL } },
 	{ XYZ, YXY, { vips_XYZ2Yxy, NULL } },
+	{ XYZ, OKLAB, { vips_XYZ2Oklab, NULL } },
+	{ XYZ, OKLCH, { vips_XYZ2Oklab, vips_Oklab2Oklch, NULL } },
 
 	{ LAB, XYZ, { vips_Lab2XYZ, NULL } },
+	{ LAB, LAB, { vips_cast_float, NULL } },
 	{ LAB, LABQ, { vips_Lab2LabQ, NULL } },
 	{ LAB, LCH, { vips_Lab2LCh, NULL } },
 	{ LAB, CMC, { vips_Lab2LCh, vips_LCh2CMC, NULL } },
@@ -246,9 +252,12 @@ static VipsColourRoute vips_colour_routes[] = {
 	{ LAB, RGB16, { vips_Lab2XYZ, vips_XYZ2scRGB, vips_scRGB2RGB16, NULL } },
 	{ LAB, GREY16, { vips_Lab2XYZ, vips_XYZ2scRGB, vips_scRGB2BW16, NULL } },
 	{ LAB, YXY, { vips_Lab2XYZ, vips_XYZ2Yxy, NULL } },
+	{ LAB, OKLAB, { vips_Lab2XYZ, vips_XYZ2Oklab, NULL } },
+	{ LAB, OKLCH, { vips_Lab2XYZ, vips_XYZ2Oklab, vips_Oklab2Oklch, NULL } },
 
 	{ LABQ, XYZ, { vips_LabQ2Lab, vips_Lab2XYZ, NULL } },
 	{ LABQ, LAB, { vips_LabQ2Lab, NULL } },
+	{ LABQ, LABQ, { NULL } },
 	{ LABQ, LCH, { vips_LabQ2Lab, vips_Lab2LCh, NULL } },
 	{ LABQ, CMC, { vips_LabQ2Lab, vips_Lab2LCh, vips_LCh2CMC, NULL } },
 	{ LABQ, LABS, { vips_LabQ2LabS, NULL } },
@@ -260,10 +269,13 @@ static VipsColourRoute vips_colour_routes[] = {
 	{ LABQ, RGB16, { vips_LabQ2Lab, vips_Lab2XYZ, vips_XYZ2scRGB, vips_scRGB2RGB16, NULL } },
 	{ LABQ, GREY16, { vips_LabQ2Lab, vips_Lab2XYZ, vips_XYZ2scRGB, vips_scRGB2BW16, NULL } },
 	{ LABQ, YXY, { vips_LabQ2Lab, vips_Lab2XYZ, vips_XYZ2Yxy, NULL } },
+	{ LABQ, OKLAB, { vips_LabQ2Lab, vips_Lab2XYZ, vips_XYZ2Oklab, NULL } },
+	{ LABQ, OKLCH, { vips_LabQ2Lab, vips_Lab2XYZ, vips_XYZ2Oklab, vips_Oklab2Oklch, NULL } },
 
 	{ LCH, XYZ, { vips_LCh2Lab, vips_Lab2XYZ, NULL } },
 	{ LCH, LAB, { vips_LCh2Lab, NULL } },
 	{ LCH, LABQ, { vips_LCh2Lab, vips_Lab2LabQ, NULL } },
+	{ LCH, LCH, { vips_cast_float, NULL } },
 	{ LCH, CMC, { vips_LCh2CMC, NULL } },
 	{ LCH, LABS, { vips_LCh2Lab, vips_Lab2LabS, NULL } },
 	{ LCH, CMYK, { vips_LCh2Lab, vips_Lab2XYZ, vips_XYZ2CMYK, NULL } },
@@ -274,11 +286,14 @@ static VipsColourRoute vips_colour_routes[] = {
 	{ LCH, RGB16, { vips_LCh2Lab, vips_Lab2XYZ, vips_XYZ2scRGB, vips_scRGB2RGB16, NULL } },
 	{ LCH, GREY16, { vips_LCh2Lab, vips_Lab2XYZ, vips_XYZ2scRGB, vips_scRGB2BW16, NULL } },
 	{ LCH, YXY, { vips_LCh2Lab, vips_Lab2XYZ, vips_XYZ2Yxy, NULL } },
+	{ LCH, OKLAB, { vips_LCh2Lab, vips_Lab2XYZ, vips_XYZ2Oklab, NULL } },
+	{ LCH, OKLCH, { vips_LCh2Lab, vips_Lab2XYZ, vips_XYZ2Oklab, vips_Oklab2Oklch, NULL } },
 
 	{ CMC, XYZ, { vips_CMC2LCh, vips_LCh2Lab, vips_Lab2XYZ, NULL } },
 	{ CMC, LAB, { vips_CMC2LCh, vips_LCh2Lab, NULL } },
 	{ CMC, LABQ, { vips_CMC2LCh, vips_LCh2Lab, vips_Lab2LabQ, NULL } },
 	{ CMC, LCH, { vips_CMC2LCh, NULL } },
+	{ CMC, CMC, { vips_cast_float, NULL } },
 	{ CMC, LABS, { vips_CMC2LCh, vips_LCh2Lab, vips_Lab2LabS, NULL } },
 	{ CMC, CMYK, { vips_CMC2LCh, vips_LCh2Lab, vips_Lab2XYZ, vips_XYZ2CMYK, NULL } },
 	{ CMC, scRGB, { vips_CMC2LCh, vips_LCh2Lab, vips_Lab2XYZ, vips_XYZ2scRGB, NULL } },
@@ -288,12 +303,15 @@ static VipsColourRoute vips_colour_routes[] = {
 	{ CMC, RGB16, { vips_CMC2LCh, vips_LCh2Lab, vips_Lab2XYZ, vips_XYZ2scRGB, vips_scRGB2RGB16, NULL } },
 	{ CMC, GREY16, { vips_CMC2LCh, vips_LCh2Lab, vips_Lab2XYZ, vips_XYZ2scRGB, vips_scRGB2BW16, NULL } },
 	{ CMC, YXY, { vips_CMC2LCh, vips_LCh2Lab, vips_Lab2XYZ, vips_XYZ2Yxy, NULL } },
+	{ CMC, OKLAB, { vips_CMC2LCh, vips_LCh2Lab, vips_Lab2XYZ, vips_XYZ2Oklab, NULL } },
+	{ CMC, OKLCH, { vips_CMC2LCh, vips_LCh2Lab, vips_Lab2XYZ, vips_XYZ2Oklab, vips_Oklab2Oklch, NULL } },
 
 	{ LABS, XYZ, { vips_LabS2Lab, vips_Lab2XYZ, NULL } },
 	{ LABS, LAB, { vips_LabS2Lab, NULL } },
 	{ LABS, LABQ, { vips_LabS2LabQ, NULL } },
 	{ LABS, LCH, { vips_LabS2Lab, vips_Lab2LCh, NULL } },
 	{ LABS, CMC, { vips_LabS2Lab, vips_Lab2LCh, vips_LCh2CMC, NULL } },
+	{ LABS, LABS, { vips_cast_short, NULL } },
 	{ LABS, CMYK, { vips_LabS2Lab, vips_Lab2XYZ, vips_XYZ2CMYK, NULL } },
 	{ LABS, scRGB, { vips_LabS2Lab, vips_Lab2XYZ, vips_XYZ2scRGB, NULL } },
 	{ LABS, sRGB, { vips_LabS2Lab, vips_Lab2XYZ, vips_XYZ2scRGB, vips_scRGB2sRGB, NULL } },
@@ -302,104 +320,128 @@ static VipsColourRoute vips_colour_routes[] = {
 	{ LABS, RGB16, { vips_LabS2Lab, vips_Lab2XYZ, vips_XYZ2scRGB, vips_scRGB2RGB16, NULL } },
 	{ LABS, GREY16, { vips_LabS2Lab, vips_Lab2XYZ, vips_XYZ2scRGB, vips_scRGB2BW16, NULL } },
 	{ LABS, YXY, { vips_LabS2Lab, vips_Lab2XYZ, vips_XYZ2Yxy, NULL } },
-
-	{ scRGB, XYZ, { vips_scRGB2XYZ, NULL } },
-	{ scRGB, LAB, { vips_scRGB2XYZ, vips_XYZ2Lab, NULL } },
-	{ scRGB, LABQ, { vips_scRGB2XYZ, vips_XYZ2Lab, vips_Lab2LabQ, NULL } },
-	{ scRGB, LCH, { vips_scRGB2XYZ, vips_XYZ2Lab, vips_Lab2LCh, NULL } },
-	{ scRGB, CMC, { vips_scRGB2XYZ, vips_XYZ2Lab, vips_Lab2LCh, vips_LCh2CMC, NULL } },
-	{ scRGB, CMYK, { vips_scRGB2XYZ, vips_XYZ2CMYK, NULL } },
-	{ scRGB, sRGB, { vips_scRGB2sRGB, NULL } },
-	{ scRGB, HSV, { vips_scRGB2sRGB, vips_sRGB2HSV, NULL } },
-	{ scRGB, BW, { vips_scRGB2BW, NULL } },
-	{ scRGB, LABS, { vips_scRGB2XYZ, vips_XYZ2Lab, vips_Lab2LabS, NULL } },
-	{ scRGB, RGB16, { vips_scRGB2RGB16, NULL } },
-	{ scRGB, GREY16, { vips_scRGB2BW16, NULL } },
-	{ scRGB, YXY, { vips_scRGB2XYZ, vips_XYZ2Yxy, NULL } },
+	{ LABS, OKLAB, { vips_LabS2Lab, vips_Lab2XYZ, vips_XYZ2Oklab, NULL } },
+	{ LABS, OKLCH, { vips_LabS2Lab, vips_Lab2XYZ, vips_XYZ2Oklab, vips_Oklab2Oklch, NULL } },
 
 	{ CMYK, XYZ, { vips_CMYK2XYZ, NULL } },
 	{ CMYK, LAB, { vips_CMYK2XYZ, vips_XYZ2Lab, NULL } },
 	{ CMYK, LABQ, { vips_CMYK2XYZ, vips_XYZ2Lab, vips_Lab2LabQ, NULL } },
 	{ CMYK, LCH, { vips_CMYK2XYZ, vips_XYZ2Lab, vips_Lab2LCh, NULL } },
 	{ CMYK, CMC, { vips_CMYK2XYZ, vips_XYZ2Lab, vips_Lab2LCh, vips_LCh2CMC, NULL } },
+	{ CMYK, LABS, { vips_CMYK2XYZ, vips_XYZ2Lab, vips_Lab2LabS, NULL } },
+	// can be 8 or 16 bit, so do nothing
+	{ CMYK, CMYK, { NULL } },
 	{ CMYK, scRGB, { vips_CMYK2XYZ, vips_XYZ2scRGB, NULL } },
 	{ CMYK, sRGB, { vips_CMYK2XYZ, vips_XYZ2scRGB, vips_scRGB2sRGB, NULL } },
 	{ CMYK, HSV, { vips_CMYK2XYZ, vips_XYZ2scRGB, vips_scRGB2sRGB, vips_sRGB2HSV, NULL } },
 	{ CMYK, BW, { vips_CMYK2XYZ, vips_XYZ2scRGB, vips_scRGB2BW, NULL } },
-	{ CMYK, LABS, { vips_CMYK2XYZ, vips_XYZ2Lab, vips_Lab2LabS, NULL } },
 	{ CMYK, RGB16, { vips_CMYK2XYZ, vips_XYZ2scRGB, vips_scRGB2RGB16, NULL } },
 	{ CMYK, GREY16, { vips_CMYK2XYZ, vips_XYZ2scRGB, vips_scRGB2BW16, NULL } },
 	{ CMYK, YXY, { vips_CMYK2XYZ, vips_XYZ2Yxy, NULL } },
+	{ CMYK, OKLAB, { vips_CMYK2XYZ, vips_XYZ2Oklab, NULL } },
+	{ CMYK, OKLCH, { vips_CMYK2XYZ, vips_XYZ2Oklab, vips_Oklab2Oklch, NULL } },
+
+	{ scRGB, XYZ, { vips_scRGB2XYZ, NULL } },
+	{ scRGB, LAB, { vips_scRGB2XYZ, vips_XYZ2Lab, NULL } },
+	{ scRGB, LABQ, { vips_scRGB2XYZ, vips_XYZ2Lab, vips_Lab2LabQ, NULL } },
+	{ scRGB, LCH, { vips_scRGB2XYZ, vips_XYZ2Lab, vips_Lab2LCh, NULL } },
+	{ scRGB, CMC, { vips_scRGB2XYZ, vips_XYZ2Lab, vips_Lab2LCh, vips_LCh2CMC, NULL } },
+	{ scRGB, LABS, { vips_scRGB2XYZ, vips_XYZ2Lab, vips_Lab2LabS, NULL } },
+	{ scRGB, CMYK, { vips_scRGB2XYZ, vips_XYZ2CMYK, NULL } },
+	{ scRGB, scRGB, { vips_cast_float, NULL } },
+	{ scRGB, sRGB, { vips_scRGB2sRGB, NULL } },
+	{ scRGB, HSV, { vips_scRGB2sRGB, vips_sRGB2HSV, NULL } },
+	{ scRGB, BW, { vips_scRGB2BW, NULL } },
+	{ scRGB, RGB16, { vips_scRGB2RGB16, NULL } },
+	{ scRGB, GREY16, { vips_scRGB2BW16, NULL } },
+	{ scRGB, YXY, { vips_scRGB2XYZ, vips_XYZ2Yxy, NULL } },
+	{ scRGB, OKLAB, { vips_scRGB2XYZ, vips_XYZ2Oklab, NULL } },
+	{ scRGB, OKLCH, { vips_scRGB2XYZ, vips_XYZ2Oklab, vips_Oklab2Oklch, NULL } },
 
 	{ sRGB, XYZ, { vips_sRGB2scRGB, vips_scRGB2XYZ, NULL } },
 	{ sRGB, LAB, { vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, NULL } },
 	{ sRGB, LABQ, { vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, vips_Lab2LabQ, NULL } },
 	{ sRGB, LCH, { vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, vips_Lab2LCh, NULL } },
 	{ sRGB, CMC, { vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, vips_Lab2LCh, vips_LCh2CMC, NULL } },
+	{ sRGB, LABS, { vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, vips_Lab2LabS, NULL } },
 	{ sRGB, CMYK, { vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2CMYK, NULL } },
 	{ sRGB, scRGB, { vips_sRGB2scRGB, NULL } },
+	{ sRGB, sRGB, { vips_cast_uchar, NULL } },
 	{ sRGB, HSV, { vips_sRGB2HSV, NULL } },
 	{ sRGB, BW, { vips_sRGB2scRGB, vips_scRGB2BW, NULL } },
-	{ sRGB, LABS, { vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, vips_Lab2LabS, NULL } },
 	{ sRGB, RGB16, { vips_sRGB2RGB16, NULL } },
 	{ sRGB, GREY16, { vips_sRGB2scRGB, vips_scRGB2BW16, NULL } },
 	{ sRGB, YXY, { vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Yxy, NULL } },
+	{ sRGB, OKLAB, { vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Oklab, NULL } },
+	{ sRGB, OKLCH, { vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Oklab, vips_Oklab2Oklch, NULL } },
 
 	{ HSV, XYZ, { vips_HSV2sRGB, vips_sRGB2scRGB, vips_scRGB2XYZ, NULL } },
 	{ HSV, LAB, { vips_HSV2sRGB, vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, NULL } },
 	{ HSV, LABQ, { vips_HSV2sRGB, vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, vips_Lab2LabQ, NULL } },
 	{ HSV, LCH, { vips_HSV2sRGB, vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, vips_Lab2LCh, NULL } },
 	{ HSV, CMC, { vips_HSV2sRGB, vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, vips_Lab2LCh, vips_LCh2CMC, NULL } },
+	{ HSV, LABS, { vips_HSV2sRGB, vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, vips_Lab2LabS, NULL } },
 	{ HSV, CMYK, { vips_HSV2sRGB, vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2CMYK, NULL } },
 	{ HSV, scRGB, { vips_HSV2sRGB, vips_sRGB2scRGB, NULL } },
 	{ HSV, sRGB, { vips_HSV2sRGB, NULL } },
+	{ HSV, HSV, { vips_cast_uchar, NULL } },
 	{ HSV, BW, { vips_HSV2sRGB, vips_sRGB2scRGB, vips_scRGB2BW, NULL } },
-	{ HSV, LABS, { vips_HSV2sRGB, vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, vips_Lab2LabS, NULL } },
 	{ HSV, RGB16, { vips_HSV2sRGB, vips_sRGB2RGB16, NULL } },
 	{ HSV, GREY16, { vips_HSV2sRGB, vips_sRGB2scRGB, vips_scRGB2BW16, NULL } },
 	{ HSV, YXY, { vips_HSV2sRGB, vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Yxy, NULL } },
-
-	{ RGB16, XYZ, { vips_sRGB2scRGB, vips_scRGB2XYZ, NULL } },
-	{ RGB16, LAB, { vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, NULL } },
-	{ RGB16, LABQ, { vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, vips_Lab2LabQ, NULL } },
-	{ RGB16, LCH, { vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, vips_Lab2LCh, NULL } },
-	{ RGB16, CMC, { vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, vips_Lab2LCh, vips_LCh2CMC, NULL } },
-	{ RGB16, CMYK, { vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2CMYK, NULL } },
-	{ RGB16, scRGB, { vips_sRGB2scRGB, NULL } },
-	{ RGB16, sRGB, { vips_RGB162sRGB, NULL } },
-	{ RGB16, HSV, { vips_RGB162sRGB, vips_sRGB2HSV, NULL } },
-	{ RGB16, BW, { vips_sRGB2scRGB, vips_scRGB2BW, NULL } },
-	{ RGB16, LABS, { vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, vips_Lab2LabS, NULL } },
-	{ RGB16, GREY16, { vips_sRGB2scRGB, vips_scRGB2BW16, NULL } },
-	{ RGB16, YXY, { vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Yxy, NULL } },
-
-	{ GREY16, XYZ, { vips_GREY162RGB16, vips_sRGB2scRGB, vips_scRGB2XYZ, NULL } },
-	{ GREY16, LAB, { vips_GREY162RGB16, vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, NULL } },
-	{ GREY16, LABQ, { vips_GREY162RGB16, vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, vips_Lab2LabQ, NULL } },
-	{ GREY16, LCH, { vips_GREY162RGB16, vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, vips_Lab2LCh, NULL } },
-	{ GREY16, CMC, { vips_GREY162RGB16, vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, vips_Lab2LCh, vips_LCh2CMC, NULL } },
-	{ GREY16, CMYK, { vips_GREY162RGB16, vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2CMYK, NULL } },
-	{ GREY16, scRGB, { vips_GREY162RGB16, vips_sRGB2scRGB, NULL } },
-	{ GREY16, sRGB, { vips_GREY162RGB16, vips_RGB162sRGB, NULL } },
-	{ GREY16, HSV, { vips_GREY162RGB16, vips_RGB162sRGB, vips_sRGB2HSV, NULL } },
-	{ GREY16, BW, { vips_GREY162RGB16, vips_sRGB2scRGB, vips_scRGB2BW, NULL } },
-	{ GREY16, LABS, { vips_GREY162RGB16, vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, vips_Lab2LabS, NULL } },
-	{ GREY16, RGB16, { vips_GREY162RGB16, NULL } },
-	{ GREY16, YXY, { vips_GREY162RGB16, vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Yxy, NULL } },
+	{ HSV, OKLAB, { vips_HSV2sRGB, vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Oklab, NULL } },
+	{ HSV, OKLCH, { vips_HSV2sRGB, vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Oklab, vips_Oklab2Oklch, NULL } },
 
 	{ BW, XYZ, { vips_BW2sRGB, vips_sRGB2scRGB, vips_scRGB2XYZ, NULL } },
 	{ BW, LAB, { vips_BW2sRGB, vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, NULL } },
 	{ BW, LABQ, { vips_BW2sRGB, vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, vips_Lab2LabQ, NULL } },
 	{ BW, LCH, { vips_BW2sRGB, vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, vips_Lab2LCh, NULL } },
 	{ BW, CMC, { vips_BW2sRGB, vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, vips_Lab2LCh, vips_LCh2CMC, NULL } },
+	{ BW, LABS, { vips_BW2sRGB, vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, vips_Lab2LabS, NULL } },
 	{ BW, CMYK, { vips_BW2sRGB, vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2CMYK, NULL } },
 	{ BW, scRGB, { vips_BW2sRGB, vips_sRGB2scRGB, NULL } },
 	{ BW, sRGB, { vips_BW2sRGB, NULL } },
 	{ BW, HSV, { vips_BW2sRGB, vips_sRGB2HSV, NULL } },
-	{ BW, LABS, { vips_BW2sRGB, vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, vips_Lab2LabS, NULL } },
+	{ BW, BW, { vips_cast_uchar, NULL } },
 	{ BW, RGB16, { vips_BW2sRGB, vips_sRGB2RGB16, NULL } },
 	{ BW, GREY16, { vips_BW2sRGB, vips_sRGB2scRGB, vips_scRGB2BW16, NULL } },
 	{ BW, YXY, { vips_BW2sRGB, vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Yxy, NULL } },
+	{ BW, OKLAB, { vips_BW2sRGB, vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Oklab, NULL } },
+	{ BW, OKLCH, { vips_BW2sRGB, vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Oklab, vips_Oklab2Oklch, NULL } },
+
+	{ RGB16, XYZ, { vips_sRGB2scRGB, vips_scRGB2XYZ, NULL } },
+	{ RGB16, LAB, { vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, NULL } },
+	{ RGB16, LABQ, { vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, vips_Lab2LabQ, NULL } },
+	{ RGB16, LCH, { vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, vips_Lab2LCh, NULL } },
+	{ RGB16, CMC, { vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, vips_Lab2LCh, vips_LCh2CMC, NULL } },
+	{ RGB16, LABS, { vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, vips_Lab2LabS, NULL } },
+	{ RGB16, CMYK, { vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2CMYK, NULL } },
+	{ RGB16, scRGB, { vips_sRGB2scRGB, NULL } },
+	{ RGB16, sRGB, { vips_RGB162sRGB, NULL } },
+	{ RGB16, HSV, { vips_RGB162sRGB, vips_sRGB2HSV, NULL } },
+	{ RGB16, BW, { vips_sRGB2scRGB, vips_scRGB2BW, NULL } },
+	{ RGB16, RGB16, { vips_cast_ushort, NULL } },
+	{ RGB16, GREY16, { vips_sRGB2scRGB, vips_scRGB2BW16, NULL } },
+	{ RGB16, YXY, { vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Yxy, NULL } },
+	{ RGB16, OKLAB, { vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Oklab, NULL } },
+	{ RGB16, OKLCH, { vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Oklab, vips_Oklab2Oklch, NULL } },
+
+	{ GREY16, XYZ, { vips_GREY162RGB16, vips_sRGB2scRGB, vips_scRGB2XYZ, NULL } },
+	{ GREY16, LAB, { vips_GREY162RGB16, vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, NULL } },
+	{ GREY16, LABQ, { vips_GREY162RGB16, vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, vips_Lab2LabQ, NULL } },
+	{ GREY16, LCH, { vips_GREY162RGB16, vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, vips_Lab2LCh, NULL } },
+	{ GREY16, CMC, { vips_GREY162RGB16, vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, vips_Lab2LCh, vips_LCh2CMC, NULL } },
+	{ GREY16, LABS, { vips_GREY162RGB16, vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, vips_Lab2LabS, NULL } },
+	{ GREY16, CMYK, { vips_GREY162RGB16, vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2CMYK, NULL } },
+	{ GREY16, scRGB, { vips_GREY162RGB16, vips_sRGB2scRGB, NULL } },
+	{ GREY16, sRGB, { vips_GREY162RGB16, vips_RGB162sRGB, NULL } },
+	{ GREY16, HSV, { vips_GREY162RGB16, vips_RGB162sRGB, vips_sRGB2HSV, NULL } },
+	{ GREY16, BW, { vips_GREY162RGB16, vips_sRGB2scRGB, vips_scRGB2BW, NULL } },
+	{ GREY16, RGB16, { vips_GREY162RGB16, NULL } },
+	{ GREY16, GREY16, { vips_cast_ushort, NULL } },
+	{ GREY16, YXY, { vips_GREY162RGB16, vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Yxy, NULL } },
+	{ GREY16, OKLAB, { vips_GREY162RGB16, vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Oklab, NULL } },
+	{ GREY16, OKLCH, { vips_GREY162RGB16, vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Oklab, vips_Oklab2Oklch, NULL } },
 
 	{ YXY, XYZ, { vips_Yxy2XYZ, NULL } },
 	{ YXY, LAB, { vips_Yxy2XYZ, vips_XYZ2Lab, NULL } },
@@ -413,7 +455,44 @@ static VipsColourRoute vips_colour_routes[] = {
 	{ YXY, HSV, { vips_Yxy2XYZ, vips_XYZ2scRGB, vips_scRGB2sRGB, vips_sRGB2HSV, NULL } },
 	{ YXY, BW, { vips_Yxy2XYZ, vips_XYZ2scRGB, vips_scRGB2BW, NULL } },
 	{ YXY, RGB16, { vips_Yxy2XYZ, vips_XYZ2scRGB, vips_scRGB2RGB16, NULL } },
-	{ YXY, GREY16, { vips_Yxy2XYZ, vips_XYZ2scRGB, vips_scRGB2BW16, NULL } }
+	{ YXY, GREY16, { vips_Yxy2XYZ, vips_XYZ2scRGB, vips_scRGB2BW16, NULL } },
+	{ YXY, YXY, { vips_cast_float, NULL } },
+	{ YXY, OKLAB, { vips_Yxy2XYZ, vips_XYZ2Oklab, NULL } },
+	{ YXY, OKLCH, { vips_Yxy2XYZ, vips_XYZ2Oklab, vips_Oklab2Oklch, NULL } },
+
+	{ OKLAB, XYZ, { vips_Oklab2XYZ, NULL } },
+	{ OKLAB, LAB, { vips_Oklab2XYZ, vips_XYZ2Lab, NULL } },
+	{ OKLAB, LABQ, { vips_Oklab2XYZ, vips_XYZ2Lab, vips_Lab2LabQ, NULL } },
+	{ OKLAB, LCH, { vips_Oklab2XYZ, vips_XYZ2Lab, vips_Lab2LCh, NULL } },
+	{ OKLAB, CMC, { vips_Oklab2XYZ, vips_XYZ2Lab, vips_Lab2LCh, vips_LCh2CMC, NULL } },
+	{ OKLAB, LABS, { vips_Oklab2XYZ, vips_XYZ2Lab, vips_Lab2LabS, NULL } },
+	{ OKLAB, CMYK, { vips_Oklab2XYZ, vips_XYZ2CMYK, NULL } },
+	{ OKLAB, scRGB, { vips_Oklab2XYZ, vips_XYZ2scRGB, NULL } },
+	{ OKLAB, sRGB, { vips_Oklab2XYZ, vips_XYZ2scRGB, vips_scRGB2sRGB, NULL } },
+	{ OKLAB, HSV, { vips_Oklab2XYZ, vips_XYZ2scRGB, vips_scRGB2sRGB, vips_sRGB2HSV, NULL } },
+	{ OKLAB, BW, { vips_Oklab2XYZ, vips_XYZ2scRGB, vips_scRGB2BW, NULL } },
+	{ OKLAB, RGB16, { vips_Oklab2XYZ, vips_XYZ2scRGB, vips_scRGB2RGB16, NULL } },
+	{ OKLAB, GREY16, { vips_Oklab2XYZ, vips_XYZ2scRGB, vips_scRGB2BW16, NULL } },
+	{ OKLAB, YXY, { vips_Oklab2XYZ, vips_XYZ2Yxy, NULL } },
+	{ OKLAB, OKLAB, { vips_cast_float, NULL } },
+	{ OKLAB, OKLCH, { vips_Oklab2Oklch, NULL } },
+
+	{ OKLCH, XYZ, { vips_Oklch2Oklab, vips_Oklab2XYZ, NULL } },
+	{ OKLCH, LAB, { vips_Oklch2Oklab, vips_Oklab2XYZ, vips_XYZ2Lab, NULL } },
+	{ OKLCH, LABQ, { vips_Oklch2Oklab, vips_Oklab2XYZ, vips_XYZ2Lab, vips_Lab2LabQ, NULL } },
+	{ OKLCH, LCH, { vips_Oklch2Oklab, vips_Oklab2XYZ, vips_XYZ2Lab, vips_Lab2LCh, NULL } },
+	{ OKLCH, CMC, { vips_Oklch2Oklab, vips_Oklab2XYZ, vips_XYZ2Lab, vips_Lab2LCh, vips_LCh2CMC, NULL } },
+	{ OKLCH, LABS, { vips_Oklch2Oklab, vips_Oklab2XYZ, vips_XYZ2Lab, vips_Lab2LabS, NULL } },
+	{ OKLCH, CMYK, { vips_Oklch2Oklab, vips_Oklab2XYZ, vips_XYZ2CMYK, NULL } },
+	{ OKLCH, scRGB, { vips_Oklch2Oklab, vips_Oklab2XYZ, vips_XYZ2scRGB, NULL } },
+	{ OKLCH, sRGB, { vips_Oklch2Oklab, vips_Oklab2XYZ, vips_XYZ2scRGB, vips_scRGB2sRGB, NULL } },
+	{ OKLCH, HSV, { vips_Oklch2Oklab, vips_Oklab2XYZ, vips_XYZ2scRGB, vips_scRGB2sRGB, vips_sRGB2HSV, NULL } },
+	{ OKLCH, BW, { vips_Oklch2Oklab, vips_Oklab2XYZ, vips_XYZ2scRGB, vips_scRGB2BW, NULL } },
+	{ OKLCH, RGB16, { vips_Oklch2Oklab, vips_Oklab2XYZ, vips_XYZ2scRGB, vips_scRGB2RGB16, NULL } },
+	{ OKLCH, GREY16, { vips_Oklch2Oklab, vips_Oklab2XYZ, vips_XYZ2scRGB, vips_scRGB2BW16, NULL } },
+	{ OKLCH, YXY, { vips_Oklch2Oklab, vips_Oklab2XYZ, vips_XYZ2Yxy, NULL } },
+	{ OKLCH, OKLAB, { vips_Oklch2Oklab, NULL } },
+	{ OKLCH, OKLCH, { vips_cast_float, NULL } },
 
 };
 
@@ -424,9 +503,9 @@ static VipsColourRoute vips_colour_routes[] = {
  * vips_colourspace_issupported: (method)
  * @image: input image
  *
- * Test if @image is in a colourspace that vips_colourspace() can process.
+ * Test if @image is in a colourspace that [method@Image.colourspace] can process.
  *
- * Returns: %TRUE if @image is in a supported colourspace.
+ * Returns: `TRUE` if @image is in a supported colourspace.
  */
 gboolean
 vips_colourspace_issupported(const VipsImage *image)
@@ -468,8 +547,7 @@ vips_colourspace_build(VipsObject *object)
 
 	int i, j;
 	VipsImage *x;
-	VipsImage **t = (VipsImage **)
-		vips_object_local_array(object, 1);
+	VipsImage **t = (VipsImage **) vips_object_local_array(object, 1);
 	VipsImage **pipe = (VipsImage **)
 		vips_object_local_array(object, MAX_STEPS);
 
@@ -502,25 +580,14 @@ vips_colourspace_build(VipsObject *object)
 	if (interpretation == VIPS_INTERPRETATION_RGB)
 		interpretation = VIPS_INTERPRETATION_sRGB;
 
-	/* No conversion necessary.
-	 */
-	if (interpretation == colourspace->space) {
-		g_object_set(colourspace, "out", vips_image_new(), NULL);
-
-		return vips_image_write(colourspace->in, colourspace->out);
-	}
-
 	for (i = 0; i < VIPS_NUMBER(vips_colour_routes); i++)
 		if (vips_colour_routes[i].from == interpretation &&
 			vips_colour_routes[i].to == colourspace->space)
 			break;
 	if (i == VIPS_NUMBER(vips_colour_routes)) {
-		vips_error("vips_colourspace",
-			_("no known route from '%s' to '%s'"),
-			vips_enum_nick(VIPS_TYPE_INTERPRETATION,
-				interpretation),
-			vips_enum_nick(VIPS_TYPE_INTERPRETATION,
-				colourspace->space));
+		vips_error("vips_colourspace", _("no known route from '%s' to '%s'"),
+			vips_enum_nick(VIPS_TYPE_INTERPRETATION, interpretation),
+			vips_enum_nick(VIPS_TYPE_INTERPRETATION, colourspace->space));
 		return -1;
 	}
 
@@ -591,22 +658,23 @@ vips_colourspace_init(VipsColourspace *colourspace)
  * @in: input image
  * @out: (out): output image
  * @space: convert to this colour space
- * @...: %NULL-terminated list of optional named arguments
- *
- * Optional arguments:
- *
- * * @source_space: input colour space
+ * @...: `NULL`-terminated list of optional named arguments
  *
  * This operation looks at the interpretation field of @in (or uses
  * @source_space, if set) and runs
  * a set of colourspace conversion functions to move it to @space.
  *
- * For example, given an image tagged as #VIPS_INTERPRETATION_YXY, running
- * vips_colourspace() with @space set to #VIPS_INTERPRETATION_LAB will
- * convert with vips_Yxy2XYZ() and vips_XYZ2Lab().
+ * For example, given an image tagged as [enum@Vips.Interpretation.YXY], running
+ * [method@Image.colourspace] with @space set to
+ * [enum@Vips.Interpretation.LAB] will convert with [method@Image.Yxy2XYZ]
+ * and [method@Image.XYZ2Lab].
  *
- * See also: vips_colourspace_issupported(),
- * vips_image_guess_interpretation().
+ * ::: tip "Optional arguments"
+ *     * @source_space: [enum@Interpretation], input colour space
+ *
+ * ::: seealso
+ *     [method@Image.colourspace_issupported],
+ *     [method@Image.guess_interpretation].
  *
  * Returns: 0 on success, -1 on error.
  */
