@@ -140,23 +140,27 @@ vips_extract_area_build(VipsObject *object)
 	VipsConversion *conversion = VIPS_CONVERSION(object);
 	VipsExtractArea *extract = (VipsExtractArea *) object;
 
+	VipsImage *in;
+
 	if (VIPS_OBJECT_CLASS(vips_extract_area_parent_class)->build(object))
 		return -1;
 
-	if (extract->left + extract->width > extract->in->Xsize ||
-		extract->top + extract->height > extract->in->Ysize ||
+	in = extract->in;
+
+	if (extract->left + extract->width > in->Xsize ||
+		extract->top + extract->height > in->Ysize ||
 		extract->left < 0 || extract->top < 0 ||
 		extract->width <= 0 || extract->height <= 0) {
 		vips_error(class->nickname, "%s", _("bad extract area"));
 		return -1;
 	}
 
-	if (vips_image_pio_input(extract->in) ||
-		vips_check_coding_known(class->nickname, extract->in))
+	if (vips_image_pio_input(in) ||
+		vips_check_coding_known(class->nickname, in))
 		return -1;
 
 	if (vips_image_pipelinev(conversion->out,
-			VIPS_DEMAND_STYLE_THINSTRIP, extract->in, NULL))
+			VIPS_DEMAND_STYLE_THINSTRIP, in, NULL))
 		return -1;
 
 	conversion->out->Xsize = extract->width;
@@ -166,7 +170,7 @@ vips_extract_area_build(VipsObject *object)
 
 	if (vips_image_generate(conversion->out,
 			vips_start_one, vips_extract_area_gen, vips_stop_one,
-			extract->in, extract))
+			in, extract))
 		return -1;
 
 	return 0;
