@@ -254,7 +254,7 @@ vips_msb_class_init(VipsMsbClass *class)
 		_("Band to msb"),
 		VIPS_ARGUMENT_OPTIONAL_INPUT,
 		G_STRUCT_OFFSET(VipsMsb, band),
-		0, 100000000, 0);
+		-1, 100000000, -1);
 }
 
 static void
@@ -267,11 +267,7 @@ vips_msb_init(VipsMsb *msb)
  * vips_msb: (method)
  * @in: input image
  * @out: (out): output image
- * @...: %NULL-terminated list of optional named arguments
- *
- * Optional arguments:
- *
- * * @band: %gint, msb just this band
+ * @...: `NULL`-terminated list of optional named arguments
  *
  * Turn any integer image to 8-bit unsigned char by discarding all but the most
  * significant byte. Signed values are converted to unsigned by adding 128.
@@ -280,7 +276,11 @@ vips_msb_init(VipsMsb *msb)
  *
  * This operator also works for LABQ coding.
  *
- * See also: vips_scale(), vips_cast().
+ * ::: tip "Optional arguments"
+ *     * @band: `gint`, msb just this band
+ *
+ * ::: seealso
+ *     [method@Image.scale], [method@Image.cast].
  *
  * Returns: 0 on success, -1 on error.
  */
