@@ -331,10 +331,10 @@ vips_conva_decompose_hlines(VipsConva *conva)
 	 * fixed n-lines which includes any negative parts.
 	 */
 	depth = (max - min) / conva->layers;
-	layers_above = VIPS_CEIL(max / depth);
+	layers_above = ceil(max / depth);
 	depth = max / layers_above;
-	layers_below = VIPS_FLOOR(min / depth);
-	conva->layers = layers_above - layers_below;
+	layers_below = floor(min / depth);
+	conva->layers = layers_above - layers_below; // FIXME: Invalidates operation cache
 
 	VIPS_DEBUG_MSG("vips_conva_decompose_hlines: depth = %g, layers = %d\n",
 		depth, conva->layers);
@@ -744,7 +744,7 @@ vips_conva_decompose_boxes(VipsConva *conva)
 	for (z = 0; z < size; z++)
 		sum += fabs(coeff[z]);
 
-	conva->divisor = VIPS_RINT(area * scale / sum);
+	conva->divisor = rint(area * scale / sum);
 	conva->rounding = (conva->divisor + 1) / 2;
 	conva->offset = offset;
 
@@ -983,7 +983,7 @@ vips_conva_horizontal(VipsConva *conva, VipsImage *in, VipsImage **out)
 {
 	VipsObjectClass *class = VIPS_OBJECT_GET_CLASS(conva);
 
-	/* Prepare output. Consider a 7x7 mask and a 7x7 image --- the output
+	/* Prepare output. Consider a 7x7 mask and a 7x7 image -- the output
 	 * would be 1x1.
 	 */
 	*out = vips_image_new();
@@ -1233,7 +1233,7 @@ vips_conva_vertical(VipsConva *conva, VipsImage *in, VipsImage **out)
 	VipsObjectClass *class = VIPS_OBJECT_GET_CLASS(conva);
 	VipsConvolution *convolution = (VipsConvolution *) conva;
 
-	/* Prepare output. Consider a 7x7 mask and a 7x7 image --- the output
+	/* Prepare output. Consider a 7x7 mask and a 7x7 image -- the output
 	 * would be 1x1.
 	 */
 	*out = vips_image_new();
@@ -1346,19 +1346,14 @@ vips_conva_init(VipsConva *conva)
  * @in: input image
  * @out: (out): output image
  * @mask: convolution mask
- * @...: %NULL-terminated list of optional named arguments
- *
- * Optional arguments:
- *
- * * @layers: %gint, number of layers for approximation
- * * @cluster: %gint, cluster lines closer than this distance
+ * @...: `NULL`-terminated list of optional named arguments
  *
  * Perform an approximate integer convolution of @in with @mask.
- * This is a low-level operation, see
- * vips_conv() for something more convenient.
+ * This is a low-level operation, see [method@Image.conv] for something more
+ * convenient.
  *
  * The output image
- * always has the same #VipsBandFormat as the input image.
+ * always has the same [enum@BandFormat] as the input image.
  * Elements of @mask are converted to
  * integers before convolution.
  *
@@ -1371,7 +1366,12 @@ vips_conva_init(VipsConva *conva)
  * Smaller values of @cluster will give more accurate results, but be slower
  * and use more memory. 10% of the mask radius is a good rule of thumb.
  *
- * See also: vips_conv().
+ * ::: tip "Optional arguments"
+ *     * @layers: `gint`, number of layers for approximation
+ *     * @cluster: `gint`, cluster lines closer than this distance
+ *
+ * ::: seealso
+ *     [method@Image.conv].
  *
  * Returns: 0 on success, -1 on error
  */
