@@ -89,8 +89,8 @@ vips_grey_class_init(VipsGreyClass *class)
 	vobject_class->description = _("make a grey ramp image");
 
 	point_class->point = vips_grey_point;
-	point_class->min = 0.0;
-	point_class->max = 1.0;
+	point_class->min = 0.0F;
+	point_class->max = 1.0F;
 }
 
 static void
@@ -103,19 +103,21 @@ vips_grey_init(VipsGrey *grey)
  * @out: (out): output image
  * @width: image size
  * @height: image size
- * @...: %NULL-terminated list of optional named arguments
- *
- * Optional arguments:
- *
- * * @uchar: output a uchar image
+ * @...: `NULL`-terminated list of optional named arguments
  *
  * Create a one-band float image with the left-most column zero and the
- * right-most 1. Intermediate pixels are a linear ramp.
+ * right-most 1.
+ *
+ * Intermediate pixels are a linear ramp.
  *
  * Set @uchar to output a uchar image with the leftmost pixel 0 and the
  * rightmost 255.
  *
- * See also: vips_xyz(), vips_identity().
+ * ::: tip "Optional arguments"
+ *     * @uchar: `gboolean`, output a uchar image
+ *
+ * ::: seealso
+ *     [ctor@Image.xyz], [ctor@Image.identity].
  *
  * Returns: 0 on success, -1 on error
  */
