@@ -167,45 +167,6 @@
 
 #include "jpeg.h"
 
-/* Stuff we track during a read.
- */
-typedef struct _ReadJpeg {
-	VipsImage *out;
-
-	/* Shrink by this much during load. 1, 2, 4, 8.
-	 */
-	int shrink;
-
-	/* Types of error to cause failure.
-	 */
-	VipsFailOn fail_on;
-
-	struct jpeg_decompress_struct cinfo;
-	ErrorManager eman;
-	gboolean invert_pels;
-
-	/* Use orientation tag to automatically rotate and flip image
-	 * during load.
-	 */
-	gboolean autorotate;
-
-	/* Remove DoS limits.
-	 */
-	gboolean unlimited;
-
-	/* cinfo->output_width and height can be larger than we want since
-	 * libjpeg rounds up on shrink-on-load. This is the real size we will
-	 * output, as opposed to the size we decompress to.
-	 */
-	int output_width;
-	int output_height;
-
-	/* The source we read from.
-	 */
-	VipsSource *source;
-
-} ReadJpeg;
-
 #define SOURCE_BUFFER_SIZE (4096)
 
 /* Private struct for source input.
@@ -230,7 +191,7 @@ source_init_source(j_decompress_ptr cinfo)
 	 */
 }
 
-/* Fill the input buffer --- called whenever buffer is emptied.
+/* Fill the input buffer -- called whenever buffer is emptied.
  */
 static boolean
 source_fill_input_buffer(j_decompress_ptr cinfo)
@@ -239,8 +200,8 @@ source_fill_input_buffer(j_decompress_ptr cinfo)
 
 	gint64 n_bytes;
 
-	if ((n_bytes = vips_source_read(src->source,
-			 src->buf, SOURCE_BUFFER_SIZE)) <= 0) {
+	if ((n_bytes =
+		vips_source_read(src->source, src->buf, SOURCE_BUFFER_SIZE)) <= 0) {
 		if (src->jpeg->fail_on >= VIPS_FAIL_ON_TRUNCATED) {
 			/* Knock the output out of cache.
 			 */
@@ -291,40 +252,40 @@ source_fill_input_buffer_mappable(j_decompress_ptr cinfo)
 static void
 skip_input_data(j_decompress_ptr cinfo, long num_bytes)
 {
-	Source *src = (Source *) cinfo->src;
+	struct jpeg_source_mgr *src = cinfo->src;
 
 	if (num_bytes > 0) {
-		while (num_bytes > (long) src->pub.bytes_in_buffer) {
-			num_bytes -= (long) src->pub.bytes_in_buffer;
-			(void) (*src->pub.fill_input_buffer)(cinfo);
+		while (num_bytes > (long) src->bytes_in_buffer) {
+			num_bytes -= (long) src->bytes_in_buffer;
+			(void) (*src->fill_input_buffer)(cinfo);
 
 			/* note we assume that fill_input_buffer will never
 			 * return FALSE, so suspension need not be handled.
 			 */
 		}
 
-		src->pub.next_input_byte += (size_t) num_bytes;
-		src->pub.bytes_in_buffer -= (size_t) num_bytes;
+		src->next_input_byte += (size_t) num_bytes;
+		src->bytes_in_buffer -= (size_t) num_bytes;
 	}
 }
 
 static void
 skip_input_data_mappable(j_decompress_ptr cinfo, long num_bytes)
 {
-	Source *src = (Source *) cinfo->src;
+	struct jpeg_source_mgr *src = cinfo->src;
 
-	if (num_bytes > (long) src->pub.bytes_in_buffer) {
-		src->pub.next_input_byte += src->pub.bytes_in_buffer;
-		src->pub.bytes_in_buffer = 0;
+	if (num_bytes > (long) src->bytes_in_buffer) {
+		src->next_input_byte += src->bytes_in_buffer;
+		src->bytes_in_buffer = 0;
 	}
 	else {
-		src->pub.next_input_byte += (size_t) num_bytes;
-		src->pub.bytes_in_buffer -= (size_t) num_bytes;
+		src->next_input_byte += (size_t) num_bytes;
+		src->bytes_in_buffer -= (size_t) num_bytes;
 	}
 }
 
-static int
-readjpeg_open_input(ReadJpeg *jpeg)
+int
+vips__readjpeg_open_input(ReadJpeg *jpeg)
 {
 	j_decompress_ptr cinfo = &jpeg->cinfo;
 
@@ -335,11 +296,8 @@ readjpeg_open_input(ReadJpeg *jpeg)
 		if (vips_source_rewind(jpeg->source))
 			return -1;
 
-		cinfo->src =
-			(struct jpeg_source_mgr *) (*cinfo->mem->alloc_small)(
-				(j_common_ptr) cinfo,
-				JPOOL_PERMANENT,
-				sizeof(Source));
+		cinfo->src = (struct jpeg_source_mgr *) (*cinfo->mem->alloc_small)(
+				(j_common_ptr) cinfo, JPOOL_PERMANENT, sizeof(Source));
 
 		src = (Source *) cinfo->src;
 		src->jpeg = jpeg;
@@ -352,11 +310,10 @@ readjpeg_open_input(ReadJpeg *jpeg)
 
 		if (vips_source_is_mappable(jpeg->source)) {
 			size_t src_len;
-			const unsigned char *src_data = vips_source_map(
-				jpeg->source, &src_len);
+			const unsigned char *src_data =
+				vips_source_map(jpeg->source, &src_len);
 
-			src->pub.fill_input_buffer =
-				source_fill_input_buffer_mappable;
+			src->pub.fill_input_buffer = source_fill_input_buffer_mappable;
 			src->pub.skip_input_data = skip_input_data_mappable;
 			src->pub.bytes_in_buffer = src_len;
 			src->pub.next_input_byte = src_data;
@@ -412,8 +369,7 @@ static int
 readjpeg_free(ReadJpeg *jpeg)
 {
 	if (jpeg->eman.pub.num_warnings != 0) {
-		g_warning(_("read gave %ld warnings"),
-			jpeg->eman.pub.num_warnings);
+		g_warning("read gave %ld warnings", jpeg->eman.pub.num_warnings);
 		g_warning("%s", vips_error_buffer());
 
 		/* Make the message only appear once.
@@ -447,8 +403,10 @@ readjpeg_minimise_cb(VipsImage *image, ReadJpeg *jpeg)
 	vips_source_minimise(jpeg->source);
 }
 
-static ReadJpeg *
-readjpeg_new(VipsSource *source, VipsImage *out,
+/* Shared with uhdr load (for fast uhdr detect).
+ */
+ReadJpeg *
+vips__readjpeg_new(VipsSource *source, VipsImage *out,
 	int shrink, VipsFailOn fail_on, gboolean autorotate,
 	gboolean unlimited)
 {
@@ -482,10 +440,8 @@ readjpeg_new(VipsSource *source, VipsImage *out,
 
 	jpeg_create_decompress(&jpeg->cinfo);
 
-	g_signal_connect(out, "close",
-		G_CALLBACK(readjpeg_close_cb), jpeg);
-	g_signal_connect(out, "minimise",
-		G_CALLBACK(readjpeg_minimise_cb), jpeg);
+	g_signal_connect(out, "close", G_CALLBACK(readjpeg_close_cb), jpeg);
+	g_signal_connect(out, "minimise", G_CALLBACK(readjpeg_minimise_cb), jpeg);
 
 	return jpeg;
 }
@@ -506,7 +462,7 @@ find_chroma_subsample(struct jpeg_decompress_struct *cinfo)
 		: (has_subsample ? "4:2:0" : "4:4:4");
 }
 
-static int
+static void
 attach_blob(VipsImage *im, const char *field, void *data, size_t data_length)
 {
 	/* Only use the first one.
@@ -516,7 +472,7 @@ attach_blob(VipsImage *im, const char *field, void *data, size_t data_length)
 		printf("attach_blob: second %s block, ignoring\n", field);
 #endif /*DEBUG*/
 
-		return 0;
+		return;
 	}
 
 #ifdef DEBUG
@@ -525,37 +481,29 @@ attach_blob(VipsImage *im, const char *field, void *data, size_t data_length)
 #endif /*DEBUG*/
 
 	vips_image_set_blob_copy(im, field, data, data_length);
-
-	return 0;
 }
 
 /* data is the XMP string ... it'll have something like
  * "http://ns.adobe.com/xap/1.0/" at the front, then a null character, then
  * the real XMP.
  */
-static int
-attach_xmp_blob(VipsImage *im, void *data, size_t data_length)
+static void
+attach_xmp_blob(VipsImage *im, char *data, size_t data_length)
 {
-	char *p = (char *) data;
-	int i;
-
-	if (data_length < 4 ||
-		!vips_isprefix("http", p))
-		return 0;
-
 	/* Search for a null char within the first few characters. 80
 	 * should be plenty for a basic URL.
 	 *
 	 * -2 for the extra null.
 	 */
-	for (i = 0; i < VIPS_MIN(80, data_length - 2); i++)
-		if (!p[i])
-			break;
-	if (p[i])
-		return 0;
+	char *p = memchr(data, '\0', VIPS_MIN(80, data_length - 2));
+	if (!p)
+		return;
 
-	return attach_blob(im, VIPS_META_XMP_NAME,
-		p + i + 1, data_length - i - 1);
+	size_t i = p - data;
+	data_length -= i + 1;
+
+	attach_blob(im, VIPS_META_XMP_NAME,
+		data + i + 1, data_length);
 }
 
 /* Number of app2 sections we can capture. Each one can be 64k, so 6400k should
@@ -654,7 +602,7 @@ read_jpeg_header(ReadJpeg *jpeg, VipsImage *out)
 			break;
 
 		default:
-			g_warning("%s", _("unknown JFIF resolution unit"));
+			g_warning("unknown JFIF resolution unit");
 			break;
 		}
 
@@ -714,9 +662,21 @@ read_jpeg_header(ReadJpeg *jpeg, VipsImage *out)
 				p->data_length,
 				p->marker - JPEG_APP0);
 
-			for (i = 0; i < 10; i++)
-				printf("\t%d) '%c' (%d)\n",
-					i, p->data[i], p->data[i]);
+			i = 0;
+			for (int row = 0; i < VIPS_MIN(100, p->data_length); row++) {
+				printf(" 0x%02x)", i);
+
+				for (int col = 0; col < 8 &&
+						i < VIPS_MIN(100, p->data_length); col++, i++) {
+					printf(" %02x", p->data[i]);
+					if (p->data[i] > 32 && p->data[i] < 127)
+						printf(" '%c'", p->data[i]);
+					else
+						printf("    ");
+				}
+
+				printf("\n");
+			}
 		}
 #endif /*DEBUG*/
 
@@ -725,16 +685,14 @@ read_jpeg_header(ReadJpeg *jpeg, VipsImage *out)
 			/* Possible EXIF or XMP data.
 			 */
 			if (p->data_length > 4 &&
-				vips_isprefix("Exif", (char *) p->data) &&
+				vips_isprefix("Exif", (char *) p->data))
 				attach_blob(out, VIPS_META_EXIF_NAME,
-					p->data, p->data_length))
-				return -1;
+					p->data, p->data_length);
 
 			if (p->data_length > 4 &&
-				vips_isprefix("http", (char *) p->data) &&
+				vips_isprefix("http", (char *) p->data))
 				attach_xmp_blob(out,
-					p->data, p->data_length))
-				return -1;
+					(char *) p->data, p->data_length);
 
 			break;
 
@@ -763,17 +721,15 @@ read_jpeg_header(ReadJpeg *jpeg, VipsImage *out)
 			 */
 			if (p->data_length > 5 &&
 				vips_isprefix("Photo", (char *) p->data)) {
-				if (attach_blob(out, VIPS_META_IPTC_NAME,
-						p->data, p->data_length))
-					return -1;
+				attach_blob(out, VIPS_META_IPTC_NAME,
+					p->data, p->data_length);
 
 				/* Older versions of libvips used this misspelt
 				 * name :-( attach under this name too for
 				 * compatibility.
 				 */
-				if (attach_blob(out, "ipct-data",
-						p->data, p->data_length))
-					return -1;
+				attach_blob(out, "ipct-data",
+					p->data, p->data_length);
 			}
 			break;
 
@@ -791,7 +747,8 @@ read_jpeg_header(ReadJpeg *jpeg, VipsImage *out)
 			 * data[11] == 2 - YCCK
 			 *
 			 * Leave this code here in case we come up with a
-			 * better rule.
+			 * better rule. Make sure to also uncomment
+			 * jpeg_save_markers() for the APP14 marker.
 			 */
 			if (p->data_length >= 12 &&
 				vips_isprefix("Adobe", (char *) p->data)) {
@@ -896,7 +853,7 @@ read_jpeg_generate(VipsRegion *out_region,
 		VIPS_GATE_STOP("read_jpeg_generate: work");
 
 #ifdef DEBUG
-		printf("read_jpeg_generate: longjmp() exit\n");
+		printf("read_jpeg_generate() exit\n");
 #endif /*DEBUG*/
 
 		return -1;
@@ -998,15 +955,18 @@ read_jpeg_image(ReadJpeg *jpeg, VipsImage *out)
 }
 
 static int
-vips__jpeg_read(ReadJpeg *jpeg, VipsImage *out, gboolean header_only)
+jpeg_read(ReadJpeg *jpeg, VipsImage *out, gboolean header_only)
 {
-	/* Need to read in APP1 (EXIF metadata), APP2 (ICC profile), APP13
-	 * (photoshop IPTC) and APP14 (Adobe flags).
+	/* Need to read in APP1 (EXIF/XMP metadata), APP2 (ICC profile) and APP13
+	 * (photoshop IPTC).
 	 */
 	jpeg_save_markers(&jpeg->cinfo, JPEG_APP0 + 1, 0xffff);
 	jpeg_save_markers(&jpeg->cinfo, JPEG_APP0 + 2, 0xffff);
 	jpeg_save_markers(&jpeg->cinfo, JPEG_APP0 + 13, 0xffff);
-	jpeg_save_markers(&jpeg->cinfo, JPEG_APP0 + 14, 0xffff);
+
+	/* APP14 (Adobe flags), deliberately ignored, see above.
+	 */
+	// jpeg_save_markers(&jpeg->cinfo, JPEG_APP0 + 14, 0xffff);
 
 #ifdef DEBUG
 	{
@@ -1054,7 +1014,7 @@ vips__jpeg_read_source(VipsSource *source, VipsImage *out,
 {
 	ReadJpeg *jpeg;
 
-	if (!(jpeg = readjpeg_new(source, out, shrink, fail_on,
+	if (!(jpeg = vips__readjpeg_new(source, out, shrink, fail_on,
 			  autorotate, unlimited)))
 		return -1;
 
@@ -1064,8 +1024,8 @@ vips__jpeg_read_source(VipsSource *source, VipsImage *out,
 	if (setjmp(jpeg->eman.jmp))
 		return -1;
 
-	if (readjpeg_open_input(jpeg) ||
-		vips__jpeg_read(jpeg, out, header_only))
+	if (vips__readjpeg_open_input(jpeg) ||
+		jpeg_read(jpeg, out, header_only))
 		return -1;
 
 	if (header_only)
