@@ -249,7 +249,7 @@ vips_foreign_load_png_set_header(VipsForeignLoadPng *png, VipsImage *image)
 		 * attacks.
 		 */
 		if (!png->unlimited && n_text > MAX_PNG_TEXT_CHUNKS) {
-			g_warning(_("%d text chunks, only %d text chunks will be loaded"),
+			g_warning("%d text chunks, only %d text chunks will be loaded",
 				n_text, MAX_PNG_TEXT_CHUNKS);
 			n_text = MAX_PNG_TEXT_CHUNKS;
 		}
@@ -713,11 +713,8 @@ vips_foreign_load_png_source_build(VipsObject *object)
 		g_object_ref(png->source);
 	}
 
-	if (VIPS_OBJECT_CLASS(vips_foreign_load_png_source_parent_class)
-			->build(object))
-		return -1;
-
-	return 0;
+	return VIPS_OBJECT_CLASS(vips_foreign_load_png_source_parent_class)
+		->build(object);
 }
 
 static gboolean
@@ -790,10 +787,8 @@ vips_foreign_load_png_file_build(VipsObject *object)
 		!(png->source = vips_source_new_from_file(file->filename)))
 		return -1;
 
-	if (VIPS_OBJECT_CLASS(vips_foreign_load_png_file_parent_class)->build(object))
-		return -1;
-
-	return 0;
+	return VIPS_OBJECT_CLASS(vips_foreign_load_png_file_parent_class)
+		->build(object);
 }
 
 static gboolean
@@ -870,11 +865,8 @@ vips_foreign_load_png_buffer_build(VipsObject *object)
 			  VIPS_AREA(buffer->blob)->length)))
 		return -1;
 
-	if (VIPS_OBJECT_CLASS(vips_foreign_load_png_buffer_parent_class)
-			->build(object))
-		return -1;
-
-	return 0;
+	return VIPS_OBJECT_CLASS(vips_foreign_load_png_buffer_parent_class)
+		->build(object);
 }
 
 static gboolean
