@@ -617,24 +617,39 @@ vips_foreign_save_spng_build(VipsObject *object)
 	if (vips_object_argument_isset(object, "colours"))
 		spng->bitdepth = ceil(log2(spng->colours));
 
-	/* The bitdepth param can change the interpretation.
-	 */
-	VipsInterpretation interpretation;
-	if (in->Bands > 2) {
-	   if (spng->bitdepth > 8)
-		   interpretation = VIPS_INTERPRETATION_RGB16;
-	   else
-		   interpretation = VIPS_INTERPRETATION_sRGB;
+	if (vips_colourspace_issupported(in)) {
+		VipsInterpretation interpretation;
+
+		/* The bitdepth param can change the interpretation.
+		 */
+		if (in->Bands > 2) {
+		   if (spng->bitdepth > 8)
+			   interpretation = VIPS_INTERPRETATION_RGB16;
+		   else
+			   interpretation = VIPS_INTERPRETATION_sRGB;
+		}
+		else {
+		   if (spng->bitdepth > 8)
+			   interpretation = VIPS_INTERPRETATION_GREY16;
+		   else
+			   interpretation = VIPS_INTERPRETATION_B_W;
+		}
+
+		if (vips_colourspace(in, &x, interpretation, NULL)) {
+			g_object_unref(in);
+			return -1;
+		}
 	}
 	else {
-	   if (spng->bitdepth > 8)
-		   interpretation = VIPS_INTERPRETATION_GREY16;
-	   else
-		   interpretation = VIPS_INTERPRETATION_B_W;
-	}
-	if (vips_colourspace(in, &x, interpretation, NULL)) {
-		g_object_unref(in);
-		return -1;
+		VipsBandFormat target_format =
+			spng->bitdepth > 8 ? VIPS_FORMAT_USHORT : VIPS_FORMAT_UCHAR;
+
+		/* Cast in down to target format if we can.
+		 */
+		if (vips_cast(in, &x, target_format, NULL)) {
+			g_object_unref(in);
+			return -1;
+		}
 	}
 	g_object_unref(in);
 	in = x;
