@@ -147,73 +147,69 @@ vips_foreign_save_tiff_build(VipsObject *object)
 	VipsForeignSave *save = (VipsForeignSave *) object;
 	VipsForeignSaveTiff *tiff = (VipsForeignSaveTiff *) object;
 
-	const char *p;
+	if (VIPS_OBJECT_CLASS(vips_foreign_save_tiff_parent_class)->build(object))
+		return -1;
+
+	VipsImage *ready = save->ready;
+	g_object_ref(ready);
 
 	/* If we are saving jpeg-in-tiff, we need a different convert_saveable
 	 * path. The regular tiff one will let through things like float and
 	 * 16-bit and alpha for example, which will make the jpeg saver choke.
 	 */
-	if (save->in &&
-		tiff->compression == VIPS_FOREIGN_TIFF_COMPRESSION_JPEG) {
+	if (tiff->compression == VIPS_FOREIGN_TIFF_COMPRESSION_JPEG) {
 		VipsImage *x;
 
 		/* See also vips_foreign_save_jpeg_class_init().
 		 */
-		if (vips__foreign_convert_saveable(save->in, &x,
-				VIPS_SAVEABLE_RGB_CMYK, bandfmt_jpeg, class->coding,
-				save->background))
+		if (vips__foreign_convert_saveable(ready, &x,
+				VIPS_FOREIGN_SAVEABLE_MONO |
+					VIPS_FOREIGN_SAVEABLE_RGB |
+					VIPS_FOREIGN_SAVEABLE_CMYK,
+				bandfmt_jpeg, class->coding,
+				save->background)) {
+			VIPS_UNREF(ready);
 			return -1;
+		}
 
-		g_object_set(object, "in", x, NULL);
-		g_object_unref(x);
+		VIPS_UNREF(ready);
+		ready = x;
 	}
 
-	if (VIPS_OBJECT_CLASS(vips_foreign_save_tiff_parent_class)->build(object))
-		return -1;
-
-	/* Default xres/yres to the values from the image. This is always
-	 * pixels/mm.
-	 */
-	if (!vips_object_argument_isset(object, "xres"))
-		tiff->xres = save->ready->Xres;
-	if (!vips_object_argument_isset(object, "yres"))
-		tiff->yres = save->ready->Yres;
-
-	/* We default to pixels/cm.
-	 */
-	tiff->xres *= 10.0;
-	tiff->yres *= 10.0;
-
 	/* resunit param overrides resunit metadata.
 	 */
+	VipsForeignTiffResunit resunit = tiff->resunit;
+	const char *p;
 	if (!vips_object_argument_isset(object, "resunit") &&
-		vips_image_get_typeof(save->ready,
-			VIPS_META_RESOLUTION_UNIT) &&
-		!vips_image_get_string(save->ready,
-			VIPS_META_RESOLUTION_UNIT, &p) &&
+		vips_image_get_typeof(ready, VIPS_META_RESOLUTION_UNIT) &&
+		!vips_image_get_string(ready, VIPS_META_RESOLUTION_UNIT, &p) &&
 		vips_isprefix("in", p))
-		tiff->resunit = VIPS_FOREIGN_TIFF_RESUNIT_INCH;
+		resunit = VIPS_FOREIGN_TIFF_RESUNIT_INCH;
 
-	if (tiff->resunit == VIPS_FOREIGN_TIFF_RESUNIT_INCH) {
-		tiff->xres *= 2.54;
-		tiff->yres *= 2.54;
+	double xres = vips_object_argument_isset(object, "xres") ?
+		tiff->xres : ready->Xres;
+
+	double yres = vips_object_argument_isset(object, "yres") ?
+		tiff->yres : ready->Yres;
+
+	if (resunit == VIPS_FOREIGN_TIFF_RESUNIT_INCH) {
+		xres *= 25.4;
+		yres *= 25.4;
+	}
+	else {
+		xres *= 10.0;
+		yres *= 10.0;
 	}
 
-	/* Handle the deprecated squash parameter.
-	 */
-	if (tiff->squash)
-		/* We set that even in the case of LAB to LABQ.
-		 */
-		tiff->bitdepth = 1;
-
-	if (vips__tiff_write_target(save->ready, tiff->target,
+	if (vips__tiff_write_target(ready, tiff->target,
 			tiff->compression, tiff->Q, tiff->predictor,
 			save->profile,
 			tiff->tile, tiff->tile_width, tiff->tile_height,
 			tiff->pyramid,
-			tiff->bitdepth,
+			// deprecated "squash" param
+			tiff->squash ? 1 : tiff->bitdepth,
 			tiff->miniswhite,
-			tiff->resunit, tiff->xres, tiff->yres,
+			resunit, xres, yres,
 			tiff->bigtiff,
 			tiff->rgbjpeg,
 			tiff->properties,
@@ -223,8 +219,12 @@ vips_foreign_save_tiff_build(VipsObject *object)
 			tiff->depth,
 			tiff->subifd,
 			tiff->premultiply,
-			save->page_height))
+			save->page_height)) {
+		VIPS_UNREF(ready);
 		return -1;
+	}
+
+	VIPS_UNREF(ready);
 
 	if (vips_target_end(tiff->target))
 		return -1;
@@ -250,8 +250,8 @@ vips_foreign_save_tiff_class_init(VipsForeignSaveTiffClass *class)
 
 	foreign_class->suffs = vips__foreign_tiff_suffs;
 
-	save_class->saveable = VIPS_SAVEABLE_ANY;
-	save_class->coding[VIPS_CODING_LABQ] = TRUE;
+	save_class->saveable = VIPS_FOREIGN_SAVEABLE_ANY;
+	save_class->coding |= VIPS_FOREIGN_CODING_LABQ;
 
 	VIPS_ARG_ENUM(class, "compression", 6,
 		_("Compression"),
@@ -365,7 +365,7 @@ vips_foreign_save_tiff_class_init(VipsForeignSaveTiffClass *class)
 		_("Deflate (1-9, default 6) or ZSTD (1-22, default 9) compression level"),
 		VIPS_ARGUMENT_OPTIONAL_INPUT,
 		G_STRUCT_OFFSET(VipsForeignSaveTiff, level),
-		1, 22, 6);
+		0, 22, 0);
 
 	VIPS_ARG_BOOL(class, "lossless", 24,
 		_("Lossless"),
@@ -449,11 +449,8 @@ vips_foreign_save_tiff_target_build(VipsObject *object)
 	tiff->target = target->target;
 	g_object_ref(tiff->target);
 
-	if (VIPS_OBJECT_CLASS(vips_foreign_save_tiff_target_parent_class)
-			->build(object))
-		return -1;
-
-	return 0;
+	return VIPS_OBJECT_CLASS(vips_foreign_save_tiff_target_parent_class)
+		->build(object);
 }
 
 static void
@@ -503,11 +500,8 @@ vips_foreign_save_tiff_file_build(VipsObject *object)
 	if (!(tiff->target = vips_target_new_to_file(file->filename)))
 		return -1;
 
-	if (VIPS_OBJECT_CLASS(vips_foreign_save_tiff_file_parent_class)
-			->build(object))
-		return -1;
-
-	return 0;
+	return VIPS_OBJECT_CLASS(vips_foreign_save_tiff_file_parent_class)
+		->build(object);
 }
 
 static void
@@ -603,43 +597,19 @@ vips_foreign_save_tiff_buffer_init(VipsForeignSaveTiffBuffer *buffer)
  * vips_tiffsave: (method)
  * @in: image to save
  * @filename: file to write to
- * @...: %NULL-terminated list of optional named arguments
- *
- * Optional arguments:
- *
- * * @compression: use this #VipsForeignTiffCompression
- * * @Q: %gint quality factor
- * * @predictor: use this #VipsForeignTiffPredictor
- * * @tile: %gboolean, set %TRUE to write a tiled tiff
- * * @tile_width: %gint for tile size
- * * @tile_height: %gint for tile size
- * * @pyramid: %gboolean, write an image pyramid
- * * @bitdepth: %int, change bit depth to 1,2, or 4 bit
- * * @miniswhite: %gboolean, write 1-bit images as MINISWHITE
- * * @resunit: #VipsForeignTiffResunit for resolution unit
- * * @xres: %gdouble horizontal resolution in pixels/mm
- * * @yres: %gdouble vertical resolution in pixels/mm
- * * @bigtiff: %gboolean, write a BigTiff file
- * * @properties: %gboolean, set %TRUE to write an IMAGEDESCRIPTION tag
- * * @region_shrink: #VipsRegionShrink How to shrink each 2x2 region.
- * * @level: %gint, Zstd or Deflate (zlib) compression level
- * * @lossless: %gboolean, WebP lossless mode
- * * @depth: #VipsForeignDzDepth how deep to make the pyramid
- * * @subifd: %gboolean write pyr layers as sub-ifds
- * * @premultiply: %gboolean write premultiplied alpha
+ * @...: `NULL`-terminated list of optional named arguments
  *
  * Write a VIPS image to a file as TIFF.
  *
- * If @in has the #VIPS_META_PAGE_HEIGHT metadata item, this is assumed to be a
- * "toilet roll" image. It will be
- * written as series of pages, each #VIPS_META_PAGE_HEIGHT pixels high.
+ * If @in has the [const@META_PAGE_HEIGHT] metadata item, this is assumed to
+ * be a "toilet roll" image. It will be written as series of pages, each
+ * [const@META_PAGE_HEIGHT] pixels high.
  *
  * Use @compression to set the tiff compression. Currently jpeg, packbits,
  * fax4, lzw, none, deflate, webp and zstd are supported. The default is no
- * compression.
- * JPEG compression is a good lossy compressor for photographs, packbits is
- * good for 1-bit images, and deflate is the best lossless compression TIFF
- * can do.
+ * compression. JPEG compression is a good lossy compressor for photographs,
+ * packbits is good for 1-bit images, and deflate is the best lossless
+ * compression TIFF can do.
  *
  * XYZ images are automatically saved as libtiff LOGLUV with SGILOG compression.
  * Float LAB images are saved as float CIELAB. Set @bitdepth to save as 8-bit
@@ -647,15 +617,16 @@ vips_foreign_save_tiff_buffer_init(VipsForeignSaveTiffBuffer *buffer)
  *
  * Use @Q to set the JPEG compression factor. Default 75.
  *
- * User @level to set the ZSTD (1-22) or Deflate (1-9) compression level. Use @lossless to
- * set WEBP lossless mode on. Use @Q to set the WEBP compression level.
+ * User @level to set the ZSTD (1-22) or Deflate (1-9) compression level.
+ * Use @lossless to set WEBP lossless mode on. Use @Q to set the WEBP
+ * compression level.
  *
  * Use @predictor to set the predictor for lzw, deflate and zstd compression.
- * It defaults to #VIPS_FOREIGN_TIFF_PREDICTOR_HORIZONTAL, meaning horizontal
+ * It defaults to [enum@Vips.ForeignTiffPredictor.HORIZONTAL], meaning horizontal
  * differencing. Please refer to the libtiff
  * specifications for further discussion of various predictors.
  *
- * Set @tile to TRUE to write a tiled tiff.  By default tiff are written in
+ * Set @tile to `TRUE` to write a tiled tiff.  By default tiff are written in
  * strips. Use @tile_width and @tile_height to set the tile size. The defaiult
  * is 128 by 128.
  *
@@ -669,11 +640,13 @@ vips_foreign_save_tiff_buffer_init(VipsForeignSaveTiffBuffer *buffer)
  * a single layer.
  *
  * Set @bitdepth to save 8-bit uchar images as 1, 2 or 4-bit TIFFs.
+ *
  * In case of depth 1: Values >128 are written as white, values <=128 as black.
  * Normally vips will write MINISBLACK TIFFs where black is a 0 bit, but if you
  * set @miniswhite, it will use 0 for a white bit. Many pre-press applications
  * only work with images which use this sense. @miniswhite only affects one-bit
  * images, it does nothing for greyscale images.
+ *
  * In case of depth 2: The same holds but values < 64 are written as black.
  * For 64 <= values < 128 they are written as dark grey, for 128 <= values < 192
  * they are written as light gray and values above are written as white.
@@ -681,11 +654,9 @@ vips_foreign_save_tiff_buffer_init(VipsForeignSaveTiffBuffer *buffer)
  * In case of depth 4: values < 16 are written as black, and so on for the
  * lighter shades. In case @miniswhite is set to true this behavior is inverted.
  *
- * Use @resunit to override the default resolution unit.
- * The default
- * resolution unit is taken from the header field
- * #VIPS_META_RESOLUTION_UNIT. If this field is not set, then
- * VIPS defaults to cm.
+ * Use @resunit to override the default resolution unit. The default
+ * resolution unit is taken from the header field [const@META_RESOLUTION_UNIT].
+ * If this field is not set, then VIPS defaults to cm.
  *
  * Use @xres and @yres to override the default horizontal and vertical
  * resolutions. By default these values are taken from the VIPS image header.
@@ -695,15 +666,14 @@ vips_foreign_save_tiff_buffer_init(VipsForeignSaveTiffBuffer *buffer)
  * format that allows more than 4GB in a file.
  *
  * Set @properties to write all vips metadata to the IMAGEDESCRIPTION tag as
- * xml. If @properties is not set, the value of #VIPS_META_IMAGEDESCRIPTION is
+ * xml. If @properties is not set, the value of [const@META_IMAGEDESCRIPTION] is
  * used instead.
  *
- * The value of #VIPS_META_XMP_NAME is written to
- * the XMP tag. #VIPS_META_ORIENTATION (if set) is used to set the value of
- * the orientation
- * tag. #VIPS_META_IPTC (if set) is used to set the value of the IPTC tag.
- * #VIPS_META_PHOTOSHOP_NAME (if set) is used to set the value of the PHOTOSHOP
- * tag.
+ * The value of [const@META_XMP_NAME] is written to the XMP tag.
+ * [const@META_ORIENTATION] (if set) is used to set the value of the
+ * orientation tag. [const@META_IPTC_NAME] (if set) is used to set the
+ * value of the IPTC tag. [const@META_PHOTOSHOP_NAME] (if set) is used to
+ * set the value of the PHOTOSHOP tag.
  *
  * By default, pyramid layers are saved as consecutive pages.
  * Set @subifd to save pyramid layers as sub-directories of the main image.
@@ -712,7 +682,31 @@ vips_foreign_save_tiff_buffer_init(VipsForeignSaveTiffBuffer *buffer)
  * Set @premultiply to save with premultiplied alpha. Some programs, such as
  * InDesign, will only work with premultiplied alpha.
  *
- * See also: vips_tiffload(), vips_image_write_to_file().
+ * ::: tip "Optional arguments"
+ *     * @compression: [enum@ForeignTiffCompression], write with this
+ *       compression
+ *     * @Q: `gint`, quality factor
+ *     * @predictor: [enum@ForeignTiffPredictor], use this predictor
+ *     * @tile: `gboolean`, set `TRUE` to write a tiled tiff
+ *     * @tile_width: `gint`, for tile size
+ *     * @tile_height: `gint`, for tile size
+ *     * @pyramid: `gboolean`, write an image pyramid
+ *     * @bitdepth: `gint`, change bit depth to 1,2, or 4 bit
+ *     * @miniswhite: `gboolean`, write 1-bit images as MINISWHITE
+ *     * @resunit: [enum@ForeignTiffResunit] for resolution unit
+ *     * @xres: `gdouble`, horizontal resolution in pixels/mm
+ *     * @yres: `gdouble`, vertical resolution in pixels/mm
+ *     * @bigtiff: `gboolean`, write a BigTiff file
+ *     * @properties: `gboolean`, set `TRUE` to write an IMAGEDESCRIPTION tag
+ *     * @region_shrink: [enum@RegionShrink] How to shrink each 2x2 region.
+ *     * @level: `gint`, Zstd or Deflate (zlib) compression level
+ *     * @lossless: `gboolean`, WebP lossless mode
+ *     * @depth: [enum@ForeignDzDepth] how deep to make the pyramid
+ *     * @subifd: `gboolean`, write pyr layers as sub-ifds
+ *     * @premultiply: `gboolean`, write premultiplied alpha
+ *
+ * ::: seealso
+ *     [ctor@Image.tiffload], [method@Image.write_to_file].
  *
  * Returns: 0 on success, -1 on error.
  */
@@ -734,38 +728,39 @@ vips_tiffsave(VipsImage *in, const char *filename, ...)
  * @in: image to save
  * @buf: (array length=len) (element-type guint8): return output buffer here
  * @len: (type gsize): return output length here
- * @...: %NULL-terminated list of optional named arguments
+ * @...: `NULL`-terminated list of optional named arguments
  *
- * Optional arguments:
- *
- * * @compression: use this #VipsForeignTiffCompression
- * * @Q: %gint quality factor
- * * @predictor: use this #VipsForeignTiffPredictor
- * * @tile: %gboolean, set %TRUE to write a tiled tiff
- * * @tile_width: %gint for tile size
- * * @tile_height: %gint for tile size
- * * @pyramid: %gboolean, write an image pyramid
- * * @bitdepth: %int, set write bit depth to 1, 2, 4 or 8
- * * @miniswhite: %gboolean, write 1-bit images as MINISWHITE
- * * @resunit: #VipsForeignTiffResunit for resolution unit
- * * @xres: %gdouble horizontal resolution in pixels/mm
- * * @yres: %gdouble vertical resolution in pixels/mm
- * * @bigtiff: %gboolean, write a BigTiff file
- * * @properties: %gboolean, set %TRUE to write an IMAGEDESCRIPTION tag
- * * @region_shrink: #VipsRegionShrink How to shrink each 2x2 region.
- * * @level: %gint, Zstd or Deflate (zlib) compression level
- * * @lossless: %gboolean, WebP lossless mode
- * * @depth: #VipsForeignDzDepth how deep to make the pyramid
- * * @subifd: %gboolean write pyr layers as sub-ifds
- * * @premultiply: %gboolean write premultiplied alpha
- *
- * As vips_tiffsave(), but save to a memory buffer.
+ * As [method@Image.tiffsave], but save to a memory buffer.
  *
  * The address of the buffer is returned in @buf, the length of the buffer in
- * @len. You are responsible for freeing the buffer with g_free() when you
+ * @len. You are responsible for freeing the buffer with [func@GLib.free] when you
  * are done with it.
  *
- * See also: vips_tiffsave(), vips_image_write_to_file().
+ * ::: tip "Optional arguments"
+ *     * @compression: [enum@ForeignTiffCompression], write with this
+ *       compression
+ *     * @Q: `gint`, quality factor
+ *     * @predictor: [enum@ForeignTiffPredictor], use this predictor
+ *     * @tile: `gboolean`, set `TRUE` to write a tiled tiff
+ *     * @tile_width: `gint`, for tile size
+ *     * @tile_height: `gint`, for tile size
+ *     * @pyramid: `gboolean`, write an image pyramid
+ *     * @bitdepth: `gint`, change bit depth to 1,2, or 4 bit
+ *     * @miniswhite: `gboolean`, write 1-bit images as MINISWHITE
+ *     * @resunit: [enum@ForeignTiffResunit] for resolution unit
+ *     * @xres: `gdouble`, horizontal resolution in pixels/mm
+ *     * @yres: `gdouble`, vertical resolution in pixels/mm
+ *     * @bigtiff: `gboolean`, write a BigTiff file
+ *     * @properties: `gboolean`, set `TRUE` to write an IMAGEDESCRIPTION tag
+ *     * @region_shrink: [enum@RegionShrink] How to shrink each 2x2 region.
+ *     * @level: `gint`, Zstd or Deflate (zlib) compression level
+ *     * @lossless: `gboolean`, WebP lossless mode
+ *     * @depth: [enum@ForeignDzDepth] how deep to make the pyramid
+ *     * @subifd: `gboolean`, write pyr layers as sub-ifds
+ *     * @premultiply: `gboolean`, write premultiplied alpha
+ *
+ * ::: seealso
+ *     [method@Image.tiffsave], [method@Image.write_to_file].
  *
  * Returns: 0 on success, -1 on error.
  */
@@ -801,34 +796,35 @@ vips_tiffsave_buffer(VipsImage *in, void **buf, size_t *len, ...)
  * vips_tiffsave_target: (method)
  * @in: image to save
  * @target: save image to this target
- * @...: %NULL-terminated list of optional named arguments
+ * @...: `NULL`-terminated list of optional named arguments
  *
- * Optional arguments:
+ * As [method@Image.tiffsave], but save to a target.
  *
- * * @compression: use this #VipsForeignTiffCompression
- * * @Q: %gint quality factor
- * * @predictor: use this #VipsForeignTiffPredictor
- * * @tile: %gboolean, set %TRUE to write a tiled tiff
- * * @tile_width: %gint for tile size
- * * @tile_height: %gint for tile size
- * * @pyramid: %gboolean, write an image pyramid
- * * @bitdepth: %int, set write bit depth to 1, 2, 4 or 8
- * * @miniswhite: %gboolean, write 1-bit images as MINISWHITE
- * * @resunit: #VipsForeignTiffResunit for resolution unit
- * * @xres: %gdouble horizontal resolution in pixels/mm
- * * @yres: %gdouble vertical resolution in pixels/mm
- * * @bigtiff: %gboolean, write a BigTiff file
- * * @properties: %gboolean, set %TRUE to write an IMAGEDESCRIPTION tag
- * * @region_shrink: #VipsRegionShrink How to shrink each 2x2 region.
- * * @level: %gint, Zstd or Deflate (zlib) compression level
- * * @lossless: %gboolean, WebP lossless mode
- * * @depth: #VipsForeignDzDepth how deep to make the pyramid
- * * @subifd: %gboolean write pyr layers as sub-ifds
- * * @premultiply: %gboolean write premultiplied alpha
+ * ::: tip "Optional arguments"
+ *     * @compression: [enum@ForeignTiffCompression], write with this
+ *       compression
+ *     * @Q: `gint`, quality factor
+ *     * @predictor: [enum@ForeignTiffPredictor], use this predictor
+ *     * @tile: `gboolean`, set `TRUE` to write a tiled tiff
+ *     * @tile_width: `gint`, for tile size
+ *     * @tile_height: `gint`, for tile size
+ *     * @pyramid: `gboolean`, write an image pyramid
+ *     * @bitdepth: `gint`, change bit depth to 1,2, or 4 bit
+ *     * @miniswhite: `gboolean`, write 1-bit images as MINISWHITE
+ *     * @resunit: [enum@ForeignTiffResunit] for resolution unit
+ *     * @xres: `gdouble`, horizontal resolution in pixels/mm
+ *     * @yres: `gdouble`, vertical resolution in pixels/mm
+ *     * @bigtiff: `gboolean`, write a BigTiff file
+ *     * @properties: `gboolean`, set `TRUE` to write an IMAGEDESCRIPTION tag
+ *     * @region_shrink: [enum@RegionShrink] How to shrink each 2x2 region.
+ *     * @level: `gint`, Zstd or Deflate (zlib) compression level
+ *     * @lossless: `gboolean`, WebP lossless mode
+ *     * @depth: [enum@ForeignDzDepth] how deep to make the pyramid
+ *     * @subifd: `gboolean`, write pyr layers as sub-ifds
+ *     * @premultiply: `gboolean`, write premultiplied alpha
  *
- * As vips_tiffsave(), but save to a target.
- *
- * See also: vips_tiffsave(), vips_image_write_to_target().
+ * ::: seealso
+ *     [method@Image.tiffsave], [method@Image.write_to_target].
  *
  * Returns: 0 on success, -1 on error.
  */
