@@ -285,10 +285,6 @@
  */
 #define MAX_ALPHA (64)
 
-/* Bioformats uses this tag for lossy jp2k compressed tiles.
- */
-#define JP2K_LOSSY 33004
-
 /* Compression types we handle ourselves.
  */
 static int wtiff_we_compress[] = {
@@ -393,9 +389,19 @@ struct _Wtiff {
 
 	/* Lock thread calls into libtiff with this.
 	 */
-	GMutex *lock;
+	GMutex lock;
 };
 
+/* libvips uses size_t for the length of binary data items, but libtiff wants
+ * uint32.
+ */
+static void
+set_data64(TIFF *tif, guint32 tag, size_t length, const void *data)
+{
+	if (length <= UINT_MAX)
+        TIFFSetField(tif, tag, (guint32) length, data);
+}
+
 /* Write an ICC Profile from a file into the JPEG stream.
  */
 static int
@@ -410,7 +416,7 @@ embed_profile_file(TIFF *tif, const char *profile)
 		size_t length;
 		const void *data = vips_blob_get(blob, &length);
 
-		TIFFSetField(tif, TIFFTAG_ICCPROFILE, length, data);
+		set_data64(tif, TIFFTAG_ICCPROFILE, length, data);
 
 #ifdef DEBUG
 		printf("vips2tiff: attached profile \"%s\"\n", profile);
@@ -432,7 +438,7 @@ embed_profile_meta(TIFF *tif, VipsImage *im)
 
 	if (vips_image_get_blob(im, VIPS_META_ICC_NAME, &data, &length))
 		return -1;
-	TIFFSetField(tif, TIFFTAG_ICCPROFILE, length, data);
+	set_data64(tif, TIFFTAG_ICCPROFILE, length, data);
 
 #ifdef DEBUG
 	printf("vips2tiff: attached profile from meta\n");
@@ -441,6 +447,22 @@ embed_profile_meta(TIFF *tif, VipsImage *im)
 	return 0;
 }
 
+static int
+wtiff_handler_error(TIFF *tiff, void* user_data,
+	const char *module, const char *fmt, va_list ap)
+{
+	vips_verror("vips2tiff", fmt, ap);
+	return 1;
+}
+
+static int
+wtiff_handler_warning(TIFF *tiff, void* user_data,
+	const char *module, const char *fmt, va_list ap)
+{
+	g_logv(G_LOG_DOMAIN, G_LOG_LEVEL_WARNING, fmt, ap);
+	return 1;
+}
+
 static void
 wtiff_layer_init(Wtiff *wtiff, Layer **layer, Layer *above,
 	int width, int height)
@@ -560,7 +582,7 @@ wtiff_embed_xmp(Wtiff *wtiff, TIFF *tif)
 	if (vips_image_get_blob(wtiff->ready, VIPS_META_XMP_NAME,
 			&data, &size))
 		return -1;
-	TIFFSetField(tif, TIFFTAG_XMLPACKET, size, data);
+	set_data64(tif, TIFFTAG_XMLPACKET, size, data);
 
 #ifdef DEBUG
 	printf("vips2tiff: attached XMP from meta\n");
@@ -585,14 +607,14 @@ wtiff_embed_iptc(Wtiff *wtiff, TIFF *tif)
 	 * long, not byte.
 	 */
 	if (size & 3) {
-		g_warning("%s", _("rounding up IPTC data length"));
+		g_warning("rounding up IPTC data length");
 		size /= 4;
 		size += 1;
 	}
 	else
 		size /= 4;
 
-	TIFFSetField(tif, TIFFTAG_RICHTIFFIPTC, size, data);
+	set_data64(tif, TIFFTAG_RICHTIFFIPTC, size, data);
 
 #ifdef DEBUG
 	printf("vips2tiff: attached IPTC from meta\n");
@@ -607,16 +629,16 @@ wtiff_embed_photoshop(Wtiff *wtiff, TIFF *tif)
 	const void *data;
 	size_t size;
 
-	if (!vips_image_get_typeof(wtiff->ready, VIPS_META_PHOTOSHOP_NAME))
-		return 0;
-	if (vips_image_get_blob(wtiff->ready, VIPS_META_PHOTOSHOP_NAME,
-			&data, &size))
-		return -1;
-	TIFFSetField(tif, TIFFTAG_PHOTOSHOP, size, data);
+	if (vips_image_get_typeof(wtiff->ready, VIPS_META_PHOTOSHOP_NAME)) {
+		if (vips_image_get_blob(wtiff->ready, VIPS_META_PHOTOSHOP_NAME,
+				&data, &size))
+			return -1;
+		set_data64(tif, TIFFTAG_PHOTOSHOP, size, data);
 
 #ifdef DEBUG
-	printf("vips2tiff: attached photoshop data from meta\n");
+		printf("vips2tiff: attached %zd bytes of photoshop data\n", size);
 #endif /*DEBUG*/
+	}
 
 	return 0;
 }
@@ -869,10 +891,10 @@ wtiff_write_header(Wtiff *wtiff, Layer *layer)
 	if (wtiff->compression == COMPRESSION_ZSTD) {
 		// Set zstd compression level - only accept valid values (1-22)
 		if (wtiff->level)
-			TIFFSetField(tif, TIFFTAG_ZSTD_LEVEL, VIPS_CLIP(1, wtiff->level, 22));
-		if (wtiff->predictor != VIPS_FOREIGN_TIFF_PREDICTOR_NONE)
 			TIFFSetField(tif,
-				TIFFTAG_PREDICTOR, wtiff->predictor);
+				TIFFTAG_ZSTD_LEVEL, VIPS_CLIP(1, wtiff->level, 22));
+		if (wtiff->predictor != VIPS_FOREIGN_TIFF_PREDICTOR_NONE)
+			TIFFSetField(tif, TIFFTAG_PREDICTOR, wtiff->predictor);
 	}
 #endif /*HAVE_TIFF_COMPRESSION_WEBP*/
 
@@ -944,8 +966,7 @@ wtiff_write_header(Wtiff *wtiff, Layer *layer)
 
 		int alpha_bands;
 
-		TIFFSetField(tif, TIFFTAG_SAMPLESPERPIXEL,
-			wtiff->ready->Bands);
+		TIFFSetField(tif, TIFFTAG_SAMPLESPERPIXEL, wtiff->ready->Bands);
 		TIFFSetField(tif, TIFFTAG_BITSPERSAMPLE,
 			vips_format_sizeof(wtiff->ready->BandFmt) << 3);
 
@@ -1093,7 +1114,7 @@ wtiff_write_header(Wtiff *wtiff, Layer *layer)
 		printf("setting %zd bytes of table data\n", length);
 #endif /*DEBUG*/
 
-		TIFFSetField(tif, TIFFTAG_JPEGTABLES, length, buffer);
+		set_data64(tif, TIFFTAG_JPEGTABLES, length, buffer);
 
 		g_free(buffer);
 	}
@@ -1153,7 +1174,8 @@ wtiff_allocate_layers(Wtiff *wtiff)
 			vips__region_no_ownership(layer->copy);
 
 			layer->tif = vips__tiff_openout_target(layer->target,
-				wtiff->bigtiff);
+				wtiff->bigtiff, wtiff_handler_error,
+				wtiff_handler_warning, wtiff);
 			if (!layer->tif)
 				return -1;
 		}
@@ -1206,7 +1228,7 @@ wtiff_free(Wtiff *wtiff)
 
 	VIPS_UNREF(wtiff->ready);
 	VIPS_FREE(wtiff->tbuf);
-	VIPS_FREEF(vips_g_mutex_free, wtiff->lock);
+	g_mutex_clear(&wtiff->lock);
 	VIPS_FREE(wtiff);
 }
 
@@ -1373,7 +1395,7 @@ wtiff_new(VipsImage *input, VipsTarget *target,
 	wtiff->page_number = 0;
 	wtiff->n_pages = 1;
 	wtiff->image_height = input->Ysize;
-	wtiff->lock = vips_g_mutex_new();
+	g_mutex_init(&wtiff->lock);
 
 	/* Any pre-processing on the image.
 	 */
@@ -1463,8 +1485,7 @@ wtiff_new(VipsImage *input, VipsTarget *target,
 		!(wtiff->bitdepth == 1 ||
 			wtiff->bitdepth == 2 ||
 			wtiff->bitdepth == 4)) {
-		g_warning("%s",
-			_("bitdepth 1, 2 or 4 only -- disabling bitdepth"));
+		g_warning("bitdepth 1, 2 or 4 only -- disabling bitdepth");
 		wtiff->bitdepth = 0;
 	}
 
@@ -1475,16 +1496,14 @@ wtiff_new(VipsImage *input, VipsTarget *target,
 		!(wtiff->ready->Coding == VIPS_CODING_NONE &&
 			wtiff->ready->BandFmt == VIPS_FORMAT_UCHAR &&
 			wtiff->ready->Bands == 1)) {
-		g_warning("%s",
-			("can only set bitdepth for 1-band uchar and "
-			 "3-band float lab -- disabling bitdepth"));
+		g_warning("can only set bitdepth for 1-band uchar and "
+				  "3-band float lab -- disabling bitdepth");
 		wtiff->bitdepth = 0;
 	}
 
 	if (wtiff->bitdepth &&
 		wtiff->compression == COMPRESSION_JPEG) {
-		g_warning("%s",
-			_("can't have <8 bit JPEG -- disabling JPEG"));
+		g_warning("can't have <8 bit JPEG -- disabling JPEG");
 		wtiff->compression = COMPRESSION_NONE;
 	}
 
@@ -1494,9 +1513,8 @@ wtiff_new(VipsImage *input, VipsTarget *target,
 		(wtiff->ready->Coding != VIPS_CODING_NONE ||
 			vips_band_format_iscomplex(wtiff->ready->BandFmt) ||
 			wtiff->ready->Bands > 2)) {
-		g_warning("%s",
-			_("can only save non-complex greyscale images "
-			  "as miniswhite -- disabling miniswhite"));
+		g_warning("can only save non-complex greyscale images "
+				  "as miniswhite -- disabling miniswhite");
 		wtiff->miniswhite = FALSE;
 	}
 
@@ -1771,11 +1789,11 @@ wtiff_row_add_tile(WtiffRow *row,
 	tile->buffer = buffer;
 	tile->length = length;
 
-	g_mutex_lock(row->wtiff->lock);
+	g_mutex_lock(&row->wtiff->lock);
 
 	row->tiles = g_slist_prepend(row->tiles, tile);
 
-	g_mutex_unlock(row->wtiff->lock);
+	g_mutex_unlock(&row->wtiff->lock);
 
 	return 0;
 }
@@ -1962,20 +1980,26 @@ wtiff_layer_write_tiles(Wtiff *wtiff, Layer *layer, VipsRegion *strip)
 		 */
 		WtiffRow row = { wtiff, strip, layer, 0 };
 
+		VipsImage *x;
+		if (vips_copy(im, &x, NULL))
+			return -1;
+
 		/* We don't want threadpool_run to minimise on completion -- we need to
 		 * keep the cache on the pipeline before us.
 		 */
-		vips_image_set_int(im, "vips-no-minimise", 1);
+		vips_image_set_int(x, "vips-no-minimise", 1);
 
-		if (vips_threadpool_run(im,
+		if (vips_threadpool_run(x,
 				vips_thread_state_new,
 				wtiff_layer_row_allocate,
 				wtiff_layer_row_work,
 				NULL,
 				&row)) {
+			VIPS_UNREF(x);
 			wtiff_row_free(&row);
 			return -1;
 		}
+		VIPS_UNREF(x);
 
 		if (wtiff_row_write(&row, layer->tif)) {
 			wtiff_row_free(&row);
@@ -2284,7 +2308,7 @@ wtiff_write_lines(Wtiff *wtiff, VipsRegion *region, VipsRect *lines)
  */
 #define CopyField(tag, v) \
 	if (TIFFGetField(in, tag, &v)) \
-	TIFFSetField(out, tag, v)
+		TIFFSetField(out, tag, v)
 
 static int
 wtiff_copy_tiles(Wtiff *wtiff, TIFF *out, TIFF *in)
@@ -2352,6 +2376,7 @@ wtiff_copy_tiff(Wtiff *wtiff, TIFF *out, TIFF *in)
 	CopyField(TIFFTAG_ROWSPERSTRIP, ui32);
 	CopyField(TIFFTAG_SUBFILETYPE, ui32);
 	CopyField(TIFFTAG_PREDICTOR, ui16);
+	CopyField(TIFFTAG_SAMPLEFORMAT, ui16);
 
 	if (TIFFGetField(in, TIFFTAG_EXTRASAMPLES, &ui16, &a))
 		TIFFSetField(out, TIFFTAG_EXTRASAMPLES, ui16, a);
@@ -2408,7 +2433,8 @@ wtiff_gather(Wtiff *wtiff)
 			if (!(source = vips_source_new_from_target(layer->target)))
 				return -1;
 
-			if (!(in = vips__tiff_openin_source(source))) {
+			if (!(in = vips__tiff_openin_source(source, wtiff_handler_error,
+				wtiff_handler_warning, NULL, FALSE))) {
 				VIPS_UNREF(source);
 				return -1;
 			}
