@@ -45,7 +45,7 @@ typedef enum {
 	VIPS_EXTEND_MIRROR,
 	VIPS_EXTEND_WHITE,
 	VIPS_EXTEND_BACKGROUND,
-	VIPS_EXTEND_LAST
+	VIPS_EXTEND_LAST	/*< skip >*/
 } VipsExtend;
 
 typedef enum {
@@ -58,20 +58,20 @@ typedef enum {
 	VIPS_COMPASS_DIRECTION_SOUTH_EAST,
 	VIPS_COMPASS_DIRECTION_SOUTH_WEST,
 	VIPS_COMPASS_DIRECTION_NORTH_WEST,
-	VIPS_COMPASS_DIRECTION_LAST
+	VIPS_COMPASS_DIRECTION_LAST	/*< skip >*/
 } VipsCompassDirection;
 
 typedef enum {
 	VIPS_DIRECTION_HORIZONTAL,
 	VIPS_DIRECTION_VERTICAL,
-	VIPS_DIRECTION_LAST
+	VIPS_DIRECTION_LAST	/*< skip >*/
 } VipsDirection;
 
 typedef enum {
 	VIPS_ALIGN_LOW,
 	VIPS_ALIGN_CENTRE,
 	VIPS_ALIGN_HIGH,
-	VIPS_ALIGN_LAST
+	VIPS_ALIGN_LAST	/*< skip >*/
 } VipsAlign;
 
 typedef enum {
@@ -79,7 +79,7 @@ typedef enum {
 	VIPS_ANGLE_D90,
 	VIPS_ANGLE_D180,
 	VIPS_ANGLE_D270,
-	VIPS_ANGLE_LAST
+	VIPS_ANGLE_LAST	/*< skip >*/
 } VipsAngle;
 
 typedef enum {
@@ -91,7 +91,7 @@ typedef enum {
 	VIPS_ANGLE45_D225,
 	VIPS_ANGLE45_D270,
 	VIPS_ANGLE45_D315,
-	VIPS_ANGLE45_LAST
+	VIPS_ANGLE45_LAST	/*< skip >*/
 } VipsAngle45;
 
 typedef enum {
@@ -102,7 +102,7 @@ typedef enum {
 	VIPS_INTERESTING_LOW,
 	VIPS_INTERESTING_HIGH,
 	VIPS_INTERESTING_ALL,
-	VIPS_INTERESTING_LAST
+	VIPS_INTERESTING_LAST	/*< skip >*/
 } VipsInteresting;
 
 typedef enum {
@@ -131,7 +131,7 @@ typedef enum {
 	VIPS_BLEND_MODE_SOFT_LIGHT,
 	VIPS_BLEND_MODE_DIFFERENCE,
 	VIPS_BLEND_MODE_EXCLUSION,
-	VIPS_BLEND_MODE_LAST
+	VIPS_BLEND_MODE_LAST	/*< skip >*/
 } VipsBlendMode;
 
 VIPS_API
