@@ -46,11 +46,7 @@
 #include <vips/buf.h>
 
 /**
- * SECTION: buf
- * @short_description: a string you can append to
- * @stability: Stable
- * @see_also: #vips
- * @include: vips/vips.h
+ * VipsBuf:
  *
  * A message buffer you can append stuff to safely and quickly. If the message
  * gets too long, you get "..." and truncation. Message buffers can be on the
@@ -58,19 +54,19 @@
  *
  * For example:
  *
- * |[
+ * ```c
  * char txt[256];
  * VipsBuf buf = VIPS_BUF_STATIC(txt);
  * int i;
  *
  * vips_buf_appends(&buf, "Numbers are: ");
- * for (i = 0; i &lt; array_length; i++) {
- *   if (i &gt; 0)
- *     vips_buf_appends(&buf, ", ");
- *   vips_buf_appendg(&buf, array[i]);
+ * for (i = 0; i < array_length; i++) {
+ *     if (i > 0)
+ *         vips_buf_appends(&buf, ", ");
+ *     vips_buf_appendg(&buf, array[i]);
  * }
  * printf("%s", vips_buf_all(&buf));
- * ]|
+ * ```
  */
 
 /**
@@ -79,10 +75,10 @@
  *
  * Initialize a heap buffer. For example:
  *
- * |[
+ * ```c
  * char txt[256];
  * VipsBuf buf = VIPS_BUF_STATIC(txt);
- * ]|
+ * ```
  */
 
 /**
@@ -162,17 +158,17 @@ vips_buf_set_static(VipsBuf *buf, char *base, int mx)
  * @base: the start of the memory area to use for storage
  * @mx: the size of the storage area
  *
- * Initialise and attach to a static memory area. VIPS_BUF_STATIC() is usually
+ * Initialise and attach to a static memory area. [func@BUF_STATIC] is usually
  * more convenient.
  *
  * For example:
  *
- * |[
+ * ```c
  * char txt[256];
  * VipsBuf buf;
  *
  * vips_buf_init_static(&buf, txt, 256);
- * ]|
+ * ```
  *
  * Static buffers don't need to be freed when they go out of scope, but their
  * size must be set at compile-time.
@@ -224,13 +220,13 @@ vips_buf_set_dynamic(VipsBuf *buf, int mx)
  * Initialise and attach to a heap memory area.
  * The memory area needs to be at least 4 bytes long.
  *
- * |[
+ * ```c
  * VipsBuf buf;
  *
- * vips_buf_init_synamic(&buf, 256);
- * ]|
+ * vips_buf_init_dynamic(&buf, 256);
+ * ```
  *
- * Dynamic buffers must be freed with vips_buf_destroy(), but their size can
+ * Dynamic buffers must be freed with [method@Buf.destroy], but their size can
  * be set at runtime.
  */
 void
@@ -247,10 +243,10 @@ vips_buf_init_dynamic(VipsBuf *buf, int mx)
  * @sz: the size of the string to append
  *
  * Append at most @sz chars from @str to @buf. @sz < 0 means unlimited. This
- * is the low-level append operation: functions like vips_buf_appendf() build
+ * is the low-level append operation: functions like [method@Buf.appendf] build
  * on top of this.
  *
- * Returns: %FALSE on overflow, %TRUE otherwise.
+ * Returns: `FALSE` on overflow, `TRUE` otherwise.
  */
 gboolean
 vips_buf_appendns(VipsBuf *buf, const char *str, int sz)
@@ -304,7 +300,7 @@ vips_buf_appendns(VipsBuf *buf, const char *str, int sz)
  *
  * Append the whole of @str to @buf.
  *
- * Returns: %FALSE on overflow, %TRUE otherwise.
+ * Returns: `FALSE` on overflow, `TRUE` otherwise.
  */
 gboolean
 vips_buf_appends(VipsBuf *buf, const char *str)
@@ -319,7 +315,7 @@ vips_buf_appends(VipsBuf *buf, const char *str)
  *
  * Append a single character @ch to @buf.
  *
- * Returns: %FALSE on overflow, %TRUE otherwise.
+ * Returns: `FALSE` on overflow, `TRUE` otherwise.
  */
 gboolean
 vips_buf_appendc(VipsBuf *buf, char ch)
@@ -340,7 +336,7 @@ vips_buf_appendc(VipsBuf *buf, char ch)
  *
  * Swap the rightmost occurrence of @o for @n.
  *
- * Returns: %FALSE on overflow, %TRUE otherwise.
+ * Returns: `FALSE` on overflow, `TRUE` otherwise.
  */
 gboolean
 vips_buf_change(VipsBuf *buf, const char *old, const char *new)
@@ -365,8 +361,7 @@ vips_buf_change(VipsBuf *buf, const char *old, const char *new)
 
 	/* Move tail of buffer to make right-size space for new.
 	 */
-	memmove(buf->base + i + nlen, buf->base + i + olen,
-		buf->i - i - olen);
+	memmove(buf->base + i + nlen, buf->base + i + olen, buf->i - i - olen);
 
 	/* Copy new in.
 	 */
@@ -383,30 +378,29 @@ vips_buf_change(VipsBuf *buf, const char *old, const char *new)
  *
  * Remove the last character, if it's @ch.
  *
- * Returns: %FALSE on failure, %TRUE otherwise.
+ * Returns: `TRUE` if a character was removed, `FALSE` otherwise.
  */
 gboolean
 vips_buf_removec(VipsBuf *buf, char ch)
 {
-	if (buf->full)
-		return FALSE;
-	if (buf->i <= 0)
-		return FALSE;
-	if (buf->base[buf->i - 1] == ch)
+	if (buf->i > 0 &&
+		buf->base[buf->i - 1] == ch) {
 		buf->i -= 1;
-
-	return TRUE;
+		return TRUE;
+	}
+	else
+		return FALSE;
 }
 
 /**
  * vips_buf_vappendf:
  * @buf: the buffer
- * @fmt: <function>printf()</function>-style format string
+ * @fmt: `printf()`-style format string
  * @ap: arguments to format string
  *
- * Append to @buf, args as <function>vprintf()</function>.
+ * Append to @buf, args as [`vprintf()`](man:vprintf(3)).
  *
- * Returns: %FALSE on overflow, %TRUE otherwise.
+ * Returns: `FALSE` on overflow, `TRUE` otherwise.
  */
 gboolean
 vips_buf_vappendf(VipsBuf *buf, const char *fmt, va_list ap)
@@ -417,7 +411,10 @@ vips_buf_vappendf(VipsBuf *buf, const char *fmt, va_list ap)
 	if (buf->full)
 		return FALSE;
 
-	avail = buf->mx - buf->i - 4;
+	// -3 to leave space for "..."
+	// not -4, since the terminating \0 will already be on the string written
+	// by g_vsnprintf()
+	avail = buf->mx - buf->i - 3;
 	p = buf->base + buf->i;
 	(void) g_vsnprintf(p, avail, fmt, ap);
 	buf->i += strlen(p);
@@ -435,12 +432,12 @@ vips_buf_vappendf(VipsBuf *buf, const char *fmt, va_list ap)
 /**
  * vips_buf_appendf:
  * @buf: the buffer
- * @fmt: <function>printf()</function>-style format string
+ * @fmt: `printf()`-style format string
  * @...: arguments to format string
  *
  * Format the string and append to @buf.
  *
- * Returns: %FALSE on overflow, %TRUE otherwise.
+ * Returns: `FALSE` on overflow, `TRUE` otherwise.
  */
 gboolean
 vips_buf_appendf(VipsBuf *buf, const char *fmt, ...)
@@ -462,7 +459,7 @@ vips_buf_appendf(VipsBuf *buf, const char *fmt, ...)
  *
  * Append a double, non-localised. Useful for config files etc.
  *
- * Returns: %FALSE on overflow, %TRUE otherwise.
+ * Returns: `FALSE` on overflow, `TRUE` otherwise.
  */
 gboolean
 vips_buf_appendg(VipsBuf *buf, double g)
@@ -482,7 +479,7 @@ vips_buf_appendg(VipsBuf *buf, double g)
  * Append a number. If the number is -ve, add brackets. Needed for
  * building function arguments.
  *
- * Returns: %FALSE on overflow, %TRUE otherwise.
+ * Returns: `FALSE` on overflow, `TRUE` otherwise.
  */
 gboolean
 vips_buf_appendd(VipsBuf *buf, int d)
@@ -496,15 +493,15 @@ vips_buf_appendd(VipsBuf *buf, int d)
 /**
  * vips_buf_appendgv:
  * @buf: the buffer
- * @value: #GValue to format and append
+ * @value: [struct@GObject.Value] to format and append
  *
- * Format and append a #GValue as a printable thing. We display text line "3144
- * bytes of binary data" for BLOBs like icc-profile-data.
+ * Format and append a [struct@GObject.Value] as a printable thing.
+ * We display text like "3144 bytes of binary data" for BLOBs like icc-profile-data.
  *
- * Use vips_image_get_as_string() to make a text representation of a field.
+ * Use [method@Image.get_as_string] to make a text representation of a field.
  * That will base64-encode blobs, for example.
  *
- * Returns: %FALSE on overflow, %TRUE otherwise.
+ * Returns: `FALSE` on overflow, `TRUE` otherwise.
  */
 gboolean
 vips_buf_appendgv(VipsBuf *buf, GValue *value)
@@ -542,8 +539,7 @@ vips_buf_appendgv(VipsBuf *buf, GValue *value)
 	} break;
 
 	case G_TYPE_INT:
-		result = vips_buf_appendf(buf,
-			"%d", g_value_get_int(value));
+		result = vips_buf_appendf(buf, "%d", g_value_get_int(value));
 		handled = TRUE;
 		break;
 
@@ -554,8 +550,7 @@ vips_buf_appendgv(VipsBuf *buf, GValue *value)
 		break;
 
 	case G_TYPE_DOUBLE:
-		result = vips_buf_appendf(buf,
-			"%g", g_value_get_double(value));
+		result = vips_buf_appendf(buf, "%g", g_value_get_double(value));
 		handled = TRUE;
 		break;
 
@@ -636,8 +631,7 @@ vips_buf_appendgv(VipsBuf *buf, GValue *value)
 
 			arr = vips_value_get_array_image(value, &n);
 			for (i = 0; i < n; i++) {
-				vips_object_summary(VIPS_OBJECT(arr[i]),
-					buf);
+				vips_object_summary(VIPS_OBJECT(arr[i]), buf);
 				result = vips_buf_appends(buf, " ");
 			}
 			handled = TRUE;
@@ -667,7 +661,7 @@ vips_buf_appendgv(VipsBuf *buf, GValue *value)
  * Turn a number of bytes into a sensible string ... eg "12", "12KB", "12MB",
  * "12GB" etc.
  *
- * Returns: %FALSE on overflow, %TRUE otherwise.
+ * Returns: `FALSE` on overflow, `TRUE` otherwise.
  */
 gboolean
 vips_buf_append_size(VipsBuf *buf, size_t n)
@@ -716,7 +710,7 @@ vips_buf_append_size(VipsBuf *buf, size_t n)
  *
  * Return the contents of the buffer as a C string.
  *
- * Returns: the %NULL-terminated contents of the buffer. This is a pointer to
+ * Returns: the `NULL`-terminated contents of the buffer. This is a pointer to
  * the memory managed by the buffer and must not be freed.
  */
 const char *
@@ -733,7 +727,7 @@ vips_buf_all(VipsBuf *buf)
  *
  * Trim to just the first line (excluding "\n").
  *
- * Returns: the %NULL-terminated contents of the buffer. This is a pointer to
+ * Returns: the `NULL`-terminated contents of the buffer. This is a pointer to
  * the memory managed by the buffer and must not be freed.
  */
 const char *
@@ -751,7 +745,7 @@ vips_buf_firstline(VipsBuf *buf)
  * vips_buf_is_empty:
  * @buf: the buffer
  *
- * Returns: %TRUE if the buffer is empty.
+ * Returns: `TRUE` if the buffer is empty.
  */
 gboolean
 vips_buf_is_empty(VipsBuf *buf)
@@ -763,7 +757,7 @@ vips_buf_is_empty(VipsBuf *buf)
  * vips_buf_is_full:
  * @buf: the buffer
  *
- * Returns: %TRUE if the buffer is full.
+ * Returns: `TRUE` if the buffer is full.
  */
 gboolean
 vips_buf_is_full(VipsBuf *buf)
