@@ -108,21 +108,8 @@
 
 #include <vips/vips.h>
 #include <vips/internal.h>
-#include <vips/thread.h>
 #include <vips/debug.h>
 
-/**
- * SECTION: generate
- * @short_description: calculate pixels and pixel buffers
- * @stability: Stable
- * @see_also: <link linkend="VipsImage">VipsImage</link>,
- * <link linkend="VipsRegion">VipsRegion</link>
- * @include: vips/vips.h
- *
- * These functions let you attach generate functions to images
- * and ask for regions of images to be calculated.
- */
-
 /* Max number of images we can handle.
  */
 #define MAX_IMAGES (1000)
@@ -181,7 +168,7 @@ vips__link_break_rev(VipsImage *image_down, VipsImage *image_up, void *b)
 void
 vips__link_break_all(VipsImage *image)
 {
-	g_mutex_lock(vips__global_lock);
+	g_mutex_lock(&vips__global_lock);
 
 	vips_slist_map2(image->upstream,
 		(VipsSListMap2Fn) vips__link_break, image, NULL);
@@ -191,7 +178,7 @@ vips__link_break_all(VipsImage *image)
 	g_assert(!image->upstream);
 	g_assert(!image->downstream);
 
-	g_mutex_unlock(vips__global_lock);
+	g_mutex_unlock(&vips__global_lock);
 }
 
 typedef struct _LinkMap {
@@ -261,7 +248,7 @@ vips__link_map(VipsImage *image, gboolean upstream,
 	 * member. There will be intense confusion if two threads try to do
 	 * this at the same time.
 	 */
-	g_mutex_lock(vips__global_lock);
+	g_mutex_lock(&vips__global_lock);
 
 	serial += 1;
 	map.serial = serial;
@@ -271,7 +258,7 @@ vips__link_map(VipsImage *image, gboolean upstream,
 	for (p = images; p; p = p->next)
 		g_object_ref(p->data);
 
-	g_mutex_unlock(vips__global_lock);
+	g_mutex_unlock(&vips__global_lock);
 
 	result = vips_slist_map2(images, fn, a, b);
 
@@ -310,8 +297,8 @@ vips__demand_hint_array(VipsImage *image,
 	 */
 	set_hint = hint;
 	for (i = 0; i < len; i++)
-		set_hint = (VipsDemandStyle) VIPS_MIN(
-			(int) set_hint, (int) in[i]->dhint);
+		set_hint = (VipsDemandStyle)
+			VIPS_MIN((int) set_hint, (int) in[i]->dhint);
 
 	image->dhint = set_hint;
 
@@ -330,10 +317,10 @@ vips__demand_hint_array(VipsImage *image,
 
 	/* im depends on all these ims.
 	 */
-	g_mutex_lock(vips__global_lock);
+	g_mutex_lock(&vips__global_lock);
 	for (i = 0; i < len; i++)
 		vips__link_make(in[i], image);
-	g_mutex_unlock(vips__global_lock);
+	g_mutex_unlock(&vips__global_lock);
 
 	/* Set a flag on the image to say we remembered to call this thing.
 	 * vips_image_generate() and friends check this.
@@ -345,7 +332,7 @@ vips__demand_hint_array(VipsImage *image,
  * vips_image_pipeline_array:
  * @image: (out): output image
  * @hint: demand hint for @image
- * @in: (array zero-terminated=1): %NULL-terminated array of input images
+ * @in: (array zero-terminated=1): `NULL`-terminated array of input images
  *
  * Add an image to a pipeline. @image depends on all of the images in @in,
  * @image prefers to supply pixels according to @hint.
@@ -353,8 +340,8 @@ vips__demand_hint_array(VipsImage *image,
  * Operations can set demand hints, that is, hints to the VIPS IO system about
  * the type of region geometry they work best with. For example,
  * operations which transform coordinates will usually work best with
- * %VIPS_DEMAND_STYLE_SMALLTILE, operations which work on local windows of
- * pixels will like %VIPS_DEMAND_STYLE_FATSTRIP.
+ * [enum@Vips.DemandStyle.SMALLTILE], operations which work on local windows of
+ * pixels will like [enum@Vips.DemandStyle.FATSTRIP].
  *
  * Header fields in @image are set from the fields in @in, with lower-numbered
  * images in @in taking priority.
@@ -369,7 +356,8 @@ vips__demand_hint_array(VipsImage *image,
  * VIPS uses the list of input images to build the tree of operations it needs
  * for the cache invalidation system.
  *
- * See also: vips_image_pipelinev(), vips_image_generate().
+ * ::: seealso
+ *     [method@Image.pipelinev], [method@Image.generate].
  *
  * Returns: 0 on success, -1 on error.
  */
@@ -377,6 +365,9 @@ int
 vips_image_pipeline_array(VipsImage *image,
 	VipsDemandStyle hint, VipsImage **in)
 {
+	if (!vips_object_sanity(VIPS_OBJECT(image)))
+		return -1;
+
 	/* This function can be called more than once per output image. For
 	 * example, jpeg header load will call this once on ->out to set the
 	 * default hint, then later call it again to connect the output image
@@ -401,11 +392,12 @@ vips_image_pipeline_array(VipsImage *image,
  * vips_image_pipelinev:
  * @image: output image of pipeline
  * @hint: hint for this image
- * @...: %NULL-terminated list of input images
+ * @...: `NULL`-terminated list of input images
  *
- * Build an array and call vips_image_pipeline_array().
+ * Build an array and call [func@Image.pipeline_array].
  *
- * See also: vips_image_generate().
+ * ::: seealso
+ *     [method@Image.generate].
  */
 int
 vips_image_pipelinev(VipsImage *image, VipsDemandStyle hint, ...)
@@ -421,7 +413,7 @@ vips_image_pipelinev(VipsImage *image, VipsDemandStyle hint, ...)
 		;
 	va_end(ap);
 	if (i == MAX_IMAGES) {
-		g_warning("%s", _("too many images"));
+		g_warning("too many images");
 
 		/* Make sure we have a sentinel there.
 		 */
@@ -439,7 +431,8 @@ vips_image_pipelinev(VipsImage *image, VipsDemandStyle hint, ...)
  *
  * Start function for one image in. Input image is @a.
  *
- * See also: vips_image_generate().
+ * ::: seealso
+ *     [method@Image.generate].
  */
 void *
 vips_start_one(VipsImage *out, void *a, void *b)
@@ -457,7 +450,8 @@ vips_start_one(VipsImage *out, void *a, void *b)
  *
  * Stop function for one image in. Input image is @a.
  *
- * See also: vips_image_generate().
+ * ::: seealso
+ *     [method@Image.generate].
  */
 int
 vips_stop_one(void *seq, void *a, void *b)
@@ -476,9 +470,10 @@ vips_stop_one(void *seq, void *a, void *b)
  * @b: user data
  *
  * Stop function for many images in. @a is a pointer to
- * a %NULL-terminated array of input images.
+ * a `NULL`-terminated array of input images.
  *
- * See also: vips_image_generate().
+ * ::: seealso
+ *     [method@Image.generate].
  */
 int
 vips_stop_many(void *seq, void *a, void *b)
@@ -503,9 +498,10 @@ vips_stop_many(void *seq, void *a, void *b)
  * @b: user data
  *
  * Start function for many images in. @a is a pointer to
- * a %NULL-terminated array of input images.
+ * a `NULL`-terminated array of input images.
  *
- * See also: vips_image_generate(), vips_allocate_input_array()
+ * ::: seealso
+ *     [method@Image.generate], [func@allocate_input_array]
  */
 void *
 vips_start_many(VipsImage *out, void *a, void *b)
@@ -540,14 +536,16 @@ vips_start_many(VipsImage *out, void *a, void *b)
 /**
  * vips_allocate_input_array:
  * @out: free array when this image closes
- * @...: %NULL-terminated list of input images
+ * @...: `NULL`-terminated list of input images
  *
- * Convenience function --- make a %NULL-terminated array of input images.
- * Use with vips_start_many().
+ * Convenience function -- make a `NULL`-terminated array of input images.
+ * Use with [func@start_many].
  *
- * See also: vips_image_generate(), vips_start_many().
+ * ::: seealso
+ *     [method@Image.generate], [func@start_many].
  *
- * Returns: %NULL-terminated array of images. Do not free the result.
+ * Returns: (transfer none) (nullable): `NULL`-terminated array of images.
+ *   Do not free the result.
  */
 VipsImage **
 vips_allocate_input_array(VipsImage *out, ...)
@@ -588,23 +586,25 @@ vips_allocate_input_array(VipsImage *out, ...)
  * Start a new processing sequence for this generate function. This allocates
  * per-thread state, such as an input region.
  *
- * See also: vips_start_one(), vips_start_many().
+ * ::: seealso
+ *     [func@start_one], [func@start_many].
  *
  * Returns: a new sequence value
  */
 
 /**
  * VipsGenerateFn:
- * @out: #VipsRegion to fill
+ * @out: [class@Region] to fill
  * @seq: sequence value
  * @a: user data
  * @b: user data
  * @stop: set this to stop processing
  *
  * Fill @out->valid with pixels. @seq contains per-thread state, such as the
- * input regions. Set @stop to %TRUE to stop processing.
+ * input regions. Set @stop to `TRUE` to stop processing.
  *
- * See also: vips_image_generate(), vips_stop_many().
+ * ::: seealso
+ *     [method@Image.generate], [func@stop_many].
  *
  * Returns: 0 on success, -1 on error.
  */
@@ -618,7 +618,8 @@ vips_allocate_input_array(VipsImage *out, ...)
  * Stop a processing sequence. This frees
  * per-thread state, such as an input region.
  *
- * See also: vips_stop_one(), vips_stop_many().
+ * ::: seealso
+ *     [func@stop_one], [func@stop_many].
  *
  * Returns: 0 on success, -1 on error.
  */
@@ -628,15 +629,22 @@ vips_allocate_input_array(VipsImage *out, ...)
 static int
 write_vips(VipsRegion *region, VipsRect *area, void *a)
 {
-	size_t nwritten, count;
+	size_t count;
 	void *buf;
 
 	count = (size_t) region->bpl * area->height;
 	buf = VIPS_REGION_ADDR(region, 0, area->top);
 
 	do {
-		nwritten = write(region->im->fd, buf, count);
-		if (nwritten == (size_t) -1)
+		// write() uses int not size_t on windows, so we need to chunk
+		// ... max 1gb, why not
+		int chunk_size = VIPS_MIN(1024 * 1024 * 1024, count);
+		gint64 nwritten = write(region->im->fd, buf, chunk_size);
+
+		/* n == 0 isn't strictly an error, but we treat it as
+		 * one to make sure we don't get stuck in this loop.
+		 */
+		if (nwritten <= 0)
 			return errno;
 
 		buf = (void *) ((char *) buf + nwritten);
@@ -649,26 +657,27 @@ write_vips(VipsRegion *region, VipsRect *area, void *a)
 /**
  * vips_image_generate:
  * @image: generate this image
- * @start_fn: start sequences with this function
- * @generate_fn: generate pixels with this function
- * @stop_fn: stop sequences with this function
+ * @start_fn: (scope async): start sequences with this function
+ * @generate_fn: (scope async): generate pixels with this function
+ * @stop_fn: (scope async): stop sequences with this function
  * @a: user data
  * @b: user data
  *
  * Generates an image. The action depends on the image type.
  *
- * For images created with vips_image_new(), vips_image_generate() just
+ * For images created with [ctor@Image.new], [method@Image.generate] just
  * attaches the start/generate/stop callbacks and returns.
  *
- * For images created with vips_image_new_memory(), memory is allocated for
- * the whole image and it is entirely generated using vips_sink_memory().
+ * For images created with [ctor@Image.new_memory], memory is allocated for
+ * the whole image and it is entirely generated using [func@sink_memory].
  *
- * For images created with vips_image_new_temp_file() and friends, memory for
+ * For images created with [ctor@Image.new_temp_file] and friends, memory for
  * a few scanlines is allocated and
- * vips_sink_disc() used to generate the image in small chunks. As each
+ * [method@Image.sink_disc] used to generate the image in small chunks. As each
  * chunk is generated, it is written to disc.
  *
- * See also: vips_sink(), vips_image_new(), vips_region_prepare().
+ * ::: seealso
+ *     [func@sink_memory], [ctor@Image.new], [method@Region.prepare].
  *
  * Returns: 0 on success, or -1 on error.
  */
@@ -682,11 +691,12 @@ vips_image_generate(VipsImage *image,
 	VIPS_DEBUG_MSG("vips_image_generate: %p\n", image);
 
 	g_assert(generate_fn);
-	g_assert(vips_object_sanity(VIPS_OBJECT(image)));
+
+	if (!vips_object_sanity(VIPS_OBJECT(image)))
+		return -1;
 
 	if (!image->hint_set) {
-		vips_error("vips_image_generate",
-			"%s", _("demand hint not set"));
+		vips_error("vips_image_generate", "%s", _("demand hint not set"));
 		return -1;
 	}
 
