@@ -74,6 +74,46 @@
 #include <vips/debug.h>
 #include <vips/internal.h>
 
+/**
+ * VipsSource:
+ *
+ * A [class@Source] provides a unified interface for reading, seeking, and
+ * mapping data, regardless of the underlying source type.
+ *
+ * This source can originate from something like a socket, file or memory
+ * area.
+ *
+ * During the header phase, we save data from unseekable sources in a buffer
+ * so readers can rewind and read again. We don't buffer data during the
+ * decode stage.
+ */
+
+/**
+ * VipsSourceCustom:
+ *
+ * Subclass of [class@Source] with action signals for handlers.
+ *
+ * This is supposed to be useful for language bindings.
+ */
+
+/**
+ * VipsTarget:
+ *
+ * A [class@Target] provides a unified interface for writing data to various
+ * output destinations.
+ *
+ * This target could be a socket, file, memory area, or any other destination
+ * that accepts byte data.
+ */
+
+/**
+ * VipsTargetCustom:
+ *
+ * Subclass of [class@Target] with action signals for handlers.
+ *
+ * This is supposed to be useful for language bindings.
+ */
+
 #define MODE_READ CLOEXEC(BINARYIZE(O_RDONLY))
 
 /* -1 on a pipe isn't actually unbounded. Have a limit to prevent
@@ -92,13 +132,14 @@ static gint64 vips__pipe_read_limit = 1024 * 1024 * 1024;
  * automatically read into memory to EOF before the loader starts. This can
  * produce high memory use if the descriptor represents a large object.
  *
- * Use vips_pipe_read_limit_set() to limit the size of object that
+ * Use [func@pipe_read_limit_set] to limit the size of object that
  * will be read in this way. The default is 1GB.
  *
  * Set a value of -1 to mean no limit.
  *
- * See also: `--vips-pipe-read-limit` and the environment variable
- * `VIPS_PIPE_READ_LIMIT`.
+ * ::: seealso
+ *     `--vips-pipe-read-limit` and the environment variable
+ *     `VIPS_PIPE_READ_LIMIT`.
  */
 void
 vips_pipe_read_limit_set(gint64 limit)
@@ -173,7 +214,7 @@ static void
 vips_source_sanity(VipsSource *source)
 {
 	if (source->data) {
-		/* Not a pipe (can map and seek).
+		/* A memory buffer. Not a pipe (can map and seek).
 		 */
 		g_assert(!source->is_pipe);
 
@@ -193,6 +234,10 @@ vips_source_sanity(VipsSource *source)
 		g_assert(source->length != -1);
 	}
 	else if (source->is_pipe) {
+		/* Something like a descriptor we can't seek. It might not have
+		 * ->descriptor valid, that could be held in a subclass.
+		 */
+
 		if (source->decode) {
 			/* Reading pixel data.
 			 */
@@ -204,8 +249,7 @@ vips_source_sanity(VipsSource *source)
 			 */
 			g_assert(source->header_bytes);
 			g_assert(source->read_position >= 0);
-			g_assert(source->read_position <=
-				source->header_bytes->len);
+			g_assert(source->read_position <= source->header_bytes->len);
 		}
 
 		/* No length available.
@@ -213,15 +257,15 @@ vips_source_sanity(VipsSource *source)
 		g_assert(source->length == -1);
 	}
 	else {
-		/* Something like a seekable file.
+		/* Something like a descriptor we can seek. It might not have
+         * ->descriptor valid, that could be held in a subclass.
 		 */
 
 		/* After we're done with the header, the sniff buffer should
 		 * be gone.
 		 */
-		if (source->decode) {
+		if (source->decode)
 			g_assert(!source->sniff);
-		}
 
 		/* Once we've tested seek, the read position must lie within
 		 * the file.
@@ -231,13 +275,6 @@ vips_source_sanity(VipsSource *source)
 			g_assert(source->read_position >= 0);
 			g_assert(source->read_position <= source->length);
 		}
-
-		/* Supports minimise, so if descriptor is -1, we must have a
-		 * filename we can reopen.
-		 */
-		g_assert(VIPS_CONNECTION(source)->descriptor != -1 ||
-			(VIPS_CONNECTION(source)->filename &&
-				VIPS_CONNECTION(source)->descriptor));
 	}
 }
 #endif /*TEST_SANITY*/
@@ -340,11 +377,10 @@ vips_source_seek_real(VipsSource *source, gint64 offset, int whence)
 	VIPS_DEBUG_MSG("vips_source_seek_real:\n");
 
 	/* Like _read_real(), we must not set a vips_error. We need to use the
-	 * vips__seek() wrapper so we can seek long files on Windows.
+	 * vips__seek wrapper so we can seek long files on Windows.
 	 */
 	if (connection->descriptor != -1)
-		return vips__seek_no_error(connection->descriptor,
-			offset, whence);
+		return vips__seek_no_error(connection->descriptor, offset, whence);
 
 	return -1;
 }
@@ -388,7 +424,7 @@ vips_source_init(VipsSource *source)
  * @descriptor: read from this file descriptor
  *
  * Create an source attached to a file descriptor. @descriptor is
- * closed with close() when source is finalized.
+ * closed with [`close()`](man:close(2)) when source is finalized.
  *
  * Returns: a new source.
  */
@@ -416,16 +452,16 @@ vips_source_new_from_descriptor(int descriptor)
 
 /**
  * vips_source_new_from_file:
- * @descriptor: read from this filename
+ * @filename: read from this filename
  *
- * Create an source attached to a file.
+ * Create a source attached to a file.
  *
  * If this descriptor does not support mmap and the source is
  * used with a loader that can only work from memory, then the data will be
  * automatically read into memory to EOF before the loader starts. This can
  * produce high memory use if the descriptor represents a large object.
  *
- * Use vips_pipe_read_limit_set() to limit the size of object that
+ * Use [func@pipe_read_limit_set] to limit the size of object that
  * will be read in this way. The default is 1GB.
  *
  * Returns: a new source.
@@ -503,10 +539,8 @@ vips_source_new_from_target(VipsTarget *target)
 	if (vips_target_end(target))
 		return NULL;
 
-	if (connection->descriptor > 0) {
-		source = vips_source_new_from_descriptor(
-			connection->descriptor);
-	}
+	if (connection->descriptor > 0)
+		source = vips_source_new_from_descriptor(connection->descriptor);
 	else if (target->memory) {
 		VipsBlob *blob;
 
@@ -589,7 +623,7 @@ vips_source_new_from_options(const char *options)
  * @source: source to operate on
  *
  * Minimise the source. As many resources as can be safely removed are
- * removed. Use vips_source_unminimise() to restore the source if you wish to
+ * removed. Use [method@Source.unminimise] to restore the source if you wish to
  * use it again.
  *
  * Loaders should call this in response to the minimise signal on their output
@@ -629,7 +663,8 @@ vips_source_minimise(VipsSource *source)
  * Restore the source after minimisation. This is called at the start
  * of every source method, so loaders should not usually need this.
  *
- * See also: vips_source_minimise().
+ * ::: seealso
+ *     [method@Source.minimise].
  *
  * Returns: 0 on success, or -1 on error.
  */
@@ -687,7 +722,8 @@ vips_source_unminimise(VipsSource *source)
  *
  * Loaders should call this at the end of header read.
  *
- * See also: vips_source_unminimise().
+ * ::: seealso
+ *     [method@Source.unminimise].
  *
  * Returns: 0 on success, -1 on error.
  */
@@ -726,8 +762,7 @@ vips_source_print(VipsSource *source)
 	printf("  source->data = %p\n", source->data);
 	printf("  source->header_bytes = %p\n", source->header_bytes);
 	if (source->header_bytes)
-		printf("  source->header_bytes->len = %d\n",
-			source->header_bytes->len);
+		printf("  source->header_bytes->len = %d\n", source->header_bytes->len);
 	printf("  source->sniff = %p\n", source->sniff);
 	if (source->sniff)
 		printf("  source->sniff->len = %d\n", source->sniff->len);
@@ -744,7 +779,7 @@ vips_source_print(VipsSource *source)
  * Return the number of bytes actually read. If all bytes have been read from
  * the file, return 0.
  *
- * Arguments exactly as read(2).
+ * Arguments exactly as [`read()`](man:read(2)).
  *
  * Returns: the number of bytes read, 0 on end of file, -1 on error.
  */
@@ -786,15 +821,11 @@ vips_source_read(VipsSource *source, void *buffer, size_t length)
 		if (source->header_bytes &&
 			source->read_position < source->header_bytes->len) {
 			gint64 available = VIPS_MIN(length,
-				source->header_bytes->len -
-					source->read_position);
+				source->header_bytes->len - source->read_position);
 
-			VIPS_DEBUG_MSG("    %zd bytes from cache\n",
-				available);
+			VIPS_DEBUG_MSG("    %zd bytes from cache\n", available);
 			memcpy(buffer,
-				source->header_bytes->data +
-					source->read_position,
-				available);
+				source->header_bytes->data + source->read_position, available);
 			source->read_position += available;
 			buffer = (char *) buffer + available;
 			length -= available;
@@ -816,12 +847,10 @@ vips_source_read(VipsSource *source, void *buffer, size_t length)
 
 			VIPS_DEBUG_MSG("    calling class->read()\n");
 			bytes_read = class->read(source, buffer, length);
-			VIPS_DEBUG_MSG("    %zd bytes from read()\n",
-				bytes_read);
+			VIPS_DEBUG_MSG("    %zd bytes from read()\n", bytes_read);
 			if (bytes_read == -1) {
 				vips_error_system(errno,
-					vips_connection_nick(
-						VIPS_CONNECTION(source)),
+					vips_connection_nick(VIPS_CONNECTION(source)),
 					"%s", _("read error"));
 				return -1;
 			}
@@ -833,8 +862,7 @@ vips_source_read(VipsSource *source, void *buffer, size_t length)
 				source->is_pipe &&
 				!source->decode &&
 				bytes_read > 0)
-				g_byte_array_append(source->header_bytes,
-					buffer, bytes_read);
+				g_byte_array_append(source->header_bytes, buffer, bytes_read);
 
 			source->read_position += bytes_read;
 			total_read += bytes_read;
@@ -899,8 +927,7 @@ vips_source_pipe_read_to_position(VipsSource *source, gint64 target)
 			break;
 		}
 
-		if (target == -1 &&
-			vips__pipe_read_limit != -1 &&
+		if (vips__pipe_read_limit != -1 &&
 			source->read_position > vips__pipe_read_limit) {
 			vips_error(nick, "%s", _("pipe too long"));
 			return -1;
@@ -995,10 +1022,10 @@ vips_source_descriptor_to_memory(VipsSource *source)
  * @source: source to operate on
  *
  * Some sources can be efficiently mapped into memory.
- * You can still use vips_source_map() if this function returns %FALSE,
+ * You can still use [method@Source.map] if this function returns `FALSE`,
  * but it will be slow.
  *
- * Returns: %TRUE if the source can be efficiently mapped into memory.
+ * Returns: `TRUE` if the source can be efficiently mapped into memory.
  */
 gboolean
 vips_source_is_mappable(VipsSource *source)
@@ -1021,13 +1048,13 @@ vips_source_is_mappable(VipsSource *source)
  * @source: source to operate on
  *
  * Test if this source is a simple file with support for seek. Named pipes,
- * for example, will fail this test. If TRUE, you can use
- * vips_connection_filename() to find the filename.
+ * for example, will fail this test. If `TRUE`, you can use
+ * [method@Connection.filename] to find the filename.
  *
  * Use this to add basic source support for older loaders which can only work
  * on files.
  *
- * Returns: %TRUE if the source is a simple file.
+ * Returns: `TRUE` if the source is a simple file.
  */
 gboolean
 vips_source_is_file(VipsSource *source)
@@ -1045,20 +1072,20 @@ vips_source_is_file(VipsSource *source)
 /**
  * vips_source_map:
  * @source: source to operate on
- * @length_out: return the file length here, or NULL
+ * @length: return the file length here, or `NULL`
  *
  * Map the source entirely into memory and return a pointer to the
- * start. If @length_out is non-NULL, the source size is written to it.
+ * start. If @length is non-NULL, the source size is written to it.
  *
- * This operation can take a long time. Use vips_source_is_mappable() to
+ * This operation can take a long time. Use [method@Source.is_mappable] to
  * check if a source can be mapped efficiently.
  *
  * The pointer is valid for as long as @source is alive.
  *
- * Returns: a pointer to the start of the file contents, or NULL on error.
+ * Returns: a pointer to the start of the file contents, or `NULL` on error.
  */
 const void *
-vips_source_map(VipsSource *source, size_t *length_out)
+vips_source_map(VipsSource *source, size_t *length)
 {
 	VIPS_DEBUG_MSG("vips_source_map:\n");
 
@@ -1085,12 +1112,12 @@ vips_source_map(VipsSource *source, size_t *length_out)
 
 	/* We don't know the length and must read and assemble in chunks.
 	 */
-	if (!source->data &&
+	if (source->is_pipe &&
 		vips_source_pipe_read_to_position(source, -1))
 		return NULL;
 
-	if (length_out)
-		*length_out = source->length;
+	if (length)
+		*length = source->length;
 
 	SANITY(source);
 
@@ -1111,10 +1138,10 @@ vips_source_map_cb(void *a, VipsArea *area)
  * vips_source_map_blob:
  * @source: source to operate on
  *
- * Just like vips_source_map(), but return a #VipsBlob containing the
+ * Just like [method@Source.map], but return a [struct@Blob] containing the
  * pointer. @source will stay alive as long as the result is alive.
  *
- * Returns: a new #VipsBlob containing the data, or NULL on error.
+ * Returns: a new [struct@Blob] containing the data, or `NULL` on error.
  */
 VipsBlob *
 vips_source_map_blob(VipsSource *source)
@@ -1124,8 +1151,7 @@ vips_source_map_blob(VipsSource *source)
 	VipsBlob *blob;
 
 	if (!(buf = vips_source_map(source, &len)) ||
-		!(blob = vips_blob_new((VipsCallbackFn) vips_source_map_cb,
-			  buf, len)))
+		!(blob = vips_blob_new((VipsCallbackFn) vips_source_map_cb, buf, len)))
 		return NULL;
 
 	/* The source must stay alive until the blob is done.
@@ -1143,7 +1169,7 @@ vips_source_map_blob(VipsSource *source)
  * @whence: seek relative to this point
  *
  * Move the file read position. You can't call this after pixel decode starts.
- * The arguments are exactly as lseek(2).
+ * The arguments are exactly as [`lseek()`](man:lseek(2)).
  *
  * Returns: the new file position, or -1 on error.
  */
@@ -1156,8 +1182,7 @@ vips_source_seek(VipsSource *source, gint64 offset, int whence)
 	gint64 new_pos;
 
 	VIPS_DEBUG_MSG(
-		"vips_source_seek: offset = %" G_GINT64_FORMAT
-		", whence = %d\n",
+		"vips_source_seek: offset = %" G_GINT64_FORMAT ", whence = %d\n",
 		offset, whence);
 
 	if (vips_source_unminimise(source) ||
@@ -1223,10 +1248,8 @@ vips_source_seek(VipsSource *source, gint64 offset, int whence)
 	/* Don't allow out of range seeks.
 	 */
 	if (new_pos < 0 ||
-		(source->length != -1 &&
-			new_pos > source->length)) {
-		vips_error(nick,
-			_("bad seek to %" G_GINT64_FORMAT), new_pos);
+		(source->length != -1 && new_pos > source->length)) {
+		vips_error(nick, _("bad seek to %" G_GINT64_FORMAT), new_pos);
 		return -1;
 	}
 
@@ -1333,8 +1356,7 @@ vips_source_sniff_at_most(VipsSource *source,
 	while (read_position < length) {
 		gint64 bytes_read;
 
-		bytes_read = vips_source_read(source, q,
-			length - read_position);
+		bytes_read = vips_source_read(source, q, length - read_position);
 		if (bytes_read == -1)
 			return -1;
 		if (bytes_read == 0)
@@ -1357,9 +1379,9 @@ vips_source_sniff_at_most(VipsSource *source,
  * @length: number of bytes to sniff
  *
  * Return a pointer to the first few bytes of the file. If the file is too
- * short, return NULL.
+ * short, return `NULL`.
  *
- * Returns: a pointer to the bytes at the start of the file, or NULL on error.
+ * Returns: a pointer to the bytes at the start of the file, or `NULL` on error.
  */
 unsigned char *
 vips_source_sniff(VipsSource *source, size_t length)
