@@ -57,25 +57,6 @@
 #include <vips/internal.h>
 #include <vips/debug.h>
 
-/**
- * SECTION: basic
- * @short_description: a few typedefs used everywhere
- * @stability: Stable
- * @include: vips/vips.h
- *
- * A few simple typedefs used by VIPS.
- */
-
-/**
- * SECTION: type
- * @short_description: basic types
- * @stability: Stable
- * @see_also: <link linkend="libvips-header">header</link>
- * @include: vips/vips.h
- *
- * A selection of %GType definitions used by VIPS.
- */
-
 /* A very simple boxed type for testing. Just an int.
  *
  * You can manipulate this thing from Python (for example) with:
@@ -93,7 +74,7 @@
  * vips_thing_new:
  * @i:
  *
- * Returns: (transfer full): a new #VipsThing.
+ * Returns: (transfer full): a new [struct@Thing].
  */
 VipsThing *
 vips_thing_new(int i)
@@ -137,7 +118,7 @@ static GSList *vips_area_all = NULL;
 VipsArea *
 vips_area_copy(VipsArea *area)
 {
-	g_mutex_lock(area->lock);
+	g_mutex_lock(&area->lock);
 
 	g_assert(area->count > 0);
 
@@ -147,7 +128,7 @@ vips_area_copy(VipsArea *area)
 	printf("vips_area_copy: %p count = %d\n", area, area->count);
 #endif /*DEBUG*/
 
-	g_mutex_unlock(area->lock);
+	g_mutex_unlock(&area->lock);
 
 	return area;
 }
@@ -175,7 +156,7 @@ vips_area_free(VipsArea *area)
 void
 vips_area_unref(VipsArea *area)
 {
-	g_mutex_lock(area->lock);
+	g_mutex_lock(&area->lock);
 
 	g_assert(area->count > 0);
 
@@ -186,35 +167,35 @@ vips_area_unref(VipsArea *area)
 #endif /*DEBUG*/
 
 	if (vips__leak) {
-		g_mutex_lock(vips__global_lock);
+		g_mutex_lock(&vips__global_lock);
 		g_assert(g_slist_find(vips_area_all, area));
-		g_mutex_unlock(vips__global_lock);
+		g_mutex_unlock(&vips__global_lock);
 	}
 
 	if (area->count == 0) {
 		vips_area_free(area);
 
-		g_mutex_unlock(area->lock);
+		g_mutex_unlock(&area->lock);
 
-		VIPS_FREEF(vips_g_mutex_free, area->lock);
+		g_mutex_clear(&area->lock);
 
 		if (vips__leak) {
-			g_mutex_lock(vips__global_lock);
+			g_mutex_lock(&vips__global_lock);
 			vips_area_all = g_slist_remove(vips_area_all, area);
-			g_mutex_unlock(vips__global_lock);
+			g_mutex_unlock(&vips__global_lock);
 		}
 
 		g_free(area);
 
 #ifdef DEBUG
-		g_mutex_lock(vips__global_lock);
+		g_mutex_lock(&vips__global_lock);
 		printf("vips_area_unref: free .. total = %d\n",
 			g_slist_length(vips_area_all));
-		g_mutex_unlock(vips__global_lock);
+		g_mutex_unlock(&vips__global_lock);
 #endif /*DEBUG*/
 	}
 	else
-		g_mutex_unlock(area->lock);
+		g_mutex_unlock(&area->lock);
 }
 
 /* autoptr needs typed versions of functions for free.
@@ -237,17 +218,19 @@ VipsArrayImage_unref(VipsArrayImage *array)
  * @data: (transfer full): data will be freed with this function
  *
  * A VipsArea wraps a chunk of memory. It adds reference counting and a free
- * function. It also keeps a count and a %GType, so the area can be an array.
+ * function. It also keeps a count and a [alias@GObject.Type], so the area can
+ * be an array.
  *
  * This type is used for things like passing an array of double or an array of
- * #VipsObject pointers to operations, and for reference-counted immutable
+ * [class@Object] pointers to operations, and for reference-counted immutable
  * strings.
  *
- * Initial count == 1, so _unref() after attaching somewhere.
+ * Initial count == 1, so [method@Area.unref] after attaching somewhere.
  *
- * See also: vips_area_unref().
+ * ::: seealso
+ *     [method@Area.unref].
  *
- * Returns: (transfer full): the new #VipsArea.
+ * Returns: (transfer full): the new [struct@Area].
  */
 VipsArea *
 vips_area_new(VipsCallbackFn free_fn, void *data)
@@ -256,7 +239,7 @@ vips_area_new(VipsCallbackFn free_fn, void *data)
 
 	area = g_new(VipsArea, 1);
 	area->count = 1;
-	area->lock = vips_g_mutex_new();
+	g_mutex_init(&area->lock);
 	area->length = 0;
 	area->data = data;
 	area->free_fn = free_fn;
@@ -264,17 +247,17 @@ vips_area_new(VipsCallbackFn free_fn, void *data)
 	area->sizeof_type = 0;
 
 	if (vips__leak) {
-		g_mutex_lock(vips__global_lock);
+		g_mutex_lock(&vips__global_lock);
 		vips_area_all = g_slist_prepend(vips_area_all, area);
-		g_mutex_unlock(vips__global_lock);
+		g_mutex_unlock(&vips__global_lock);
 	}
 
 #ifdef DEBUG
-	g_mutex_lock(vips__global_lock);
+	g_mutex_lock(&vips__global_lock);
 	printf("vips_area_new: %p count = %d (%d in total)\n",
 		area, area->count,
 		g_slist_length(vips_area_all));
-	g_mutex_unlock(vips__global_lock);
+	g_mutex_unlock(&vips__global_lock);
 #endif /*DEBUG*/
 
 	return area;
@@ -307,16 +290,17 @@ vips__type_leak(void)
 
 /**
  * vips_area_new_array:
- * @type: %GType of elements to store
- * @sizeof_type: sizeof() an element in the array
+ * @type: [alias@GObject.Type] of elements to store
+ * @sizeof_type: `sizeof()` an element in the array
  * @n: number of elements in the array
  *
- * An area which holds an array of elements of some %GType. To set values for
- * the elements, get the pointer and write.
+ * An area which holds an array of elements of some [alias@GObject.Type].
+ * To set values for the elements, get the pointer and write.
  *
- * See also: vips_area_unref().
+ * ::: seealso
+ *     [method@Area.unref].
  *
- * Returns: (transfer full): the new #VipsArea.
+ * Returns: (transfer full): the new [struct@Area].
  */
 VipsArea *
 vips_area_new_array(GType type, size_t sizeof_type, int n)
@@ -352,15 +336,17 @@ vips_area_free_array_object(GObject **array, VipsArea *area)
  * vips_area_new_array_object: (constructor)
  * @n: number of elements in the array
  *
- * An area which holds an array of %GObject s. See vips_area_new_array(). When
- * the area is freed, each %GObject will be unreffed.
+ * An area which holds an array of [class@GObject.Object] s. See
+ * [ctor@Area.new_array]. When the area is freed, each [class@GObject.Object]
+ * will be unreffed.
  *
- * Add an extra NULL element at the end, handy for eg.
- * vips_image_pipeline_array() etc.
+ * Add an extra `NULL` element at the end, handy for eg.
+ * [func@Image.pipeline_array] etc.
  *
- * See also: vips_area_unref().
+ * ::: seealso
+ *     [method@Area.unref].
  *
- * Returns: (transfer full): the new #VipsArea.
+ * Returns: (transfer full): the new [struct@Area].
  */
 VipsArea *
 vips_area_new_array_object(int n)
@@ -381,15 +367,15 @@ vips_area_new_array_object(int n)
 
 /**
  * vips_area_get_data:
- * @area: #VipsArea to fetch from
+ * @area: [struct@Area] to fetch from
  * @length: (out) (optional): optionally return length in bytes here
  * @n: (out) (optional): optionally return number of elements here
  * @type: (out) (optional): optionally return element type here
- * @sizeof_type: (out) (optional): optionally return sizeof() element type here
+ * @sizeof_type: (out) (optional): optionally return `sizeof()` element type here
  *
  * Return the data pointer plus optionally the length in bytes of an area,
- * the number of elements, the %GType of each element and the sizeof() each
- * element.
+ * the number of elements, the [alias@GObject.Type] of each element and the
+ * `sizeof()` each element.
  *
  * Returns: (transfer none): The pointer held by @area.
  */
@@ -439,15 +425,13 @@ G_DEFINE_BOXED_TYPE_WITH_CODE(VipsArea, vips_area,
 static void
 transform_int_save_string(const GValue *src_value, GValue *dest_value)
 {
-	vips_value_set_save_stringf(dest_value,
-		"%d", g_value_get_int(src_value));
+	vips_value_set_save_stringf(dest_value, "%d", g_value_get_int(src_value));
 }
 
 static void
 transform_save_string_int(const GValue *src_value, GValue *dest_value)
 {
-	g_value_set_int(dest_value,
-		atoi(vips_value_get_save_string(src_value)));
+	g_value_set_int(dest_value, atoi(vips_value_get_save_string(src_value)));
 }
 
 static void
@@ -457,8 +441,7 @@ transform_double_save_string(const GValue *src_value, GValue *dest_value)
 
 	/* Need to be locale independent.
 	 */
-	g_ascii_dtostr(buf, G_ASCII_DTOSTR_BUF_SIZE,
-		g_value_get_double(src_value));
+	g_ascii_dtostr(buf, G_ASCII_DTOSTR_BUF_SIZE, g_value_get_double(src_value));
 	vips_value_set_save_string(dest_value, buf);
 }
 
@@ -466,8 +449,7 @@ static void
 transform_save_string_double(const GValue *src_value, GValue *dest_value)
 {
 	g_value_set_double(dest_value,
-		g_ascii_strtod(vips_value_get_save_string(src_value),
-			NULL));
+		g_ascii_strtod(vips_value_get_save_string(src_value), NULL));
 }
 
 static void
@@ -477,8 +459,7 @@ transform_float_save_string(const GValue *src_value, GValue *dest_value)
 
 	/* Need to be locale independent.
 	 */
-	g_ascii_dtostr(buf, G_ASCII_DTOSTR_BUF_SIZE,
-		g_value_get_float(src_value));
+	g_ascii_dtostr(buf, G_ASCII_DTOSTR_BUF_SIZE, g_value_get_float(src_value));
 	vips_value_set_save_string(dest_value, buf);
 }
 
@@ -486,8 +467,7 @@ static void
 transform_save_string_float(const GValue *src_value, GValue *dest_value)
 {
 	g_value_set_float(dest_value,
-		g_ascii_strtod(vips_value_get_save_string(src_value),
-			NULL));
+		g_ascii_strtod(vips_value_get_save_string(src_value), NULL));
 }
 
 /* Save meta fields to the header. We have a new string type for header fields
@@ -525,7 +505,9 @@ G_DEFINE_BOXED_TYPE_WITH_CODE(VipsSaveString, vips_save_string,
 							  g_value_register_transform_func(
 								  g_define_type_id,
 								  G_TYPE_FLOAT,
-								  transform_save_string_float);)
+								  transform_save_string_float);
+
+)
 
 /* Transform a refstring to a G_TYPE_STRING and back.
  */
@@ -568,9 +550,11 @@ transform_save_string_ref_string(const GValue *src_value, GValue *dest_value)
  *
  * Strings must be valid utf-8; use blob for binary data.
  *
- * See also: vips_area_unref().
+ * ::: seealso
+ *     [method@Area.unref].
  *
- * Returns: (transfer full) (nullable): the new #VipsRefString, or NULL on error.
+ * Returns: (transfer full) (nullable): the new [struct@RefString], or `NULL` on
+ * error.
  */
 VipsRefString *
 vips_ref_string_new(const char *str)
@@ -590,13 +574,14 @@ vips_ref_string_new(const char *str)
 
 /**
  * vips_ref_string_get:
- * @refstr: the #VipsRefString to fetch from
+ * @refstr: the [struct@RefString] to fetch from
  * @length: (out) (optional): return length here, optionally
  *
  * Get a pointer to the private string inside a refstr. Handy for language
  * bindings.
  *
- * See also: vips_value_get_ref_string().
+ * ::: seealso
+ *     [func@value_get_ref_string].
  *
  * Returns: (transfer none): The C string held by @refstr.
  */
@@ -637,16 +622,17 @@ G_DEFINE_BOXED_TYPE_WITH_CODE(VipsRefString, vips_ref_string,
  * @data: (array length=length) (element-type guint8) (transfer full): data to store
  * @length: number of bytes in @data
  *
- * Like vips_area_new(), but track a length as well. The returned #VipsBlob
- * takes ownership of @data and will free it with @free_fn. Pass %NULL for
+ * Like [ctor@Area.new], but track a length as well. The returned [struct@Blob]
+ * takes ownership of @data and will free it with @free_fn. Pass `NULL` for
  * @free_fn to not transfer ownership.
  *
  * An area of mem with a free func and a length (some sort of binary object,
  * like an ICC profile).
  *
- * See also: vips_area_unref().
+ * ::: seealso
+ *     [method@Area.unref].
  *
- * Returns: (transfer full): the new #VipsBlob.
+ * Returns: (transfer full): the new [struct@Blob].
  */
 VipsBlob *
 vips_blob_new(VipsCallbackFn free_fn, const void *data, size_t length)
@@ -664,12 +650,13 @@ vips_blob_new(VipsCallbackFn free_fn, const void *data, size_t length)
  * @data: (array length=length) (element-type guint8) (transfer none): data to store
  * @length: number of bytes in @data
  *
- * Like vips_blob_new(), but take a copy of the data. Useful for bindings
+ * Like [ctor@Blob.new], but take a copy of the data. Useful for bindings
  * which struggle with callbacks.
  *
- * See also: vips_blob_new().
+ * ::: seealso
+ *     [ctor@Blob.new].
  *
- * Returns: (transfer full): the new #VipsBlob.
+ * Returns: (transfer full): the new [struct@Blob].
  */
 VipsBlob *
 vips_blob_copy(const void *data, size_t length)
@@ -687,12 +674,13 @@ vips_blob_copy(const void *data, size_t length)
 
 /**
  * vips_blob_get:
- * @blob: #VipsBlob to fetch from
+ * @blob: [struct@Blob] to fetch from
  * @length: return number of bytes of data
  *
- * Get the data from a #VipsBlob.
+ * Get the data from a [struct@Blob].
  *
- * See also: vips_blob_new().
+ * ::: seealso
+ *     [ctor@Blob.new].
  *
  * Returns: (array length=length) (element-type guint8) (transfer none): the
  * data
@@ -706,7 +694,7 @@ vips_blob_get(VipsBlob *blob, size_t *length)
 
 /**
  * vips_blob_set:
- * @blob: #VipsBlob to set
+ * @blob: [struct@Blob] to set
  * @free_fn: (scope async) (allow-none): @data will be freed with this function
  * @data: (array length=length) (element-type guint8) (transfer full): data to store
  * @length: number of bytes in @data
@@ -716,7 +704,8 @@ vips_blob_get(VipsBlob *blob, size_t *length)
  * It's sometimes useful to be able to create blobs as empty and then fill
  * them later.
  *
- * See also: vips_blob_new().
+ * ::: seealso
+ *     [ctor@Blob.new].
  */
 void
 vips_blob_set(VipsBlob *blob,
@@ -724,7 +713,7 @@ vips_blob_set(VipsBlob *blob,
 {
 	VipsArea *area = VIPS_AREA(blob);
 
-	g_mutex_lock(area->lock);
+	g_mutex_lock(&area->lock);
 
 	vips_area_free(area);
 
@@ -732,7 +721,7 @@ vips_blob_set(VipsBlob *blob,
 	area->length = length;
 	area->data = (void *) data;
 
-	g_mutex_unlock(area->lock);
+	g_mutex_unlock(&area->lock);
 }
 
 /* Transform a blob to a G_TYPE_STRING.
@@ -813,11 +802,12 @@ G_DEFINE_BOXED_TYPE_WITH_CODE(VipsBlob, vips_blob,
  * @n: number of ints
  *
  * Allocate a new array of ints and copy @array into it. Free with
- * vips_area_unref().
+ * [method@Area.unref].
  *
- * See also: #VipsArea.
+ * ::: seealso
+ *     [struct@Area].
  *
- * Returns: (transfer full): A new #VipsArrayInt.
+ * Returns: (transfer full): A new [struct@ArrayInt].
  */
 VipsArrayInt *
 vips_array_int_new(const int *array, int n)
@@ -838,11 +828,12 @@ vips_array_int_new(const int *array, int n)
  * @...: list of int arguments
  *
  * Allocate a new array of @n ints and copy @... into it. Free with
- * vips_area_unref().
+ * [method@Area.unref].
  *
- * See also: vips_array_int_new()
+ * ::: seealso
+ *     [ctor@ArrayInt.new]
  *
- * Returns: (transfer full): A new #VipsArrayInt.
+ * Returns: (transfer full): A new [struct@ArrayInt].
  */
 VipsArrayInt *
 vips_array_int_newv(int n, ...)
@@ -865,10 +856,10 @@ vips_array_int_newv(int n, ...)
 
 /**
  * vips_array_int_get:
- * @array: the #VipsArrayInt to fetch from
+ * @array: the [struct@ArrayInt] to fetch from
  * @n: length of array
  *
- * Fetch an int array from a #VipsArrayInt. Useful for language bindings.
+ * Fetch an int array from a [struct@ArrayInt]. Useful for language bindings.
  *
  * Returns: (array length=n) (transfer none): array of int
  */
@@ -1058,11 +1049,12 @@ G_DEFINE_BOXED_TYPE_WITH_CODE(VipsArrayInt, vips_array_int,
  * @n: number of doubles
  *
  * Allocate a new array of doubles and copy @array into it. Free with
- * vips_area_unref().
+ * [method@Area.unref].
  *
- * See also: #VipsArea.
+ * ::: seealso
+ *     [struct@Area].
  *
- * Returns: (transfer full): A new #VipsArrayDouble.
+ * Returns: (transfer full): A new [struct@ArrayDouble].
  */
 VipsArrayDouble *
 vips_array_double_new(const double *array, int n)
@@ -1083,11 +1075,12 @@ vips_array_double_new(const double *array, int n)
  * @...: list of double arguments
  *
  * Allocate a new array of @n doubles and copy @... into it. Free with
- * vips_area_unref().
+ * [method@Area.unref].
  *
- * See also: vips_array_double_new()
+ * ::: seealso
+ *     [ctor@ArrayDouble.new]
  *
- * Returns: (transfer full): A new #VipsArrayDouble.
+ * Returns: (transfer full): A new [struct@ArrayDouble].
  */
 VipsArrayDouble *
 vips_array_double_newv(int n, ...)
@@ -1110,10 +1103,10 @@ vips_array_double_newv(int n, ...)
 
 /**
  * vips_array_double_get:
- * @array: the #VipsArrayDouble to fetch from
+ * @array: the [struct@ArrayDouble] to fetch from
  * @n: length of array
  *
- * Fetch a double array from a #VipsArrayDouble. Useful for language bindings.
+ * Fetch a double array from a [struct@ArrayDouble]. Useful for language bindings.
  *
  * Returns: (array length=n) (transfer none): array of double
  */
@@ -1130,8 +1123,11 @@ vips_array_double_get(VipsArrayDouble *array, int *n)
 	return VIPS_ARRAY_ADDR(array, 0);
 }
 
+typedef void (*SetStringFn)(GValue *dest_value, const char *str);
+
 static void
-transform_array_double_g_string(const GValue *src_value, GValue *dest_value)
+array_double_to_value(const GValue *src_value,
+	GValue *dest_value, SetStringFn set)
 {
 	int n;
 	double *array;
@@ -1141,20 +1137,32 @@ transform_array_double_g_string(const GValue *src_value, GValue *dest_value)
 	int i;
 
 	if ((array = vips_value_get_array_double(src_value, &n)))
-		for (i = 0; i < n; i++)
-			/* Use space as a separator since ',' may be a decimal
-			 * point in this locale.
-			 */
-			vips_buf_appendf(&buf, "%g ", array[i]);
+		for (i = 0; i < n; i++) {
+			// locale independent
+			vips_buf_appendg(&buf, array[i]);
+			vips_buf_appends(&buf, " ");
+		}
 
-	g_value_set_string(dest_value, vips_buf_all(&buf));
+	set(dest_value, vips_buf_all(&buf));
+}
+
+static void
+transform_array_double_g_string(const GValue *src_value, GValue *dest_value)
+{
+	array_double_to_value(src_value, dest_value, g_value_set_string);
+}
+
+static void
+transform_array_double_save_string(const GValue *src_value, GValue *dest_value)
+{
+	array_double_to_value(src_value, dest_value, vips_value_set_save_string);
 }
 
 /* It'd be great to be able to write a generic string->array function, but
- * it doesn't seem possible.
+ * it doesn't seem to be possible.
  */
 static void
-transform_g_string_array_double(const GValue *src_value, GValue *dest_value)
+string_to_array_double(const char *input, GValue *dest_value)
 {
 	char *str;
 	int n;
@@ -1162,10 +1170,7 @@ transform_g_string_array_double(const GValue *src_value, GValue *dest_value)
 	int i;
 	double *array;
 
-	/* Walk the string to get the number of elements.
-	 * We need a copy of the string, since we insert \0 during scan.
-	 */
-	str = g_value_dup_string(src_value);
+	str = g_strdup(input);
 
 	n = 0;
 	for (p = str; (q = vips_break_token(p, "\t;, ")); p = q)
@@ -1176,13 +1181,15 @@ transform_g_string_array_double(const GValue *src_value, GValue *dest_value)
 	vips_value_set_array_double(dest_value, NULL, n);
 	array = vips_value_get_array_double(dest_value, NULL);
 
-	str = g_value_dup_string(src_value);
+	str = g_strdup(input);
 
 	i = 0;
 	for (p = str; (q = vips_break_token(p, "\t; ")); p = q) {
+		// this is locale-independent
 		if (vips_strtod(p, &array[i])) {
 			/* Set array to length zero to indicate an error.
 			 */
+			g_warning("bad argument: not a numeric value \"%s\"", p);
 			vips_value_set_array_double(dest_value, NULL, 0);
 			g_free(str);
 			return;
@@ -1194,7 +1201,19 @@ transform_g_string_array_double(const GValue *src_value, GValue *dest_value)
 	g_free(str);
 }
 
-/* We need a arraydouble, we have a double, make a one-element array.
+static void
+transform_g_string_array_double(const GValue *src_value, GValue *dest_value)
+{
+	string_to_array_double(g_value_get_string(src_value), dest_value);
+}
+
+static void
+transform_save_string_array_double(const GValue *src_value, GValue *dest_value)
+{
+	string_to_array_double(vips_value_get_save_string(src_value), dest_value);
+}
+
+/* We need an arraydouble, we have a double, make a one-element array.
  */
 static void
 transform_double_array_double(const GValue *src_value, GValue *dest_value)
@@ -1241,6 +1260,7 @@ transform_double_enum(const GValue *src_value, GValue *dest_value)
 G_DEFINE_BOXED_TYPE_WITH_CODE(VipsArrayDouble, vips_array_double,
 							  (GBoxedCopyFunc) vips_area_copy,
 							  (GBoxedFreeFunc) vips_area_unref,
+
 							  g_value_register_transform_func(
 								  g_define_type_id,
 								  G_TYPE_STRING,
@@ -1251,6 +1271,16 @@ G_DEFINE_BOXED_TYPE_WITH_CODE(VipsArrayDouble, vips_array_double,
 								  g_define_type_id,
 								  transform_g_string_array_double);
 
+							  g_value_register_transform_func(
+								  g_define_type_id,
+								  VIPS_TYPE_SAVE_STRING,
+								  transform_array_double_save_string);
+
+							  g_value_register_transform_func(
+								  VIPS_TYPE_SAVE_STRING,
+								  g_define_type_id,
+								  transform_save_string_array_double);
+
 							  g_value_register_transform_func(
 								  G_TYPE_DOUBLE,
 								  g_define_type_id,
@@ -1264,26 +1294,29 @@ G_DEFINE_BOXED_TYPE_WITH_CODE(VipsArrayDouble, vips_array_double,
 							  g_value_register_transform_func(
 								  G_TYPE_DOUBLE,
 								  G_TYPE_ENUM,
-								  transform_double_enum);)
+								  transform_double_enum);
+
+)
 
 /**
  * vips_array_image_new: (constructor)
- * @array: (array length=n): array of #VipsImage
+ * @array: (array length=n): array of [class@Image]
  * @n: number of images
  *
  * Allocate a new array of images and copy @array into it. Free with
- * vips_area_unref().
+ * [method@Area.unref].
  *
  * The images will all be reffed by this function. They
  * will be automatically unreffed for you by
- * vips_area_unref().
+ * [method@Area.unref].
  *
- * Add an extra NULL element at the end, handy for eg.
- * vips_image_pipeline_array() etc.
+ * Add an extra `NULL` element at the end, handy for eg.
+ * [func@Image.pipeline_array] etc.
  *
- * See also: #VipsArea.
+ * ::: seealso
+ *     [struct@Area].
  *
- * Returns: (transfer full): A new #VipsArrayImage.
+ * Returns: (transfer full): A new [struct@ArrayImage].
  */
 VipsArrayImage *
 vips_array_image_new(VipsImage **array, int n)
@@ -1307,21 +1340,22 @@ vips_array_image_new(VipsImage **array, int n)
 /**
  * vips_array_image_newv: (constructor)
  * @n: number of images
- * @...: list of #VipsImage arguments
+ * @...: list of [class@Image] arguments
  *
- * Allocate a new array of @n #VipsImage and copy @... into it. Free with
- * vips_area_unref().
+ * Allocate a new array of @n [class@Image] and copy @... into it. Free with
+ * [method@Area.unref].
  *
  * The images will all be reffed by this function. They
  * will be automatically unreffed for you by
- * vips_area_unref().
+ * [method@Area.unref].
  *
- * Add an extra NULL element at the end, handy for eg.
- * vips_image_pipeline_array() etc.
+ * Add an extra `NULL` element at the end, handy for eg.
+ * [func@Image.pipeline_array] etc.
  *
- * See also: vips_array_image_new()
+ * ::: seealso
+ *     [ctor@ArrayImage.new]
  *
- * Returns: (transfer full): A new #VipsArrayImage.
+ * Returns: (transfer full): A new [struct@ArrayImage].
  */
 VipsArrayImage *
 vips_array_image_newv(int n, ...)
@@ -1395,12 +1429,13 @@ vips_array_image_new_from_string(const char *string, VipsAccess access)
  * vips_array_image_empty: (constructor)
  *
  * Make an empty image array.
- * Handy with vips_array_image_add() for bindings
+ * Handy with [method@ArrayImage.append] for bindings
  * which can't handle object array arguments.
  *
- * See also: vips_array_image_add().
+ * ::: seealso
+ *     [method@ArrayImage.append].
  *
- * Returns: (transfer full): A new #VipsArrayImage.
+ * Returns: (transfer full): A new [struct@ArrayImage].
  */
 VipsArrayImage *
 vips_array_image_empty(void)
@@ -1413,14 +1448,15 @@ vips_array_image_empty(void)
  * @array: (transfer none): append to this
  * @image: add this
  *
- * Make a new #VipsArrayImage, one larger than @array, with @image appended
+ * Make a new [struct@ArrayImage], one larger than @array, with @image appended
  * to the end.
- * Handy with vips_array_image_empty() for bindings
+ * Handy with [ctor@ArrayImage.empty] for bindings
  * which can't handle object array arguments.
  *
- * See also: vips_array_image_empty().
+ * ::: seealso
+ *     [ctor@ArrayImage.empty].
  *
- * Returns: (transfer full): A new #VipsArrayImage.
+ * Returns: (transfer full): A new [struct@ArrayImage].
  */
 VipsArrayImage *
 vips_array_image_append(VipsArrayImage *array, VipsImage *image)
@@ -1450,12 +1486,12 @@ vips_array_image_append(VipsArrayImage *array, VipsImage *image)
 
 /**
  * vips_array_image_get:
- * @array: the #VipsArrayImage to fetch from
+ * @array: the [struct@ArrayImage] to fetch from
  * @n: length of array
  *
- * Fetch an image array from a #VipsArrayImage. Useful for language bindings.
+ * Fetch an image array from a [struct@ArrayImage]. Useful for language bindings.
  *
- * Returns: (array length=n) (transfer none): array of #VipsImage
+ * Returns: (array length=n) (transfer none): array of [class@Image]
  */
 VipsImage **
 vips_array_image_get(VipsArrayImage *array, int *n)
@@ -1578,8 +1614,8 @@ vips_value_set_save_string(GValue *value, const char *str)
 /**
  * vips_value_set_save_stringf:
  * @value: (out): GValue to set
- * @fmt: printf()-style format string
- * @...: arguments to printf()-formatted @fmt
+ * @fmt: `printf()`-style format string
+ * @...: arguments to `printf()`-formatted @fmt
  *
  * Generates a string and copies it into @value.
  */
@@ -1600,10 +1636,10 @@ vips_value_set_save_stringf(GValue *value, const char *fmt, ...)
 
 /**
  * vips_value_get_ref_string:
- * @value: %GValue to get from
+ * @value: [struct@GObject.Value] to get from
  * @length: (out) (optional): return length here, optionally
  *
- * Get the C string held internally by the %GValue.
+ * Get the C string held internally by the [struct@GObject.Value].
  *
  * Returns: (transfer none): The C string held by @value.
  */
@@ -1615,14 +1651,14 @@ vips_value_get_ref_string(const GValue *value, size_t *length)
 
 /**
  * vips_value_set_ref_string:
- * @value: (out): %GValue to set
+ * @value: (out): [struct@GObject.Value] to set
  * @str: C string to copy into the GValue
  *
  * Copies the C string @str into @value.
  *
  * vips_ref_string are immutable C strings that are copied between images by
  * copying reference-counted pointers, making them much more efficient than
- * regular %GValue strings.
+ * regular [struct@GObject.Value] strings.
  *
  * @str should be a valid utf-8 string.
  */
@@ -1654,7 +1690,8 @@ vips_value_set_ref_string(GValue *value, const char *str)
  * are saved to VIPS files for you coded as base64 inside the XML. They are
  * copied by copying reference-counted pointers.
  *
- * See also: vips_value_get_blob()
+ * ::: seealso
+ *     [func@value_get_blob]
  */
 void
 vips_value_set_blob(GValue *value,
@@ -1676,13 +1713,14 @@ vips_value_set_blob(GValue *value,
  * memory
  * @length: length of memory area
  *
- * Just like vips_value_set_blob(), but when
+ * Just like [func@value_set_blob], but when
  * @value is freed, @data will be
- * freed with g_free().
+ * freed with [func@GLib.free].
  *
  * This can be easier to call for language bindings.
  *
- * See also: vips_value_set_blob()
+ * ::: seealso
+ *     [func@value_set_blob]
  */
 void
 vips_value_set_blob_free(GValue *value, void *data, size_t length)
@@ -1707,7 +1745,8 @@ vips_value_set_blob_free(GValue *value, void *data, size_t length)
  * are saved to VIPS files for you coded as base64 inside the XML. They are
  * copied by copying reference-counted pointers.
  *
- * See also: vips_value_set_blob()
+ * ::: seealso
+ *     [func@value_set_blob]
  *
  * Returns: (transfer none) (array length=length) (element-type guint8): The pointer held
  * by @value.
@@ -1720,7 +1759,7 @@ vips_value_get_blob(const GValue *value, size_t *length)
 
 /**
  * vips_value_set_array:
- * @value: (out): %GValue to set
+ * @value: (out): [struct@GObject.Value] to set
  * @n: number of elements
  * @type: the type of each element
  * @sizeof_type: the sizeof each element
@@ -1742,7 +1781,7 @@ vips_value_set_array(GValue *value, int n, GType type, size_t sizeof_type)
 
 /**
  * vips_value_get_array:
- * @value: %GValue to get from
+ * @value: [struct@GObject.Value] to get from
  * @n: (out) (optional): return the number of elements here, optionally
  * @type: (out) (optional): return the type of each element here, optionally
  * @sizeof_type: (out) (optional): return the sizeof each element here, optionally
@@ -1751,7 +1790,8 @@ vips_value_set_array(GValue *value, int n, GType type, size_t sizeof_type)
  * Optionally return the other properties of the array in @n, @type,
  * @sizeof_type.
  *
- * See also: vips_value_set_array().
+ * ::: seealso
+ *     [func@value_set_array].
  *
  * Returns: (transfer none): The array address.
  */
@@ -1779,13 +1819,14 @@ vips_value_get_array(const GValue *value,
 
 /**
  * vips_value_get_array_int:
- * @value: %GValue to get from
+ * @value: [struct@GObject.Value] to get from
  * @n: (out) (optional): return the number of elements here, optionally
  *
  * Return the start of the array of ints held by @value.
  * optionally return the number of elements in @n.
  *
- * See also: vips_array_int_new().
+ * ::: seealso
+ *     [ctor@ArrayInt.new].
  *
  * Returns: (transfer none) (array length=n): The array address.
  */
@@ -1797,13 +1838,14 @@ vips_value_get_array_int(const GValue *value, int *n)
 
 /**
  * vips_value_set_array_int:
- * @value: %GValue to get from
+ * @value: [struct@GObject.Value] to get from
  * @array: (array length=n) (allow-none): array of ints
  * @n: the number of elements
  *
  * Set @value to hold a copy of @array. Pass in the array length in @n.
  *
- * See also: vips_array_int_get().
+ * ::: seealso
+ *     [method@ArrayInt.get].
  */
 void
 vips_value_set_array_int(GValue *value, const int *array, int n)
@@ -1820,13 +1862,14 @@ vips_value_set_array_int(GValue *value, const int *array, int n)
 
 /**
  * vips_value_get_array_double:
- * @value: %GValue to get from
+ * @value: [struct@GObject.Value] to get from
  * @n: (out) (optional): return the number of elements here, optionally
  *
  * Return the start of the array of doubles held by @value.
  * optionally return the number of elements in @n.
  *
- * See also: vips_array_double_new().
+ * ::: seealso
+ *     [ctor@ArrayDouble.new].
  *
  * Returns: (transfer none) (array length=n): The array address.
  */
@@ -1838,13 +1881,14 @@ vips_value_get_array_double(const GValue *value, int *n)
 
 /**
  * vips_value_set_array_double:
- * @value: %GValue to get from
+ * @value: [struct@GObject.Value] to get from
  * @array: (array length=n) (allow-none): array of doubles
  * @n: the number of elements
  *
  * Set @value to hold a copy of @array. Pass in the array length in @n.
  *
- * See also: vips_array_double_get().
+ * ::: seealso
+ *     [method@ArrayDouble.get].
  */
 void
 vips_value_set_array_double(GValue *value, const double *array, int n)
@@ -1861,13 +1905,14 @@ vips_value_set_array_double(GValue *value, const double *array, int n)
 
 /**
  * vips_value_get_array_image:
- * @value: %GValue to get from
+ * @value: [struct@GObject.Value] to get from
  * @n: (out) (optional): return the number of elements here, optionally
  *
  * Return the start of the array of images held by @value.
  * optionally return the number of elements in @n.
  *
- * See also: vips_value_set_array_image().
+ * ::: seealso
+ *     [func@value_set_array_image].
  *
  * Returns: (transfer none) (array length=n): The array address.
  */
@@ -1879,12 +1924,13 @@ vips_value_get_array_image(const GValue *value, int *n)
 
 /**
  * vips_value_set_array_image:
- * @value: %GValue to get from
+ * @value: [struct@GObject.Value] to get from
  * @n: the number of elements
  *
  * Set @value to hold an array of images. Pass in the array length in @n.
  *
- * See also: vips_array_image_get().
+ * ::: seealso
+ *     [method@ArrayImage.get].
  */
 void
 vips_value_set_array_image(GValue *value, int n)
@@ -1899,13 +1945,14 @@ vips_value_set_array_image(GValue *value, int n)
 
 /**
  * vips_value_get_array_object: (skip)
- * @value: %GValue to get from
+ * @value: [struct@GObject.Value] to get from
  * @n: (out) (optional): return the number of elements here, optionally
  *
- * Return the start of the array of %GObject held by @value.
+ * Return the start of the array of [class@GObject.Object] held by @value.
  * Optionally return the number of elements in @n.
  *
- * See also: vips_area_new_array_object().
+ * ::: seealso
+ *     [ctor@Area.new_array_object].
  *
  * Returns: (transfer none) (array length=n): The array address.
  */
@@ -1917,12 +1964,14 @@ vips_value_get_array_object(const GValue *value, int *n)
 
 /**
  * vips_value_set_array_object:
- * @value: (out): %GValue to set
+ * @value: (out): [struct@GObject.Value] to set
  * @n: the number of elements
  *
- * Set @value to hold an array of %GObject. Pass in the array length in @n.
+ * Set @value to hold an array of [class@GObject.Object]. Pass in the array
+ * length in @n.
  *
- * See also: vips_value_get_array_object().
+ * ::: seealso
+ *     [func@value_get_array_object].
  */
 void
 vips_value_set_array_object(GValue *value, int n)
