@@ -32,6 +32,10 @@
 #define DEBUG
  */
 
+/* Enable linux extensions like O_TMPFILE, if available.
+ */
+#define _GNU_SOURCE
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif /*HAVE_CONFIG_H*/
@@ -45,13 +49,13 @@
 
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <fcntl.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif /*HAVE_UNISTD_H*/
 #ifdef HAVE_IO_H
 #include <io.h>
 #endif /*HAVE_IO_H*/
-#include <fcntl.h>
 
 #include <vips/vips.h>
 
@@ -68,7 +72,14 @@
 
 #define MODE_READ CLOEXEC(BINARYIZE(O_RDONLY))
 
-/* Test two lists for eqality.
+/**
+ * vips_slist_equal:
+ * @l1: (element-type guint8): a [struct@GLib.SList]
+ * @l2: (element-type guint8): another [struct@GLib.SList]
+ *
+ * Test two lists for equality.
+ *
+ * Returns: `TRUE` if @l1 is equal to @l2. `FALSE` otherwise.
  */
 gboolean
 vips_slist_equal(GSList *l1, GSList *l2)
@@ -87,7 +98,17 @@ vips_slist_equal(GSList *l1, GSList *l2)
 	return TRUE;
 }
 
-/* Map over an slist. _copy() the list in case the callback changes it.
+/**
+ * vips_slist_map2:
+ * @list: (element-type guint8): a [struct@GLib.SList]
+ * @fn: (scope call): function to apply to each list element
+ * @a: user data
+ * @b: user data
+ *
+ * Map over a slist. _copy() the list in case the callback changes it.
+ *
+ * Returns: `NULL` if @fn returns `NULL` for all arguments, otherwise the first
+ * non-`NULL` value from @fn.
  */
 void *
 vips_slist_map2(GSList *list, VipsSListMap2Fn fn, void *a, void *b)
@@ -105,7 +126,17 @@ vips_slist_map2(GSList *list, VipsSListMap2Fn fn, void *a, void *b)
 	return result;
 }
 
-/* Map backwards. We _reverse() rather than recurse and unwind to save stack.
+/**
+ * vips_slist_map2_rev:
+ * @list: (element-type guint8): a [struct@GLib.SList]
+ * @fn: (scope call): function to apply to each list element
+ * @a: user data
+ * @b: user data
+ *
+ * Map backwards. We _reverse() rather than recurse and unwind to save stack.
+ *
+ * Returns: `NULL` if @fn returns `NULL` for all arguments, otherwise the first
+ * non-`NULL` value from @fn.
  */
 void *
 vips_slist_map2_rev(GSList *list, VipsSListMap2Fn fn, void *a, void *b)
@@ -124,6 +155,20 @@ vips_slist_map2_rev(GSList *list, VipsSListMap2Fn fn, void *a, void *b)
 	return result;
 }
 
+/**
+ * vips_slist_map4:
+ * @list: (element-type guint8): a [struct@GLib.SList]
+ * @fn: (scope call): function to apply to each list element
+ * @a: user data
+ * @b: user data
+ * @c: user data
+ * @d: user data
+ *
+ * Map over a slist. _copy() the list in case the callback changes it.
+ *
+ * Returns: `NULL` if @fn returns `NULL` for all arguments, otherwise the first
+ * non-`NULL` value from @fn.
+ */
 void *
 vips_slist_map4(GSList *list,
 	VipsSListMap4Fn fn, void *a, void *b, void *c, void *d)
@@ -142,6 +187,19 @@ vips_slist_map4(GSList *list,
 	return result;
 }
 
+/**
+ * vips_slist_fold2:
+ * @list: (element-type guint8): a [struct@GLib.SList]
+ * @start: initial value for the accumulator
+ * @fn: (scope call): function to apply to each list element
+ * @a: user data
+ * @b: user data
+ *
+ * Fold over a slist, applying @fn to each element.
+ *
+ * Returns: `NULL` if @fn returns `NULL` for all arguments, otherwise the first
+ * non-`NULL` value from @fn.
+ */
 void *
 vips_slist_fold2(GSList *list, void *start,
 	VipsSListFold2Fn fn, void *a, void *b)
@@ -159,7 +217,17 @@ vips_slist_fold2(GSList *list, void *start,
 	return c;
 }
 
-/* Remove all occurrences of an item from a list.
+/**
+ * vips_slist_filter:
+ * @list: (element-type guint8): a [struct@GLib.SList]
+ * @fn: (scope call): function to call for each element.
+ * @a: user data
+ * @b: user data
+ *
+ * Remove all occurrences of an item from a list.
+ * Returns the new head of the list.
+ *
+ * Returns: (element-type guint8) (transfer full): new head of @list
  */
 GSList *
 vips_slist_filter(GSList *list, VipsSListMap2Fn fn, void *a, void *b)
@@ -198,7 +266,11 @@ vips_slist_free_all_cb(void *thing, void *dummy)
 	g_free(thing);
 }
 
-/* Free a g_slist of things which need g_free()ing.
+/**
+ * vips_slist_free_all:
+ * @list: (element-type guint8): a [struct@GLib.SList]
+ *
+ * Free a [struct@GLib.SList] of things which need [func@GLib.free]ing.
  */
 void
 vips_slist_free_all(GSList *list)
@@ -229,7 +301,17 @@ vips_hash_table_predicate(const char *key, void *value, Pair *pair)
 	return (pair->result = pair->fn(value, pair->a, pair->b)) != NULL;
 }
 
-/* Like slist map, but for a hash table.
+/**
+ * vips_hash_table_map:
+ * @hash: a [struct@GLib.HashTable]
+ * @fn: (scope call): function to apply to each hash value
+ * @a: user data
+ * @b: user data
+ *
+ * Like slist map, but for a hash table.
+ *
+ * Returns: `NULL` if @fn returns `NULL` for all arguments, otherwise the first
+ * non-`NULL` value from @fn.
  */
 void *
 vips_hash_table_map(GHashTable *hash, VipsSListMap2Fn fn, void *a, void *b)
@@ -489,7 +571,7 @@ vips__write(int fd, const void *buf, size_t count)
 /* Set the create date on a file. On Windows, the create date may be copied
  * over from an existing file of the same name, unless you reset it.
  *
- * See https://blogs.msdn.microsoft.com/oldnewthing/20050715-14/?p=34923
+ * See https://devblogs.microsoft.com/oldnewthing/20050715-14/?p=34923
  */
 void
 vips__set_create_time(int fd)
@@ -521,8 +603,14 @@ vips__open(const char *filename, int flags, int mode)
 
 	/* Various bad things happen if you accidentally open a directory as a
 	 * file.
+	 *
+	 * Except in O_TMPFILE mode, when you have to.
 	 */
-	if (g_file_test(filename, G_FILE_TEST_IS_DIR)) {
+	if (
+#ifdef O_TMPFILE
+		!(flags & O_TMPFILE) &&
+#endif /*O_TMPFILE*/
+		g_file_test(filename, G_FILE_TEST_IS_DIR)) {
 		errno = EISDIR;
 		return -1;
 	}
@@ -666,25 +754,22 @@ vips__file_read(FILE *fp, const char *filename, size_t *length_out)
 	if (len == -1) {
 		int size;
 
-		/* Can't get length: read in chunks and realloc() to end of
+		/* Can't get length: read in chunks and g_realloc() to end of
 		 * file.
 		 */
 		str = NULL;
 		len = 0;
 		size = 0;
 		do {
-			char *str2;
-
 			/* Again, a 1gb sanity limit.
 			 */
 			size += 1024;
-			if (size > 1024 * 1024 * 1024 ||
-				!(str2 = realloc(str, size))) {
-				free(str);
+			if (size > 1024 * 1024 * 1024) {
+				g_free(str);
 				vips_error("vips__file_read", "%s", _("out of memory"));
 				return NULL;
 			}
-			str = str2;
+			str = g_realloc(str, size);
 
 			/* -1 to allow space for an extra NULL we add later.
 			 */
@@ -850,7 +935,11 @@ vips__gvalue_ref_string_new(const char *text)
 	return value;
 }
 
-/* Free a GSList of GValue.
+/**
+ * vips__gslist_gvalue_free:
+ * @list: (element-type GValue): a [struct@GLib.SList] of GValue
+ *
+ * Free a GSList of GValue.
  */
 void
 vips__gslist_gvalue_free(GSList *list)
@@ -859,7 +948,13 @@ vips__gslist_gvalue_free(GSList *list)
 	g_slist_free(list);
 }
 
-/* Copy a GSList of GValue.
+/**
+ * vips__gslist_gvalue_copy:
+ * @list: (element-type GValue): a [struct@GLib.SList] of GValue
+ *
+ * Copy a GSList of GValue.
+ *
+ * Returns: (element-type GValue) (transfer full): a copy of @list
  */
 GSList *
 vips__gslist_gvalue_copy(const GSList *list)
@@ -870,17 +965,22 @@ vips__gslist_gvalue_copy(const GSList *list)
 	copy = NULL;
 
 	for (p = list; p; p = p->next)
-		copy = g_slist_prepend(copy,
-			vips__gvalue_copy((GValue *) p->data));
+		copy = g_slist_prepend(copy, vips__gvalue_copy((GValue *) p->data));
 
 	copy = g_slist_reverse(copy);
 
 	return copy;
 }
 
-/* Merge two GSList of GValue ... append to a all elements in b which are not
- * in a. Return the new value of a. Works for any vips refcounted type
- * (string, blob, etc.).
+/**
+ * vips__gslist_gvalue_merge:
+ * @a: (element-type GValue): a [struct@GLib.SList] of GValue
+ * @b: (element-type GValue): a [struct@GLib.SList] of GValue
+ *
+ * Merge two GSList of GValue ... append to a all elements in b which are not
+ * in a. Works for any vips refcounted type (string, blob, etc.).
+ *
+ * Returns: (element-type GValue) (transfer full): the new value of @a
  */
 GSList *
 vips__gslist_gvalue_merge(GSList *a, const GSList *b)
@@ -898,8 +998,7 @@ vips__gslist_gvalue_merge(GSList *a, const GSList *b)
 		for (j = a; j; j = j->next) {
 			GValue *value2 = (GValue *) j->data;
 
-			g_assert(G_VALUE_TYPE(value2) ==
-				VIPS_TYPE_REF_STRING);
+			g_assert(G_VALUE_TYPE(value2) == VIPS_TYPE_REF_STRING);
 
 			/* Just do a pointer compare ... good enough 99.9% of
 			 * the time.
@@ -910,8 +1009,7 @@ vips__gslist_gvalue_merge(GSList *a, const GSList *b)
 		}
 
 		if (!j)
-			tail = g_slist_prepend(tail,
-				vips__gvalue_copy(value));
+			tail = g_slist_prepend(tail, vips__gvalue_copy(value));
 	}
 
 	a = g_slist_concat(a, g_slist_reverse(tail));
@@ -919,8 +1017,16 @@ vips__gslist_gvalue_merge(GSList *a, const GSList *b)
 	return a;
 }
 
-/* Make a char * from GSList of GValue. Each GValue should be a ref_string.
- * free the result. Empty list -> "", not NULL. Join strings with '\n'.
+/**
+ * vips__gslist_gvalue_get:
+ * @list: (element-type GValue): a [struct@GLib.SList] of GValue
+ *
+ * Make a char * from GSList of GValue. Each GValue should be a ref_string.
+ *
+ * If @list is empty, the return value will be `NULL`.
+ *
+ * Returns: (transfer full) (nullable): a newly-allocated string containing
+ *   all of the list elements joined together, with '\n' between them.
  */
 char *
 vips__gslist_gvalue_get(const GSList *list)
@@ -999,8 +1105,7 @@ vips__seek(int fd, gint64 pos, int whence)
 	gint64 new_pos;
 
 	if ((new_pos = vips__seek_no_error(fd, pos, whence)) == -1) {
-		vips_error_system(errno, "vips__seek",
-			"%s", _("unable to seek"));
+		vips_error_system(errno, "vips__seek", "%s", _("unable to seek"));
 		return -1;
 	}
 
@@ -1263,8 +1368,7 @@ vips__token_must(const char *p, VipsToken *token,
 	char *string, int size)
 {
 	if (!(p = vips__token_get(p, token, string, size))) {
-		vips_error("get_token",
-			"%s", _("unexpected end of string"));
+		vips_error("get_token", "%s", _("unexpected end of string"));
 		return NULL;
 	}
 
@@ -1489,7 +1593,7 @@ vips_amiMSBfirst(void)
 #endif
 }
 
-/* Return the tmp dir. On Windows, GetTempPath() will also check the values of
+/* Return the tmp dir. On Windows, GetTempPathW() will also check the values of
  * TMP, TEMP and USERPROFILE.
  */
 static const char *
@@ -1499,14 +1603,22 @@ vips__temp_dir(void)
 
 	if (!(tmpd = g_getenv("TMPDIR"))) {
 #ifdef G_OS_WIN32
-		static gboolean done = FALSE;
-		static char buf[256];
+		static char *tmp_dir = NULL;
 
-		if (!done) {
-			if (!GetTempPath(256, buf))
-				strcpy(buf, "C:\\temp");
+		if (tmp_dir == NULL) {
+			char *dir = NULL;
+			wchar_t wdir[MAX_PATH];
+
+			if (GetTempPathW(G_N_ELEMENTS(wdir), wdir))
+				dir = g_utf16_to_utf8(wdir, -1, NULL, NULL, NULL);
+
+			if (dir == NULL)
+				dir = g_strdup("C:\\temp");
+
+			tmp_dir = g_steal_pointer(&dir);
 		}
-		tmpd = buf;
+
+		tmpd = tmp_dir;
 #else  /*!G_OS_WIN32*/
 		tmpd = "/tmp";
 #endif /*!G_OS_WIN32*/
@@ -1531,8 +1643,7 @@ vips__temp_name(const char *format)
 
 	int serial = g_atomic_int_add(&global_serial, 1);
 
-	g_snprintf(file, FILENAME_MAX, "vips-%d-%u",
-		serial, g_random_int());
+	g_snprintf(file, FILENAME_MAX, "vips-%d-%u", serial, g_random_int());
 	g_snprintf(file2, FILENAME_MAX, format, file);
 	name = g_build_filename(vips__temp_dir(), file2, NULL);
 
@@ -1610,6 +1721,7 @@ vips__parse_size(const char *size_string)
 	 */
 	unit = g_strdup(size_string);
 
+	size = 0;
 	n = sscanf(size_string, "%" G_GUINT64_FORMAT " %s", &size, unit);
 	if (n > 1)
 		for (int j = 0; j < VIPS_NUMBER(units); j++)
@@ -1671,9 +1783,15 @@ vips_enum_from_nick(const char *domain, GType type, const char *nick)
 	if ((enum_value = g_enum_get_value_by_nick(genum, nick)))
 		return enum_value->value;
 
-	/* -1 since we always have a "last" member.
+	/* Compat for "last" members. Assumes all enums define a `_LAST` value;
+	 * behaviour is undefined otherwise. Note that there could be potential
+	 * gaps in enum values (e.g. VipsInterpretation), so we cannot return
+	 * `genum->n_values` directly.
 	 */
-	for (i = 0; i < genum->n_values - 1; i++) {
+	if (nick && g_str_equal(nick, "last"))
+		return genum->values[genum->n_values - 1].value + 1;
+
+	for (i = 0; i < genum->n_values; i++) {
 		if (i > 0)
 			vips_buf_appends(&buf, ", ");
 		vips_buf_appends(&buf, genum->values[i].value_nick);
@@ -1730,9 +1848,11 @@ vips_flags_from_nick(const char *domain, GType type, const char *nick)
  * lowest-numbered one for @sub. @buf is @len bytes in size.
  *
  * If there are no %ns, use the first %s.
+ *
+ * Set @c to the %s char we search for.
  */
 int
-vips__substitute(char *buf, size_t len, char *sub)
+vips__substitutec(char *buf, size_t len, char c, char *sub)
 {
 	size_t buflen = strlen(buf);
 	size_t sublen = strlen(sub);
@@ -1754,7 +1874,7 @@ vips__substitute(char *buf, size_t len, char *sub)
 
 			for (q = p + 1; g_ascii_isdigit(*q); q++)
 				;
-			if (q[0] == 's') {
+			if (q[0] == c) {
 				int n;
 
 				n = atoi(p + 1);
@@ -1769,7 +1889,7 @@ vips__substitute(char *buf, size_t len, char *sub)
 
 	if (!sub_start)
 		for (p = buf; (p = strchr(p, '%')); p++)
-			if (p[1] == 's') {
+			if (p[1] == c) {
 				sub_start = p;
 				sub_end = p + 2;
 				break;
@@ -1792,6 +1912,12 @@ vips__substitute(char *buf, size_t len, char *sub)
 	return 0;
 }
 
+int
+vips__substitute(char *buf, size_t len, char *sub)
+{
+	return vips__substitutec(buf, len, 's', sub);
+}
+
 /* Absoluteize a path. Free the result with g_free().
  */
 char *
@@ -1857,8 +1983,7 @@ vips_icc_dir_once(void *null)
 		 */
 		char *windowsdir;
 
-		if ((windowsdir = g_utf16_to_utf8(wwindowsdir,
-				 -1, NULL, NULL, NULL))) {
+		if ((windowsdir = g_utf16_to_utf8(wwindowsdir, -1, NULL, NULL, NULL))) {
 			gchar *full_path;
 
 			full_path = g_build_filename(windowsdir,
@@ -1878,8 +2003,7 @@ vips__icc_dir(void)
 {
 	static GOnce once = G_ONCE_INIT;
 
-	return (const char *) g_once(&once,
-		vips_icc_dir_once, NULL);
+	return (const char *) g_once(&once, vips_icc_dir_once, NULL);
 }
 
 #ifdef G_OS_WIN32
@@ -1902,8 +2026,7 @@ vips__windows_prefix_once(void *null)
 	char *prefix;
 
 #ifdef G_OS_WIN32
-	prefix = g_win32_get_package_installation_directory_of_module(
-		vips__dll);
+	prefix = g_win32_get_package_installation_directory_of_module(vips__dll);
 #else  /*!G_OS_WIN32*/
 	prefix = (char *) g_getenv("VIPSHOME");
 #endif /*G_OS_WIN32*/
@@ -1916,8 +2039,7 @@ vips__windows_prefix(void)
 {
 	static GOnce once = G_ONCE_INIT;
 
-	return (const char *) g_once(&once,
-		vips__windows_prefix_once, NULL);
+	return (const char *) g_once(&once, vips__windows_prefix_once, NULL);
 }
 
 char *
