@@ -103,20 +103,6 @@
 #include <vips/debug.h>
 #include <vips/internal.h>
 
-/**
- * SECTION: vips
- * @short_description: startup, shutdown, version
- * @stability: Stable
- * @see_also: <link linkend="VipsOperation">VipsOperation</link>
- * @include: vips/vips.h
- *
- * Start VIPS up, shut VIPS down, get version information, relocation.
- *
- * VIPS is a relocatable package, meaning you can move the directory tree you
- * compiled it to at runtime and it will still be able to find all data files.
- * This is required for OS X and Windows, but slightly unusual in the Unix
- * world. See vips_init() and vips_guess_prefix().
- */
 
 /* Open mode for image write.
  *
@@ -192,10 +178,9 @@ vips__open_image_write(const char *filename, gboolean temp)
 		g_info("vips__open_image_write: opening with O_TMPFILE");
 		dirname = g_path_get_dirname(filename);
 		fd = vips_tracked_open(dirname, O_TMPFILE | O_RDWR, 0644);
-		g_free(dirname);
-
 		if (fd < 0)
 			g_info("vips__open_image_write: O_TMPFILE failed!");
+		g_free(dirname);
 	}
 #endif /*O_TMPFILE*/
 
@@ -373,20 +358,18 @@ vips__read_header_bytes(VipsImage *im, unsigned char *from)
 	 */
 	value = g_enum_get_value(g_type_class_ref(VIPS_TYPE_INTERPRETATION),
 		im->Type);
-	if (!value ||
-		strcmp(value->value_nick, "last") == 0)
+	if (!value)
 		im->Type = VIPS_INTERPRETATION_ERROR;
 	value = g_enum_get_value(g_type_class_ref(VIPS_TYPE_CODING),
 		im->Coding);
-	if (!value ||
-		strcmp(value->value_nick, "last") == 0)
+	if (!value)
 		im->Coding = VIPS_CODING_ERROR;
 
 	/* Offset, Res, etc. don't affect vips file layout, just
 	 * pixel interpretation, don't clip them.
 	 */
 
-	/* Coding values imply Bands and BandFmt settings --- make sure they
+	/* Coding values imply Bands and BandFmt settings -- make sure they
 	 * are sane.
 	 */
 	switch (im->Coding) {
@@ -506,7 +489,7 @@ vips__has_extension_block(VipsImage *im)
 /* Read everything after the pixels into memory.
  */
 void *
-vips__read_extension_block(VipsImage *im, int *size)
+vips__read_extension_block(VipsImage *im, size_t *size)
 {
 	gint64 psize;
 	void *buf;
@@ -515,8 +498,7 @@ vips__read_extension_block(VipsImage *im, int *size)
 	g_assert(im->file_length > 0);
 	if (im->file_length - psize > 100 * 1024 * 1024) {
 		vips_error("VipsImage",
-			"%s", _("more than 100 megabytes of XML? "
-					"sufferin' succotash!"));
+			"%s", _("more than 100 megabytes of XML? sufferin' succotash!"));
 		return NULL;
 	}
 	if (im->file_length - psize == 0)
@@ -617,9 +599,9 @@ parser_element_start_handler(void *user_data,
 	if (strcmp(name, "field") == 0) {
 		for (p = atts; *p; p += 2) {
 			if (strcmp(p[0], "name") == 0)
-				vips_strncpy(vep->name, p[1], MAX_PARSE_ATTR);
+				g_strlcpy(vep->name, p[1], MAX_PARSE_ATTR);
 			if (strcmp(p[0], "type") == 0)
-				vips_strncpy(vep->type, p[1], MAX_PARSE_ATTR);
+				g_strlcpy(vep->type, p[1], MAX_PARSE_ATTR);
 		}
 
 		vips_dbuf_reset(&vep->dbuf);
@@ -743,8 +725,9 @@ parser_data_handler(void *user_data, const XML_Char *data, int len)
 static int
 readhist(VipsImage *im)
 {
+	VipsExpatParse vep = { 0 };
+
 	XML_Parser parser;
-	VipsExpatParse vep;
 
 	if (vips__seek(im->fd, image_pixel_length(im), SEEK_SET) == -1)
 		return -1;
@@ -752,8 +735,6 @@ readhist(VipsImage *im)
 	parser = XML_ParserCreate("UTF-8");
 
 	vep.image = im;
-	vips_dbuf_init(&vep.dbuf);
-	vep.error = FALSE;
 	XML_SetUserData(parser, &vep);
 
 	XML_SetElementHandler(parser,
@@ -774,7 +755,7 @@ readhist(VipsImage *im)
 }
 
 int
-vips__write_extension_block(VipsImage *im, void *buf, int size)
+vips__write_extension_block(VipsImage *im, void *buf, size_t size)
 {
 	gint64 length;
 	gint64 psize;
@@ -794,7 +775,7 @@ vips__write_extension_block(VipsImage *im, void *buf, int size)
 		return -1;
 
 #ifdef DEBUG
-	printf("vips__write_extension_block: written %d bytes of XML to %s\n",
+	printf("vips__write_extension_block: written %zd bytes of XML to %s\n",
 		size, im->filename);
 #endif /*DEBUG*/
 
@@ -1040,8 +1021,7 @@ vips_image_open_input(VipsImage *image)
 			VIPS_SIZEOF_HEADER ||
 		vips__read_header_bytes(image, header)) {
 		vips_error_system(errno, "VipsImage",
-			_("unable to read header for \"%s\""),
-			image->filename);
+			_("unable to read header for \"%s\""), image->filename);
 		return -1;
 	}
 
@@ -1054,8 +1034,8 @@ vips_image_open_input(VipsImage *image)
 		return -1;
 	image->file_length = rsize;
 	if (psize > rsize)
-		g_warning(_("unable to read data for \"%s\", %s"),
-			image->filename, _("file has been truncated"));
+		g_warning("unable to read data for \"%s\", %s",
+			image->filename, "file has been truncated");
 
 	/* Set demand style. This suits a disc file we read sequentially.
 	 */
@@ -1066,8 +1046,7 @@ vips_image_open_input(VipsImage *image)
 	 * harmless.
 	 */
 	if (readhist(image)) {
-		g_warning(_("error reading vips image metadata: %s"),
-			vips_error_buffer());
+		g_warning("error reading vips image metadata: %s", vips_error_buffer());
 		vips_error_clear();
 	}
 
