@@ -51,10 +51,10 @@
 #endif /*HAVE_CONFIG_H*/
 #include <glib/gi18n-lib.h>
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <stdint.h>
-#include <math.h>
+#include <cstdio>
+#include <cstdlib>
+#include <cstdint>
+#include <cmath>
 
 #include <vips/vips.h>
 #include <vips/vector.h>
@@ -82,6 +82,10 @@ typedef struct _VipsReduceh {
 	 */
 	double hoffset;
 
+	/* The hshrink we do after integer reduction.
+	 */
+	double residual_hshrink;
+
 	/* Precalculated interpolation matrices. short (used for pel
 	 * sizes up to int), and double (for all others). We go to
 	 * scale + 1 so we can round-to-nearest safely.
@@ -125,6 +129,12 @@ vips_reduce_get_points(VipsKernel kernel, double shrink)
 	case VIPS_KERNEL_LANCZOS3:
 		return 2 * rint(3 * shrink) + 1;
 
+	case VIPS_KERNEL_MKS2013:
+		return 2 * rint(3 * shrink) + 1;
+
+	case VIPS_KERNEL_MKS2021:
+		return 2 * rint(5 * shrink) + 1;
+
 	default:
 		g_assert_not_reached();
 		return 0;
@@ -196,7 +206,7 @@ static void inline reduceh_notab(VipsReduceh *reduceh,
 	typename LongT<T>::type cx[MAX_POINT];
 
 	vips_reduce_make_mask(cx, reduceh->kernel, reduceh->n_point,
-		reduceh->hshrink, x);
+		reduceh->residual_hshrink, x);
 
 	for (int z = 0; z < bands; z++)
 		out[z] = reduce_sum<T>(in + z, bands, cx, n);
@@ -224,9 +234,9 @@ vips_reduceh_gen(VipsRegion *out_region, void *seq,
 		r->width, r->height, r->left, r->top);
 #endif /*DEBUG*/
 
-	s.left = r->left * reduceh->hshrink - reduceh->hoffset;
+	s.left = r->left * reduceh->residual_hshrink - reduceh->hoffset;
 	s.top = r->top;
-	s.width = r->width * reduceh->hshrink + reduceh->n_point;
+	s.width = r->width * reduceh->residual_hshrink + reduceh->n_point;
 	s.height = r->height;
 	if (vips_region_prepare(ir, &s))
 		return -1;
@@ -241,7 +251,7 @@ vips_reduceh_gen(VipsRegion *out_region, void *seq,
 
 		q = VIPS_REGION_ADDR(out_region, r->left, r->top + y);
 
-		X = (r->left + 0.5) * reduceh->hshrink - 0.5 -
+		X = (r->left + 0.5) * reduceh->residual_hshrink - 0.5 -
 			reduceh->hoffset;
 
 		/* We want p0 to be the start (ie. x == 0) of the input
@@ -313,7 +323,7 @@ vips_reduceh_gen(VipsRegion *out_region, void *seq,
 				break;
 			}
 
-			X += reduceh->hshrink;
+			X += reduceh->residual_hshrink;
 			q += ps;
 		}
 	}
@@ -344,9 +354,9 @@ vips_reduceh_uchar_vector_gen(VipsRegion *out_region, void *seq,
 		r->width, r->height, r->left, r->top);
 #endif /*DEBUG*/
 
-	s.left = r->left * reduceh->hshrink - reduceh->hoffset;
+	s.left = r->left * reduceh->residual_hshrink - reduceh->hoffset;
 	s.top = r->top;
-	s.width = r->width * reduceh->hshrink + reduceh->n_point;
+	s.width = r->width * reduceh->residual_hshrink + reduceh->n_point;
 	s.height = r->height;
 	if (vips_region_prepare(ir, &s))
 		return -1;
@@ -361,14 +371,14 @@ vips_reduceh_uchar_vector_gen(VipsRegion *out_region, void *seq,
 
 		q = VIPS_REGION_ADDR(out_region, r->left, r->top + y);
 
-		X = (r->left + 0.5) * reduceh->hshrink - 0.5 -
+		X = (r->left + 0.5) * reduceh->residual_hshrink - 0.5 -
 			reduceh->hoffset;
 
 		p0 = VIPS_REGION_ADDR(ir, ir->valid.left, r->top + y) -
 			ir->valid.left * ps;
 
 		vips_reduceh_uchar_hwy(q, p0, reduceh->n_point, r->width,
-			bands, reduceh->matrixs, X, reduceh->hshrink);
+			bands, reduceh->matrixs, X, reduceh->residual_hshrink);
 	}
 
 	VIPS_GATE_STOP("vips_reduceh_uchar_vector_gen: work");
@@ -416,6 +426,10 @@ vips_reduceh_build(VipsObject *object)
 	 */
 	extra_pixels = width * reduceh->hshrink - in->Xsize;
 
+	/* The hshrink we do after integer reduction.
+	 */
+	reduceh->residual_hshrink = reduceh->hshrink;
+
 	if (reduceh->gap > 0.0 &&
 		reduceh->kernel != VIPS_KERNEL_NEAREST) {
 		if (reduceh->gap < 1.0) {
@@ -427,7 +441,7 @@ vips_reduceh_build(VipsObject *object)
 		/* The int part of our reduce.
 		 */
 		int_hshrink = VIPS_MAX(1,
-			VIPS_FLOOR((double) in->Xsize / width / reduceh->gap));
+			floor((double) in->Xsize / width / reduceh->gap));
 
 		if (int_hshrink > 1) {
 			g_info("shrinkh by %d", int_hshrink);
@@ -437,16 +451,16 @@ vips_reduceh_build(VipsObject *object)
 				return -1;
 			in = t[0];
 
-			reduceh->hshrink /= int_hshrink;
+			reduceh->residual_hshrink /= int_hshrink;
 			extra_pixels /= int_hshrink;
 		}
 	}
 
-	if (reduceh->hshrink == 1.0)
+	if (reduceh->residual_hshrink == 1.0)
 		return vips_image_write(in, resample->out);
 
 	reduceh->n_point =
-		vips_reduce_get_points(reduceh->kernel, reduceh->hshrink);
+		vips_reduce_get_points(reduceh->kernel, reduceh->residual_hshrink);
 	g_info("reduceh: %d point mask", reduceh->n_point);
 	if (reduceh->n_point > MAX_POINT) {
 		vips_error(object_class->nickname,
@@ -473,7 +487,7 @@ vips_reduceh_build(VipsObject *object)
 			return -1;
 
 		vips_reduce_make_mask(reduceh->matrixf[x], reduceh->kernel,
-			reduceh->n_point, reduceh->hshrink,
+			reduceh->n_point, reduceh->residual_hshrink,
 			(float) x / VIPS_TRANSFORM_SCALE);
 
 		for (int i = 0; i < reduceh->n_point; i++)
@@ -496,7 +510,7 @@ vips_reduceh_build(VipsObject *object)
 	/* Add new pixels around the input so we can interpolate at the edges.
 	 */
 	if (vips_embed(in, &t[2],
-			VIPS_CEIL(reduceh->n_point / 2.0) - 1, 0,
+			ceil(reduceh->n_point / 2.0) - 1, 0,
 			in->Xsize + reduceh->n_point, in->Ysize,
 			"extend", VIPS_EXTEND_COPY,
 			nullptr))
@@ -615,9 +629,39 @@ vips_reduceh_init(VipsReduceh *reduceh)
 	reduceh->kernel = VIPS_KERNEL_LANCZOS3;
 }
 
-/* See reduce.c for the doc comment.
+/**
+ * vips_reduceh: (method)
+ * @in: input image
+ * @out: (out): output image
+ * @hshrink: horizontal reduce
+ * @...: `NULL`-terminated list of optional named arguments
+ *
+ * Reduce @in horizontally by a float factor.
+ *
+ * The pixels in @out are
+ * interpolated with a 1D mask generated by @kernel.
+ *
+ * Set @gap to speed up reducing by having [method@Image.shrinkh] to shrink
+ * with a box filter first. The bigger @gap, the closer the result
+ * to the fair resampling. The smaller @gap, the faster resizing.
+ * The default value is 0.0 (no optimization).
+ *
+ * This is a very low-level operation: see [method@Image.resize] for a more
+ * convenient way to resize images.
+ *
+ * This operation does not change xres or yres. The image resolution needs to
+ * be updated by the application.
+ *
+ * ::: tip "Optional arguments"
+ *     * @kernel: [enum@Kernel], kernel to interpolate with
+ *       (default: [enum@Vips.Kernel.LANCZOS3])
+ *     * @gap: `gboolean`, reducing gap to use (default: 0.0)
+ *
+ * ::: seealso
+ *     [method@Image.shrink], [method@Image.resize], [method@Image.affine].
+ *
+ * Returns: 0 on success, -1 on error
  */
-
 int
 vips_reduceh(VipsImage *in, VipsImage **out, double hshrink, ...)
 {
