@@ -416,6 +416,137 @@ class TestForeign:
         # and this should fail with a warning once more
         x = im.avg()
 
+    @skip_if_no("uhdrload")
+    def test_uhdrload(self):
+        # decode as sRGB + gainmap
+        im = pyvips.Image.uhdrload(UHDR_FILE)
+
+        assert im.width == 3840
+        assert im.height == 2160
+        assert im.bands == 3
+        assert im.format == "uchar"
+        assert im.interpretation == "srgb"
+
+        for name in ["gainmap-max-content-boost",
+                     "gainmap-min-content-boost",
+                     "gainmap-gamma",
+                     "gainmap-offset-sdr",
+                     "gainmap-offset-hdr"]:
+            value = im.get(name)
+            assert isinstance(value, list)
+            assert len(value) == 3
+
+        for name in ["gainmap-hdr-capacity-min",
+                     "gainmap-hdr-capacity-max",
+                     "gainmap-use-base-cg"]:
+            value = im.get(name)
+            assert isinstance(value, (int, float))
+
+        value = im.get("gainmap-data")
+        assert len(value) > 10000
+
+        value = im.get("icc-profile-data")
+        assert len(value) > 100
+
+    @skip_if_no("uhdrsave")
+    def test_uhdrsave(self):
+        im = pyvips.Image.uhdrload(UHDR_FILE)
+        data = im.uhdrsave_buffer()
+        im2 = pyvips.Image.uhdrload_buffer(data)
+
+        assert im2.width == 3840
+        assert im2.height == 2160
+        assert im2.bands == 3
+        assert im2.format == "uchar"
+        assert im2.interpretation == "srgb"
+        value = im2.get("gainmap-data")
+        assert len(value) > 10000
+
+    @skip_if_no("uhdrsave")
+    def test_uhdrsave_roundtrip(self):
+        im = pyvips.Image.uhdrload(UHDR_FILE)
+        data = im.uhdrsave_buffer()
+        im_hdr2 = pyvips.Image.uhdrload_buffer(data).uhdr2scRGB()
+        im_hdr = pyvips.Image.uhdrload(UHDR_FILE).uhdr2scRGB()
+
+        assert (im_hdr2 - im_hdr).abs().avg() < 0.02
+
+    @skip_if_no("uhdrsave")
+    def test_uhdrsave_roundtrip_hdr(self):
+        im = pyvips.Image.uhdrload(UHDR_FILE).uhdr2scRGB()
+        data = im.uhdrsave_buffer()
+        im2 = pyvips.Image.uhdrload_buffer(data).uhdr2scRGB()
+
+        assert (im2 - im).abs().avg() < 0.05
+
+    @skip_if_no("uhdrload")
+    def test_uhdr_thumbnail(self):
+        im = pyvips.Image.uhdrload(UHDR_FILE)
+        thumb = pyvips.Image.thumbnail(UHDR_FILE, 128)
+        buf = thumb.uhdrsave_buffer()
+        im2 = pyvips.Image.uhdrload_buffer(buf)
+
+        gainmap_data_before = im.get("gainmap-data")
+        gainmap_data_after = im2.get("gainmap-data")
+        assert len(gainmap_data_after) < len(gainmap_data_before)
+
+        gainmap_before = pyvips.Image.jpegload_buffer(gainmap_data_before)
+        gainmap_after = pyvips.Image.jpegload_buffer(gainmap_data_after)
+        assert gainmap_before.width > gainmap_after.width
+        assert gainmap_before.height > gainmap_after.height
+
+    @skip_if_no("uhdrload")
+    def test_uhdr_thumbnail_crop(self):
+        thumb = pyvips.Image.thumbnail(UHDR_FILE, 128, crop="centre")
+        buf = thumb.uhdrsave_buffer()
+        im2 = pyvips.Image.uhdrload_buffer(buf)
+
+        gainmap_data_after = im2.get("gainmap-data")
+        gainmap_after = pyvips.Image.jpegload_buffer(gainmap_data_after)
+        assert abs(gainmap_after.width - gainmap_after.height) < 5
+
+    @skip_if_no("uhdrload")
+    @skip_if_no("dzsave")
+    def test_uhdr_dzsave(self):
+        filename = temp_filename(self.tempdir, '')
+        uhdr = pyvips.Image.uhdrload(UHDR_FILE)
+
+        gainmap_data_before = uhdr.get("gainmap-data")
+        gainmap_before = pyvips.Image.jpegload_buffer(gainmap_data_before)
+        hscale = uhdr.width / gainmap_before.width
+        vscale = uhdr.height / gainmap_before.height
+
+        uhdr.dzsave(filename, keep="gainmap")
+
+        # _files/12 is the full res image
+        deepest = 12
+
+        for [level, tile_x, tile_y] in [
+                [deepest, 0, 0],
+                [deepest, 3, 1],
+                [10, 1, 0],
+                [9, 0, 0]
+            ]:
+            tile_path = f"{filename}_files/{level}/{tile_x}_{tile_y}.jpeg"
+
+            tile = pyvips.Image.uhdrload(tile_path)
+            gainmap_data_after = tile.get("gainmap-data")
+            gainmap_after = pyvips.Image.jpegload_buffer(gainmap_data_after)
+
+            # rounding plus overlaps
+            assert abs(gainmap_after.width - tile.width / hscale) < 2
+            assert abs(gainmap_after.height - tile.height / vscale) < 2
+
+            shrunk_gainmap = gainmap_before.resize(1 / (1 << (deepest - level)),
+                                                   kernel="linear")
+            left = tile_x * tile.width / hscale
+            top = tile_y * tile.height / vscale
+            expected_gainmap_after = shrunk_gainmap.crop(left,
+                                                         top,
+                                                         gainmap_after.width,
+                                                         gainmap_after.height)
+            assert abs(expected_gainmap_after.avg() - gainmap_after.avg()) < 1
+
     @skip_if_no("pngload")
     def test_png(self):
         def png_valid(im):
@@ -769,6 +900,13 @@ class TestForeign:
             z = y.hist_find(band=0)
             assert z(0, 0)[0] + z(255, 0)[0] == y.width * y.height
 
+        # metadata tile-width and tile-height should be correct
+        x = pyvips.Image.new_from_file(TIF_FILE)
+        buf = x.tiffsave_buffer(tile=True, tile_width=192, tile_height=224)
+        y = pyvips.Image.new_from_buffer(buf, "")
+        assert y.get("tile-width") == 192
+        assert y.get("tile-height") == 224
+
     @skip_if_no("tiffload")
     @pytest.mark.xfail(raises=AssertionError, reason="fails when libtiff was configured with --disable-old-jpeg")
     def test_tiff_ojpeg(self):
@@ -779,6 +917,8 @@ class TestForeign:
             assert im.height == 213
             assert im.bands == 4
             assert im.get("bits-per-sample") == 8
+            assert im.get("tile-width") == 240
+            assert im.get("tile-height") == 224
 
         self.file_loader("tiffload", TIF_OJPEG_TILE_FILE, tiff_ojpeg_tile_valid)
         self.buffer_loader("tiffload_buffer", TIF_OJPEG_TILE_FILE, tiff_ojpeg_tile_valid)
@@ -816,6 +956,9 @@ class TestForeign:
 
         self.file_loader("magickload", BMP_FILE, bmp_valid)
         self.buffer_loader("magickload_buffer", BMP_FILE, bmp_valid)
+        source = pyvips.Source.new_from_file(BMP_FILE)
+        x = pyvips.Image.new_from_source(source, "")
+        bmp_valid(x)
 
         # we should have rgb or rgba for svg files ... different versions of
         # IM handle this differently. GM even gives 1 band.
@@ -887,11 +1030,12 @@ class TestForeign:
         assert im.width == 433
         assert im.height == 433
 
-
         # load should see metadata like eg. icc profiles
         im = pyvips.Image.magickload(JPEG_FILE)
         assert len(im.get("icc-profile-data")) == 564
 
+        im = pyvips.Image.magickload(JPEG_FILE)
+
     # added in 8.7
     @skip_if_no("magicksave")
     def test_magicksave(self):
@@ -952,6 +1096,15 @@ class TestForeign:
         b2 = im.webpsave_buffer(Q=90)
         assert len(b2) > len(b1)
 
+        # test exact mode
+        im = pyvips.Image.new_from_file(RGBA_FILE)
+        buf = im.webpsave_buffer(lossless=True, exact=True)
+        im2 = pyvips.Image.new_from_buffer(buf, "")
+        assert (im - im2).abs().max() == 0
+        buf = im.webpsave_buffer(lossless=True)
+        im2 = pyvips.Image.new_from_buffer(buf, "")
+        assert (im - im2).abs().max() != 0
+
         # try saving an image with an ICC profile and reading it back ... if we
         # can do it, our webp supports metadata load/save
         buf = self.colour.webpsave_buffer()
@@ -1077,6 +1230,8 @@ class TestForeign:
             assert im.width == 2220
             assert im.height == 2967
             assert im.bands == 4
+            assert im.get("tile-width") == 240
+            assert im.get("tile-height") == 240
 
         self.file_loader("openslideload", OPENSLIDE_FILE, openslide_valid)
 
@@ -1106,6 +1261,46 @@ class TestForeign:
         assert abs(im.width * 2 - x.width) < 2
         assert abs(im.height * 2 - x.height) < 2
 
+        im = pyvips.Image.new_from_file(PDF_PAGE_BOX_FILE)
+        assert im.width == 709
+        assert im.height == 955
+        assert im.get("pdf-creator") == "Adobe InDesign 20.4 (Windows)"
+        assert im.get("pdf-producer") == "Adobe PDF Library 17.0"
+
+        pdfloadOp = pyvips.Operation.new_from_name("pdfload").get_description()
+
+        if "poppler" in pdfloadOp:
+            # only crop is implemented, ignore requested page box
+            im = pyvips.Image.new_from_file(PDF_FILE, page_box="art")
+            assert im.width == 1134
+            assert im.height == 680
+            im = pyvips.Image.new_from_file(PDF_PAGE_BOX_FILE, page_box="art")
+            assert im.width == 709
+            assert im.height == 955
+
+        if "pdfium" in pdfloadOp:
+            im = pyvips.Image.new_from_file(PDF_FILE, page_box="art")
+            assert im.width == 1121
+            assert im.height == 680
+            im = pyvips.Image.new_from_file(PDF_FILE, page_box="trim") # missing, will fallback to crop
+            assert im.width == 1134
+            assert im.height == 680
+            im = pyvips.Image.new_from_file(PDF_PAGE_BOX_FILE, page_box="media")
+            assert im.width == 822
+            assert im.height == 1069
+            im = pyvips.Image.new_from_file(PDF_PAGE_BOX_FILE, page_box="crop")
+            assert im.width == 709
+            assert im.height == 955
+            im = pyvips.Image.new_from_file(PDF_PAGE_BOX_FILE, page_box="bleed")
+            assert im.width == 652
+            assert im.height == 899
+            im = pyvips.Image.new_from_file(PDF_PAGE_BOX_FILE, page_box="trim")
+            assert im.width == 595
+            assert im.height == 842
+            im = pyvips.Image.new_from_file(PDF_PAGE_BOX_FILE, page_box="art")
+            assert im.width == 539
+            assert im.height == 785
+
     @skip_if_no("gifload")
     def test_gifload(self):
         def gif_valid(im):
@@ -1588,6 +1783,11 @@ class TestForeign:
             y = pyvips.Image.new_from_buffer(buf, "")
             assert y.get("exif-ifd0-XPComment").startswith("banana")
 
+    @skip_if_no("heifsave")
+    def test_avifsave_tune(self):
+        buf = self.colour.heifsave_buffer(compression="av1", tune="ssim")
+        assert len(buf) > 10000
+
     @skip_if_no("heifsave")
     @pytest.mark.xfail(raises=pyvips.error.Error, reason="requires libheif built with patent-encumbered HEVC dependencies")
     def test_heicsave_16_to_12(self):
@@ -1638,6 +1838,8 @@ class TestForeign:
             assert im.height == 400
             assert im.bands == 3
             assert im.get("bits-per-sample") == 8
+            assert im.get("tile-width") == 800
+            assert im.get("tile-height") == 400
 
         self.file_loader("jp2kload", JP2K_FILE, jp2k_valid)
         self.buffer_loader("jp2kload_buffer", JP2K_FILE, jp2k_valid)
@@ -1745,6 +1947,13 @@ class TestForeign:
         lossless = self.colour.jxlsave_buffer(lossless=True)
         assert len(lossy) < len(lossless) / 5
 
+        # bitdepth=1 should be smaller
+        buf8 = self.colour.jxlsave_buffer()
+        buf1 = self.colour.jxlsave_buffer(bitdepth=1)
+        assert len(buf1) < len(buf8)
+        im = pyvips.Image.jxlload_buffer(buf1)
+        assert im.get("bits-per-sample") == 1
+
     @skip_if_no("gifload")
     @skip_if_no("gifsave")
     def test_gifsave(self):
