@@ -13,7 +13,7 @@ test_rotate() {
 
 	printf "testing $inter ... "
 
-	# 90 degree clockwise rotate 
+	# 90 degree clockwise rotate
 	trn="0 1 1 0"
 
 	$vips affine $im $tmp/t1.v "$trn" --interpolate $inter
@@ -81,3 +81,24 @@ test_size $tmp/t1.jpg 66 100
 cat $image | $vipsthumbnail stdin -s 100 -o .jpg | cat > $tmp/t1.jpg
 echo ok
 test_size $tmp/t1.jpg 66 100
+
+# test max-coord
+# this will coredumop on an assert fail in debug builds, so block coredumps
+ulimit -c 0
+echo -n "testing --vips-max-coord ... "
+if $vips black $tmp/t1.v 1000 1000 --vips-max-coord 120 > /dev/null 2>&1; then
+  echo "FAIL"
+  echo "--vips-max-coord CLI arg test failed"
+  exit 1
+fi
+echo ok
+
+echo -n "testing VIPS_MAX_COORD ... "
+export VIPS_MAX_COORD=120
+if $vips black $tmp/t1.v 1000 1000 > /dev/null 2>&1; then
+  echo "FAIL"
+  echo "VIPS_MAX_COORD env var test failed"
+  exit 1
+fi
+echo ok
+unset VIPS_MAX_COORD
